import { Request, Response } from 'express';

import * as logModel from './../../models/log.model';
import { Log } from './../../schemas/log.schema';

import {
    trataErro,
    trataRequisicaNome,
    agrupaEFiltraDuplicados,
    desagrupa,
    getId_Token,
    print,
    filtraLogradouro,
    limitaNumeroResultados
} from './../../utils';

import {
    API_CONFIG,
    API_CODES,
    API_MSGS,
    LOG_CODES,
    LOG_MSGS
} from './../../config';

import {
    procuraPessoaDetalhadoCPF,
    procuraPessoaSimplificadoCPF,
    procuraPessoaSimplificadoNome,
    procuraPessoaSimplificadoCNH,
    procuraPessoaSimplificadoTitulo,
    procuraPessoaSimplificadoRG,
    procuraPessoaSimplificadoNomePai,
    procuraPessoaSimplificadoNomeMae,
    procuraPessoaIntegradoCPF,
    getPessoasListaCPFs,
    procuraPessoaSimplificadoTelefone,
    procuraVizinhosPessoaCPF,
    procuraPessoaSimplificadoEndereco,
    procuraPessoaSimplificadoEmail
} from './pessoas.functions';


export let getPessoaDetalhadoCPF = function (req : Request, res : Response){
    let cpf = req.params.cpf;
    cpf = cpf.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if (cpf && cpf.length === 11){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.PESSOA_DETALHADO_CPF}: ${cpf}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraPessoaDetalhadoCPF(cpf)

            .then(pessoas => agrupaEFiltraDuplicados(pessoas))
            .then(pessoas => desagrupa(pessoas))
            .then(pessoas => limitaNumeroResultados(pessoas))

            .then(pessoas => res.status(200).send(pessoas))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)))
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
};

export let getPessoaSimplificadoCPF = function (req : Request, res : Response){
    let cpf = req.params.cpf;
    cpf = cpf.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if (cpf && cpf.length === 11){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.PESSOA_SIMPLIFICADO_CPF}: ${cpf}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraPessoaSimplificadoCPF(cpf)

            .then(pessoas => agrupaEFiltraDuplicados(pessoas))
            .then(pessoas => desagrupa(pessoas))
            .then(pessoas => limitaNumeroResultados(pessoas))

            .then(pessoas => res.status(200).send(pessoas))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
};

export let getPessoaSimplificadoNome = function (req : Request, res : Response){
    let nome = req.params.nome;
    let nomeArray = trataRequisicaNome(nome);
    let processo = req.query.processo;

    if (nomeArray.length < 2){
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_NOME_INSUFICIENTE));
    }else{
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.PESSOA_SIMPLIFICADO_NOME}: ${nome}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraPessoaSimplificadoNome(nome)

            .then(pessoas => agrupaEFiltraDuplicados(pessoas))
            .then(pessoas => desagrupa(pessoas))
            .then(pessoas => limitaNumeroResultados(pessoas))

            .then(pessoas => res.status(200).send(pessoas))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)))
    }
};

export let getPessoaSimplificadoCNH = function (req : Request, res : Response){
    let cnh = req.params.cnh;
    cnh = cnh.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if (cnh && ( cnh.length >= 5 && cnh.length <= 12) ){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.PESSOA_SIMPLIFICADO_CNH}: ${cnh}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraPessoaSimplificadoCNH(cnh)

            .then(pessoas => agrupaEFiltraDuplicados(pessoas))
            .then(pessoas => desagrupa(pessoas))
            .then(pessoas => limitaNumeroResultados(pessoas))

            .then(pessoas => res.status(200).send(pessoas))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getPessoaSimplificadoTitulo = function (req : Request, res : Response){
    let titulo = req.params.titulo;
    titulo = titulo.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if (titulo && ( titulo.length === 12 || titulo.length === 13) ){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.PESSOA_SIMPLIFICADO_TITULO}: ${titulo}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraPessoaSimplificadoTitulo(titulo)

            .then(pessoas => agrupaEFiltraDuplicados(pessoas))
            .then(pessoas => desagrupa(pessoas))
            .then(pessoas => limitaNumeroResultados(pessoas))

            .then(pessoas => res.status(200).send(pessoas))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getPessoaSimplificadoRG = function (req : Request, res : Response){
    let rg = req.params.rg;
    rg = rg.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    // RG tem algum padrão ? :(
    if (rg && ( rg.length >= 3 && rg.length <= 15) ){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.PESSOA_SIMPLIFICADO_RG}: ${rg}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraPessoaSimplificadoRG(rg)

            .then(pessoas => agrupaEFiltraDuplicados(pessoas))
            .then(pessoas => desagrupa(pessoas))
            .then(pessoas => limitaNumeroResultados(pessoas))

            .then(pessoas => res.status(200).send(pessoas))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getPessoaSimplificadoNomePai = function (req : Request, res : Response){
    let nomePai = req.params.nomepai;
    let nomeArray = trataRequisicaNome(nomePai);
    let processo = req.query.processo;

    if (nomeArray.length < 2){
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_NOME_INSUFICIENTE));
    }else{
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.PESSOA_SIMPLIFICADO_NOME_PAI}: ${nomePai}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraPessoaSimplificadoNomePai(nomePai)

            .then(pessoas => agrupaEFiltraDuplicados(pessoas))
            .then(pessoas => desagrupa(pessoas))
            .then(pessoas => limitaNumeroResultados(pessoas))

            .then(pessoas => res.status(200).send(pessoas))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    }
}

export let getPessoaSimplificadoNomeMae = function (req : Request, res : Response){
    const nomeMae = req.params.nomemae;
    let nomeArray = trataRequisicaNome(nomeMae);
    let processo = req.query.processo;

    if (nomeArray.length < 2){
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_NOME_INSUFICIENTE));
    }else{
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.PESSOA_SIMPLIFICADO_NOME_MAE}: ${nomeMae}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraPessoaSimplificadoNomeMae(nomeMae)

            .then(pessoas => agrupaEFiltraDuplicados(pessoas))
            .then(pessoas => desagrupa(pessoas))
            .then(pessoas => limitaNumeroResultados(pessoas))

            .then(pessoas => res.status(200).send(pessoas))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    }
}

export let getPessoaSimplificadoEndereco = function (req : Request, res : Response){
    const endereco = req.params.endereco;
    let enderecoArray = filtraLogradouro(endereco);
    let processo = req.query.processo;

    if (enderecoArray.length < 2){
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_NOME_INSUFICIENTE));
    }else{
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.PESSOA_SIMPLIFICADO_ENDERECO}: ${endereco}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraPessoaSimplificadoEndereco(endereco)

            .then(pessoas => agrupaEFiltraDuplicados(pessoas))
            .then(pessoas => desagrupa(pessoas))
            .then(pessoas => limitaNumeroResultados(pessoas))

            .then(pessoas => res.status(200).send(pessoas))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    }
}

export let getPessoaSimplificadoTelefone = function (req: Request, res: Response){
    let telefone = req.params.telefone;
    telefone = telefone.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if (telefone && ( telefone.length >= 8 && telefone.length <= 15) ){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.PESSOA_SIMPLIFICADO_TELEFONE}: ${telefone}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraPessoaSimplificadoTelefone(telefone)

            .then(pessoas => agrupaEFiltraDuplicados(pessoas))
            .then(pessoas => desagrupa(pessoas))
            .then(pessoas => {
                if (pessoas.dados.length > 0){
                    const cpfsEncontrados = pessoas.dados.map(pessoa => pessoa.cpf);
                    return getPessoasListaCPFs(cpfsEncontrados);
                }else {
                    return pessoas;
                }
            })

            .then(pessoas => res.status(200).send(pessoas))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getPessoaSimplificadoEmail = function (req: Request, res: Response){
    let email = req.params.email;
    let processo = req.query.processo;

    if (email && email.length > 0 ){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.PESSOA_SIMPLIFICADO_EMAIL}: ${email}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraPessoaSimplificadoEmail(email)

            .then(pessoas => agrupaEFiltraDuplicados(pessoas))
            .then(pessoas => desagrupa(pessoas))
            .then(pessoas => limitaNumeroResultados(pessoas))

            .then(pessoas => res.status(200).send(pessoas))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getPessoaIntegradoCPF = function (req: Request, res: Response){
    let cpf = req.params.cpf;
    cpf = cpf.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if (cpf && cpf.length === 11){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.PESSOA_INTEGRADO_CPF}: ${cpf}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraPessoaIntegradoCPF(cpf)

            .then(resultados => agrupaEFiltraDuplicados(resultados, ['endereco', 'telefone', 'veiculo']))

            .then(integrado => res.status(200).send(integrado))
            .catch(error => {res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error))});
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getVizinhosPessoaCPF = function (req: Request, res: Response){
    let cpf = req.params.cpf;
    cpf = cpf.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if (cpf && cpf.length === 11){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.PESSOA_INTEGRADO_CPF}: ${cpf}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraVizinhosPessoaCPF(cpf)

            .then(pessoas => agrupaEFiltraDuplicados(pessoas))
            .then(pessoas => desagrupa(pessoas))
            .then(pessoas => limitaNumeroResultados(pessoas))

            .then(pessoas => res.status(200).send(pessoas))
            .catch(error => {res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error))});
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}