import {
    Request,
    Response
} from 'express';

import * as logModel from '../../models/log.model';
import { Log } from '../../schemas/log.schema';

import {
    trataErro,
    trataRequisicaNome,
    agrupaEFiltraDuplicados,
    filtraNaoEncontrados,
    desagrupa,
    getId_Token,
    print,
    limitaNumeroResultados,
} from '../../utils';

import {
    API_CONFIG,
    API_CODES,
    API_MSGS,
    LOG_CODES,
    LOG_MSGS
} from '../../config';

import {
    procuraPresoDetalhadoCPF,
    procuraPresoSimplificadoCPF,
    procuraPresoSimplificadoVulgo,
    procuraPresoSimplificadoNome,
    procuraPresoSimplificadoNomeMae,
    procuraPresoSimplificadoCNC,
    procuraPresoDetalhadoCNC,
} from './presos.functions';


export let getPrisionalDetalhadoCPF = function (req: Request, res: Response) {
    let cpf = req.params.cpf;
    cpf = cpf.replace(/[^0-9]/g, '');
    let processo = req.query.processo;

    if (cpf && cpf.length === 11) {
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.PRESO_DETALHADO_CPF}: ${cpf}`, user_agent: req.headers['user-agent'], processo: processo }));
        }

        procuraPresoDetalhadoCPF(cpf)

            .then(presos => agrupaEFiltraDuplicados(presos))
            .then(presos => desagrupa(presos))
            .then(presos => limitaNumeroResultados(presos))

            .then(presos => res.status(200).send(presos))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
};

export let getPrisionalSimplificadoCPF = function (req: Request, res: Response) {
    let cpf = req.params.cpf;
    cpf = cpf.replace(/[^0-9]/g, '');
    let processo = req.query.processo;

    if (cpf && cpf.length === 11) {
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.PRESO_SIMPLIFICADO_CPF}: ${cpf}`, user_agent: req.headers['user-agent'], processo: processo }));
        }

        procuraPresoSimplificadoCPF(cpf)

            .then(presos => agrupaEFiltraDuplicados(presos))
            .then(presos => desagrupa(presos))
            .then(presos => limitaNumeroResultados(presos))

            .then(presos => res.status(200).send(presos))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
};

export let getPrisionalSimplificadoCNC = function (req: Request, res: Response) {
    let cnc = req.params.cnc;
    cnc = cnc.replace(/[^0-9]/g, '');
    let processo = req.query.processo;

    if (cnc && (cnc.length === 13 || cnc.length === 14)) {
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.PRESO_SIMPLIFICADO_CNC}: ${cnc}`, user_agent: req.headers['user-agent'], processo: processo }));
        }

        procuraPresoSimplificadoCNC(cnc)

            .then(presos => agrupaEFiltraDuplicados(presos))
            .then(presos => desagrupa(presos))
            .then(presos => limitaNumeroResultados(presos))

            .then(presos => res.status(200).send(presos))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
};

export let getPrisionalDetalhadoCNC = function (req: Request, res: Response) {
    let cnc = req.params.cnc;
    cnc = cnc.replace(/[^0-9]/g, '');
    let processo = req.query.processo;

    if (cnc && (cnc.length === 13 || cnc.length === 14)) {
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.PRESO_DETALHADO_CNC}: ${cnc}`, user_agent: req.headers['user-agent'], processo: processo }));
        }

        procuraPresoDetalhadoCNC(cnc)

            .then(presos => agrupaEFiltraDuplicados(presos))
            .then(presos => desagrupa(presos))
            .then(presos => limitaNumeroResultados(presos))

            .then(presos => res.status(200).send(presos))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
};

export let getPrisionalSimplificadoVulgo = function (req: Request, res: Response) {
    let vulgo = req.params.vulgo;
    let processo = req.query.processo;

    if (vulgo && vulgo.length >= 1) {
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.PRESO_SIMPLIFICADO_VULGO}: ${vulgo}`, user_agent: req.headers['user-agent'], processo: processo }));
        }

        procuraPresoSimplificadoVulgo(vulgo)

            .then(presos => agrupaEFiltraDuplicados(presos))
            .then(presos => desagrupa(presos))
            .then(presos => limitaNumeroResultados(presos))

            .then(presos => res.status(200).send(presos))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
};

export let getPrisionalSimplificadoNome = function (req: Request, res: Response) {
    const nome = req.params.nome;
    let nomeArray = trataRequisicaNome(nome);
    let processo = req.query.processo;

    if (nomeArray.length > 1) {
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.PRESO_SIMPLIFICADO_NOME}: ${nome}`, user_agent: req.headers['user-agent'], processo: processo }));
        }

        procuraPresoSimplificadoNome(nome)

            .then(presos => agrupaEFiltraDuplicados(presos))
            .then(presos => desagrupa(presos))
            .then(presos => limitaNumeroResultados(presos))

            .then(presos => res.status(200).send(presos))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_NOME_INSUFICIENTE));
    }
};

export let getPrisionalSimplificadoNomeMae = function (req: Request, res: Response) {
    const nomeMae = req.params.nomemae;
    let nomeArray = trataRequisicaNome(nomeMae);
    let processo = req.query.processo;

    if (nomeArray.length < 2) {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_NOME_INSUFICIENTE));
    } else {
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.PRESO_SIMPLIFICADO_NOME_MAE}: ${nomeMae}`, user_agent: req.headers['user-agent'], processo: processo }));
        }

        procuraPresoSimplificadoNomeMae(nomeMae)

            .then(presos => agrupaEFiltraDuplicados(presos))
            .then(presos => desagrupa(presos))
            .then(presos => limitaNumeroResultados(presos))

            .then(presos => res.status(200).send(presos))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    }
}