import { Request, Response } from 'express';
import * as _ from 'underscore';

import * as relacionamentoModel from './../../models/relacionamento.model';
import * as logModel from './../../models/log.model';
import { Log } from './../../schemas/log.schema';

import {
    trataErro,
    getId_Token,
    print,
    checaCPF,
    checaCNPJ,
    toTextSearch,
    filtraLogradouro,
} from './../../utils';

import {
    adicionaPessoaAoGrafo,
    adicionaEmpresaAoGrafo,
} from './relacionamentos.utils';

import {
    validaTipoBuscaPessoa,
    validaTipoBuscaTelefone,
    validaTipoBuscaEmpresa,
    filtraNodesPorCiclos,
    validaTipoBuscaEndereco
} from './relacionamentos.functions';

import {
    API_CODES,
    API_MSGS,
    API_CONFIG,
    LOG_CODES,
    LOG_MSGS
} from './../../config';

export let getRelacionamentosPessoaLista = function (req: Request, res: Response){
    let lista = req.params.lista;
    if (lista) {lista = lista.split(',');}

    const cpfsValidos = lista.map(d => checaCPF(d)).filter(d => d !== null);
    const cnpjsValidos = lista.map(d => checaCNPJ(d)).filter(d => d !== null);

    if (cpfsValidos.length !== 0 || cnpjsValidos.length !== 0) {
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_RELACIONAMENTOS, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.RELACIONAMENTO_LISTA}: ${lista.join(',')}`, user_agent: req.headers['user-agent']}));
        }

        const listaCpf  = cpfsValidos;
        const listaCnpj = cnpjsValidos;
        const tipobusca = 'completa';

        Promise.all([
            /**
             * PF
             */
            relacionamentoModel.getNodesParentescosPessoaCPF(listaCpf, tipobusca),
            relacionamentoModel.getEdgesParentescosPessoaCPF(listaCpf, tipobusca),

            relacionamentoModel.getNodesEmpresasResponsavelPessoaCPF(listaCpf, tipobusca),
            relacionamentoModel.getEdgesEmpresasResponsavelPessoaCPF(listaCpf, tipobusca),

            relacionamentoModel.getNodesEmpresasSocioPessoaCPF(listaCpf, tipobusca),
            relacionamentoModel.getEdgesEmpresasSocioPessoaCPF(listaCpf, tipobusca),

            relacionamentoModel.getNodesTelefonesPessoaCPF(listaCpf, tipobusca),
            relacionamentoModel.getEdgesTelefonesPessoaCPF(listaCpf, tipobusca),

            relacionamentoModel.getNodesEnderecosPessoaCPF(listaCpf, tipobusca),
            relacionamentoModel.getEdgesEnderecosPessoaCPF(listaCpf, tipobusca),

            relacionamentoModel.getNodesOrgaosPublicosEstaduaisPessoaCPF(listaCpf, tipobusca),
            relacionamentoModel.getEdgesOrgaosPublicosEstaduaisPessoaCPF(listaCpf, tipobusca),

            relacionamentoModel.getNodesOrgaosPublicosMunicipaisPessoaCPF(listaCpf, tipobusca),
            relacionamentoModel.getEdgesOrgaosPublicosMunicipaisPessoaCPF(listaCpf, tipobusca),

            /**
             * PJ
             */
            relacionamentoModel.getNodesEnderecosEmpresaCNPJ(listaCnpj, tipobusca),
            relacionamentoModel.getEdgesEnderecosEmpresaCNPJ(listaCnpj, tipobusca),

            relacionamentoModel.getNodesResponsavelEmpresaCNPJ(listaCnpj, tipobusca),
            relacionamentoModel.getEdgesResponsavelEmpresaCNPJ(listaCnpj, tipobusca),

            relacionamentoModel.getNodesSociosPFEmpresaCNPJ(listaCnpj, tipobusca),
            relacionamentoModel.getEdgesSociosPFEmpresaCNPJ(listaCnpj, tipobusca),

            relacionamentoModel.getNodesSociosPJEmpresaCNPJ(listaCnpj, tipobusca),
            relacionamentoModel.getEdgesSociosPJEmpresaCNPJ(listaCnpj, tipobusca),
        ])
            .then(res => {
                const graph = {
                    nodes: [],
                    edges: []
                };

                // Filtra os nos repetidos
                graph.nodes = graph.nodes.concat(res[0]).concat(res[2]).concat(res[4]).concat(res[6]).concat(res[8]).concat(res[10]).concat(res[12])
                    .concat(res[14]).concat(res[16]).concat(res[18]).concat(res[20]);
                graph.nodes = _.uniq(graph.nodes, (d) => d.id);

                graph.edges = graph.edges.concat(res[1]).concat(res[3]).concat(res[5]).concat(res[7]).concat(res[9]).concat(res[11]).concat(res[13])
                    .concat(res[15]).concat(res[17]).concat(res[19]).concat(res[21]);
                graph.edges = _.uniq(graph.edges, (d) => (d.id) ? d.id : d.origem + '-' + d.relacao + '-' + d.destino);

                return graph;
            })
            .then(grafo => res.status(200).json({ status: API_CODES.CODE_SUCESSO, dados: grafo }))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)))
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getRelacionamentosPessoaCPF = function (req: Request, res: Response){
    let cpf = req.query.cpf;
    cpf = checaCPF(cpf);

    let tipobusca = req.query.tipobusca;

    if (cpf && validaTipoBuscaPessoa(tipobusca)) {
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_RELACIONAMENTOS, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.RELACIONAMENTO_CPF}: ${cpf}`, user_agent: req.headers['user-agent']}));
        }

        const listaCpf = [cpf];

        Promise.all([
            relacionamentoModel.getNodesParentescosPessoaCPF(listaCpf, tipobusca),
            relacionamentoModel.getEdgesParentescosPessoaCPF(listaCpf, tipobusca),

            relacionamentoModel.getNodesEmpresasResponsavelPessoaCPF(listaCpf, tipobusca),
            relacionamentoModel.getEdgesEmpresasResponsavelPessoaCPF(listaCpf, tipobusca),

            relacionamentoModel.getNodesEmpresasSocioPessoaCPF(listaCpf, tipobusca),
            relacionamentoModel.getEdgesEmpresasSocioPessoaCPF(listaCpf, tipobusca),

            relacionamentoModel.getNodesTelefonesPessoaCPF(listaCpf, tipobusca),
            relacionamentoModel.getEdgesTelefonesPessoaCPF(listaCpf, tipobusca),

            relacionamentoModel.getNodesEnderecosPessoaCPF(listaCpf, tipobusca),
            relacionamentoModel.getEdgesEnderecosPessoaCPF(listaCpf, tipobusca),

            relacionamentoModel.getNodesOrgaosPublicosMunicipaisPessoaCPF(listaCpf, tipobusca),
            relacionamentoModel.getEdgesOrgaosPublicosMunicipaisPessoaCPF(listaCpf, tipobusca),

            relacionamentoModel.getNodesOrgaosPublicosEstaduaisPessoaCPF(listaCpf, tipobusca),
            relacionamentoModel.getEdgesOrgaosPublicosEstaduaisPessoaCPF(listaCpf, tipobusca),
        ])
            .then(res => {
                const graph = {
                    nodes: [],
                    edges: []
                };

                // Filtra os nos repetidos
                graph.nodes = graph.nodes.concat(res[0]).concat(res[2]).concat(res[4]).concat(res[6]).concat(res[8]).concat(res[10]).concat(res[12]);
                graph.nodes = _.uniq(graph.nodes, (d) => d.id);

                graph.edges = graph.edges.concat(res[1]).concat(res[3]).concat(res[5]).concat(res[7]).concat(res[9]).concat(res[11]).concat(res[13]);
                graph.edges = _.uniq(graph.edges, (d) => (d.id) ? d.id : d.origem + '-' + d.relacao + '-' + d.destino);

                return graph;
            })
            .then(grafo => res.status(200).json({ status: API_CODES.CODE_SUCESSO, dados: grafo }))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)))

    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getRelacionamentosTelefoneTelefone = function (req: Request, res: Response){
    let telefone = req.query.telefone;
    if (telefone) { telefone = telefone.split(',');}

    let tipobusca = req.query.tipobusca;

    if (telefone && validaTipoBuscaTelefone(tipobusca)) {
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_RELACIONAMENTOS, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.RELACIONAMENTO_TELEFONE}: ${telefone}`, user_agent: req.headers['user-agent']}));
        }

        const listaTelefone = telefone;

        Promise.all([
            relacionamentoModel.getNodesPessoasTelefone(listaTelefone, tipobusca),
            relacionamentoModel.getEdgesPessoasTelefone(listaTelefone, tipobusca),
        ])
            .then(res => {
                const graph = {
                    nodes: [],
                    edges: []
                };

                // Filtra os nos repetidos
                graph.nodes = graph.nodes.concat(res[0]);
                graph.nodes = _.uniq(graph.nodes, (d) => d.id);

                graph.edges = graph.edges.concat(res[1]);
                graph.edges = _.uniq(graph.edges, (d) => (d.id) ? d.id : d.origem + '-' + d.relacao + '-' + d.destino);

                return graph;
            })
            .then(grafo => res.status(200).json({ status: API_CODES.CODE_SUCESSO, dados: grafo }))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)))

    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getRelacionamentosEmpresaCNPJ = function (req: Request, res: Response){
    let cnpj = req.query.cnpj;
    cnpj = checaCNPJ(cnpj);

    let tipobusca = req.query.tipobusca;

    if (cnpj && validaTipoBuscaEmpresa(tipobusca)){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_RELACIONAMENTOS, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.RELACIONAMENTO_CNPJ}: ${cnpj}`, user_agent: req.headers['user-agent']}));
        }

        const listaCNPJ = [cnpj];

        Promise.all([
            relacionamentoModel.getNodesEnderecosEmpresaCNPJ(listaCNPJ, tipobusca),
            relacionamentoModel.getEdgesEnderecosEmpresaCNPJ(listaCNPJ, tipobusca),

            relacionamentoModel.getNodesResponsavelEmpresaCNPJ(listaCNPJ, tipobusca),
            relacionamentoModel.getEdgesResponsavelEmpresaCNPJ(listaCNPJ, tipobusca),

            relacionamentoModel.getNodesSociosPFEmpresaCNPJ(listaCNPJ, tipobusca),
            relacionamentoModel.getEdgesSociosPFEmpresaCNPJ(listaCNPJ, tipobusca),

            relacionamentoModel.getNodesSociosPJEmpresaCNPJ(listaCNPJ, tipobusca),
            relacionamentoModel.getEdgesSociosPJEmpresaCNPJ(listaCNPJ, tipobusca),
        ])
        .then(res => {
            const graph = {
                nodes: [],
                edges: []
            };

            // Filtra os nos repetidos
            graph.nodes = graph.nodes.concat(res[0]).concat(res[2]).concat(res[4]).concat(res[6]);
            graph.nodes = _.uniq(graph.nodes, (d) => d.id);

            graph.edges = graph.edges.concat(res[1]).concat(res[3]).concat(res[5]).concat(res[7]);
            graph.edges = _.uniq(graph.edges, (d) => (d.id) ? d.id : d.origem + '-' + d.relacao + '-' + d.destino);

            return graph;
        })
        .then(grafo => res.status(200).json({ status: API_CODES.CODE_SUCESSO, dados: grafo }))
        .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)))

    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getRelacionamentosEndereco = function (req: Request, res: Response){
    let endereco = req.query.endereco;
    if (endereco) { endereco = endereco.split(','); }

    let tipobusca = req.query.tipobusca;

    console.log(endereco, tipobusca);

    if (endereco && validaTipoBuscaEndereco(tipobusca)){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_RELACIONAMENTOS, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.RELACIONAMENTO_ENDERECO}: ${endereco}`, user_agent: req.headers['user-agent']}));
        }

        const lista      = endereco[0].split('|');
        const logradouro = toTextSearch(filtraLogradouro(lista[0]));;
        const numero     = lista[1];
        const municipio  = lista[2];

        Promise.all([
            relacionamentoModel.getNodesPessoasEndereco(logradouro, numero, municipio, tipobusca),
            relacionamentoModel.getEdgesPessoasEndereco(logradouro, numero, municipio, tipobusca),

            relacionamentoModel.getNodesEmpresasEndereco(logradouro, numero, municipio, tipobusca),
            relacionamentoModel.getEdgesEmpresasEndereco(logradouro, numero, municipio, tipobusca),
        ])
        .then(res => {
            const graph = {
                nodes: [],
                edges: []
            };

            // Filtra os nos repetidos
            graph.nodes = graph.nodes.concat(res[0]).concat(res[2]);
            graph.nodes = _.uniq(graph.nodes, (d) => d.id);

            graph.edges = graph.edges.concat(res[1]).concat(res[3]);
            graph.edges = _.uniq(graph.edges, (d) => (d.id) ? d.id : d.origem + '-' + d.relacao + '-' + d.destino);

            return graph;
        })
        .then(grafo => res.status(200).json({ status: API_CODES.CODE_SUCESSO, dados: grafo }))
        .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)))

    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}




/**
 * Para a aplicacao de analise de ciclos
 */
export let getRelacionamentosOrgaoCdugestora = function (req: Request, res: Response){
    let cdugestora = req.query.cdugestora;

    let anoinicial = req.query.anoinicial;
    let anofinal   = req.query.anofinal;

    let tipobusca = req.query.tipobusca;

    if (cdugestora){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_RELACIONAMENTOS, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.RELACIONAMENTO_CDUGESTORA}: ${cdugestora}`, user_agent: req.headers['user-agent']}));
        }

        const graph = {
            nodes: [],
            edges: []
        };

        let empresasPagas;
        let servidores;
        let pessoas;

        Promise.all([
            relacionamentoModel.getNodesOrgaoPublicoCdUgestoraMunicipal(cdugestora, tipobusca),

            relacionamentoModel.getNodesEmpresasPagasCdUgestoraMunicipal(cdugestora, anoinicial, anofinal, tipobusca),
            relacionamentoModel.getEdgesEmpresasPagasCdUgestoraMunicipal(cdugestora, anoinicial, anofinal, tipobusca),

            relacionamentoModel.getNodesServidoresCdUgestoraMunicipal(cdugestora, anoinicial, anofinal, tipobusca),
            relacionamentoModel.getEdgesServidoresCdUgestoraMunicipal(cdugestora, anoinicial, anofinal, tipobusca),
        ])
        .then(res => {
            // Filtra os nos repetidos
            graph.nodes = graph.nodes.concat(res[0]).concat(res[1]).concat(res[3]);
            graph.nodes = _.uniq(graph.nodes, (d) => d.id);

            graph.edges = graph.edges.concat(res[2]).concat(res[4]);
            graph.edges = _.uniq(graph.edges, (d) => (d.id) ? d.id : d.origem + '-' + d.relacao + '-' + d.destino);

            empresasPagas = graph.nodes.filter(d => d.entidade === 'empresa').map(d => d.cnpj);
            servidores    = graph.nodes.filter(d => d.entidade === 'pessoa').map(d => d.cpf);

            return graph;
        })
        .then(() => {
            return Promise.all([
                // PF
                relacionamentoModel.getNodesParentescosPessoaCPF(servidores, tipobusca, 'completa'),
                relacionamentoModel.getEdgesParentescosPessoaCPF(servidores, tipobusca, 'completa'),

                relacionamentoModel.getNodesEmpresasResponsavelPessoaCPF(servidores, tipobusca),
                relacionamentoModel.getEdgesEmpresasResponsavelPessoaCPF(servidores, tipobusca),

                relacionamentoModel.getNodesEmpresasSocioPessoaCPF(servidores, tipobusca),
                relacionamentoModel.getEdgesEmpresasSocioPessoaCPF(servidores, tipobusca),

                // PJ
                relacionamentoModel.getNodesResponsavelEmpresaCNPJ(empresasPagas, 'completa'),
                relacionamentoModel.getEdgesResponsavelEmpresaCNPJ(empresasPagas, 'completa'),

                relacionamentoModel.getNodesSociosPFEmpresaCNPJ(empresasPagas, 'completa'),
                relacionamentoModel.getEdgesSociosPFEmpresaCNPJ(empresasPagas, 'completa'),

                relacionamentoModel.getNodesSociosPJEmpresaCNPJ(empresasPagas, 'completa'),
                relacionamentoModel.getEdgesSociosPJEmpresaCNPJ(empresasPagas, 'completa'),

            ])
        })
        .then(res => {
            // Filtra os nos repetidos
            graph.nodes = graph.nodes.concat(res[0]).concat(res[2]).concat(res[4]).concat(res[6]).concat(res[8]).concat(res[10]);
            graph.nodes = _.uniq(graph.nodes, (d) => d.id);

            graph.edges = graph.edges.concat(res[1]).concat(res[3]).concat(res[5]).concat(res[7]).concat(res[9]).concat(res[11]);
            graph.edges = _.uniq(graph.edges, (d) => (d.id) ? d.id : d.origem + '-' + d.relacao + '-' + d.destino);

            pessoas = [].concat(res[6]).concat(res[8]).filter(d => d.entidade === 'pessoa').map(d => d.cpf);
        })
        .then(() => {
            return Promise.all([
                // PF
                relacionamentoModel.getNodesParentescosPessoaCPF(pessoas, tipobusca, 'completa'),
                relacionamentoModel.getEdgesParentescosPessoaCPF(pessoas, tipobusca, 'completa'),
            ])
        })
        .then(res => {
            // Filtra os nos repetidos
            graph.nodes = graph.nodes.concat(res[0]);
            graph.nodes = _.uniq(graph.nodes, (d) => d.id);

            graph.edges = graph.edges.concat(res[1]);
            graph.edges = _.uniq(graph.edges, (d) => (d.id) ? d.id : d.origem + '-' + d.relacao + '-' + d.destino);
        })
        .then(() => filtraNodesPorCiclos(graph, cdugestora))
        // .then(grafoFiltrado => {
        //     graph = grafoFiltrado;
        // })
        .then(grafoFiltrado => res.status(200).json({ status: API_CODES.CODE_SUCESSO, dados: grafoFiltrado }))
        // .then(() => res.status(200).json({ status: API_CODES.CODE_SUCESSO, dados: graph }))
        .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)))

    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}
