import * as _ from 'underscore';
import * as rp from 'request-promise-native';

import { BenchmarkService } from './../../services/benchmark.service';

import * as pessoaModel from './../../models/pessoa.model';
import * as empresaModel from './../../models/empresa.model';
import * as socioModel from './../../models/socio.model';
import { Pessoa, Empresa } from './../../models/schemas';

import {
    trataRequisicaNome,
    toTextSearch,
    filtraNaoEncontrados,
    agrupaEFiltraDuplicados,
    removeDuplicados,
    desagrupa,
    removerAcentos,
    agrupaOrdena,
    print,
    printTempoExecucao,
    flat,
} from './../../utils';

import {
    heuristicaFilhos,
    heuristicaPai,
    heuristicaMae
} from './relacionamentos.utils';

export let getFilhosPessoa = function (pessoa: Pessoa) {

    return procuraFilhos(pessoa)
        .then((resultado: any) => {
            //console.log('resultado procuraFilhos', resultado);

            if (resultado.length) {
                return heuristicaFilhos(resultado, pessoa);
            } else {
                return resultado;
            }
        })
        .then(resultado => {return {filho: resultado}});
}

export let getPaiPessoa = function (pessoa: Pessoa, rankTop = 3) {

    return procuraPai(pessoa)
        .then((resultado: any) => {
            //console.log('resultado procuraPai', resultado);

            pessoa.nomePai = resultado.nome;
            if (resultado.possiveisPais.length) {
                return heuristicaPai(resultado.possiveisPais, pessoa, rankTop);
            } else {
                return resultado.possiveisPais;
            }
        })
        .then(resultado => { return { pai: resultado } });
}

export let getMaePessoa = function (pessoa: Pessoa, rankTop = 3) {

    return procuraMae(pessoa)
        .then((resultado: any) => {
            //console.log('resultado procuraMae', resultado);

            pessoa.nomeMae = resultado.nome;
            if (resultado.possiveisPais.length) {
                return heuristicaMae(resultado.possiveisPais, pessoa, rankTop);
            } else {
                return resultado.possiveisPais;
            }
        })
        .then(resultado => { return { mae: resultado } });
}

export let getConjugePessoa = function (pessoa: Pessoa, rankTop = 3) {

    return getFilhosPessoa(pessoa)
        .then(resultado => resultado.filho)
        .then(filhos => {

            if(filhos){
                return Promise.all(filhos.map(filho => (pessoa.sexo === 'Masculino') ? getMaePessoa(filho) : getPaiPessoa(filho)))
                    .then(r => {
                        if(r.length) {
                            const parente = r[0];
                            return parente[Object.keys(parente)[0]]
                        } else {
                            return r;
                        }
                    })
                    .then(r => flat(r))
                    .then(r => removeDuplicados(r))
            } else {
                return filhos;
            }
        })
        .then(resultado => { return { conjuge: resultado } });
}

export let getEmpresasPessoa = function (pessoa: Pessoa) {
    return procuraEmpresas(pessoa)
            .then(resultado => { return { empresa: resultado } });
}

export let getEmpresasPessoaResponsavel = function (pessoa: Pessoa) {
    return procuraEmpresasResponsavel(pessoa)
            .then(resultado => { return { responsavel: resultado } });
}

export let getSociosPFEmpresa = function (empresa: Empresa) {
    return procuraSociosPF(empresa);
}

export let getSociosPJEmpresa = function (empresa: Empresa) {
    return procuraSociosPJ(empresa);
}

export let getSociosEstrangeiroEmpresa = function (empresa: Empresa) {
    return procuraSociosEstrangeiro(empresa);
}


let procuraFilhos = function (pessoa: Pessoa) {
    const nome           = pessoa.nome;
    const nomeArray      = trataRequisicaNome(nome);
    const nomeTextSearch = toTextSearch(nomeArray);

    const anosDiferencaPaiFilho = 13;
    let dataNascimentoPai       = new Date(pessoa.dataNascimento.getTime());
    dataNascimentoPai.setFullYear(dataNascimentoPai.getFullYear() + anosDiferencaPaiFilho);

    if (pessoa.sexo) {
        if (String(pessoa.sexo).toLowerCase() === "masculino" || String(pessoa.sexo) === "1" || String(pessoa.sexo) === "M") {
            return Promise.all([
                pessoaModel.getPessoaSimplificadoNomePai_TSE_Eleitor(nomeTextSearch, 'fulltext', dataNascimentoPai, '>', true),
                pessoaModel.getPessoaNomePai_Sispesquisa_CNH(nomeTextSearch, 'fulltext', dataNascimentoPai, '>')

                //pessoaModel.getPessoaSimplificadoNomePai_TSE_Eleitor(nome, 'igual', dataNascimentoPai, '>', false),
                //pessoaModel.getPessoaNomePai_Sispesquisa_CNH(nome, 'igual', dataNascimentoPai, '>')
            ])

                .then(pessoas => filtraNaoEncontrados(pessoas))
                .then(pessoas => agrupaEFiltraDuplicados(pessoas))
                .then(pessoas => desagrupa(pessoas))
                .then(pessoas => pessoas.dados)

                .catch(error => console.error('ERRO: procuraFilhos: ' + error))
        } else if (String(pessoa.sexo).toLowerCase() === "feminino" || String(pessoa.sexo) === "2" || String(pessoa.sexo) === "F") {
            return Promise.all([
                pessoaModel.getPessoaSimplificadoNomeMae_TSE_Eleitor(nomeTextSearch, 'fulltext', dataNascimentoPai, '>', true),
                pessoaModel.getPessoaNomeMae_ReceitaNovo_PessoaFisica(nomeTextSearch, 'fulltext', dataNascimentoPai, '>'),

                //pessoaModel.getPessoaNomeMae_ReceitaNovo_PessoaFisica(nome, 'igual', dataNascimentoPai, '>'),
                //pessoaModel.getPessoaSimplificadoNomeMae_TSE_Eleitor(nome, 'igual', dataNascimentoPai, '>', false)
            ])

                .then(pessoas => filtraNaoEncontrados(pessoas))
                .then(pessoas => agrupaEFiltraDuplicados(pessoas))
                .then(pessoas => desagrupa(pessoas))
                .then(pessoas => pessoas.dados)

                .catch(error => console.error('ERRO: procuraFilhos: ' + error))
        } else {
            console.error('Erro: sexo inválido.');
        }
    } else {
        console.error('Erro: sexo inexistente. Testar ambos.');
        return Promise.all([
            pessoaModel.getPessoaSimplificadoNomePai_TSE_Eleitor(nomeTextSearch, 'fulltext', dataNascimentoPai, '>'),
            pessoaModel.getPessoaNomePai_Sispesquisa_CNH(nomeTextSearch, 'fulltext', dataNascimentoPai, '>'),

            pessoaModel.getPessoaNomeMae_ReceitaNovo_PessoaFisica(nomeTextSearch, 'fulltext', dataNascimentoPai, '>'),
            pessoaModel.getPessoaSimplificadoNomeMae_TSE_Eleitor(nomeTextSearch, 'fulltext', dataNascimentoPai, '>')
        ])

            .then(pessoas => filtraNaoEncontrados(pessoas))
            .then(pessoas => agrupaEFiltraDuplicados(pessoas))
            .then(pessoas => desagrupa(pessoas))

            .catch(error => console.error('ERRO: procuraFilhos: ' + error))
    }
}

let procuraPai = function (pessoa: Pessoa) {

    // Se já tem um nome de pai disponível
    if (pessoa.nomePai) {
        return getDadosPaiMae(pessoa.nomePai, pessoa.dataNascimento);
    } else {
        // Procura nos dados do filho se existe algum nome de pai
        return Promise.all([
            pessoaModel.getPessoaSimplificadoCPF_Sispesquisa_CNH(pessoa.cpf),
            pessoaModel.getPessoaSimplificadoCPF_TSE_Eleitor(pessoa.cpf)
        ])

            .then(pessoas => filtraNaoEncontrados(pessoas))
            .then(pessoas => agrupaOrdena(pessoas))
            .then(pessoas => desagrupa(pessoas))
            .then(pessoas => pessoas.dados)

            .then(resultados => resultados.filter(r => r.nomePai)) // Pega os resultados que tenham nomePai válido
            .then(resultados => {
                //console.log('resultado procuraPai:\n', JSON.stringify(resultados, null, 2));
                if (resultados.length === 0) { return { possiveisPais: [], nome: '' }; }
                else { return getDadosPaiMae(resultados[0].nomePai, pessoa.dataNascimento); }
            })
            .catch(error => console.error('ERROR: procuraPai - ', error));
    }
}

let procuraMae = function (pessoa: Pessoa) {
    //console.log('procuraMae', pessoa);

    // Se já tem um nome de mãe disponível
    if (pessoa.nomeMae) {
        return getDadosPaiMae(pessoa.nomeMae, pessoa.dataNascimento);
    } else {
        // Procura nos dados do filho se existe algum nome de mãe
        return Promise.all([
            pessoaModel.getPessoaSimplificadoCPF_ReceitaNovo_PessoaFisica(pessoa.cpf),
            pessoaModel.getPessoaSimplificadoCPF_TSE_Eleitor(pessoa.cpf)
        ])

            .then(pessoas => filtraNaoEncontrados(pessoas))
            .then(pessoas => agrupaEFiltraDuplicados(pessoas))
            .then(pessoas => desagrupa(pessoas))
            .then(pessoas => pessoas.dados)

            .then(resultados => resultados.filter(r => r.nomeMae)) // Pega os resultados que tenham nomeMae válido
            .then(resultados => {
                //console.log('resultado procuraMae:\n', JSON.stringify(resultados, null, 2));
                if (resultados.length === 0) { return { possiveisPais: [], nome: '' }; }
                else { return getDadosPaiMae(resultados[0].nomeMae, pessoa.dataNascimento); }
            })
            .catch(error => console.log('ERROR: procuraMae - ', error));
    }
}

let getDadosPaiMae = function (nome: string, dtNascimentoFilho = undefined) {
    //console.log('Procurando dados do pai ou mae ', nome);

    const anosDiferencaPaiFilho = 13;
    let dataNascimentoFilho     = new Date(dtNascimentoFilho.getTime());

    dataNascimentoFilho.setFullYear(dataNascimentoFilho.getFullYear() - anosDiferencaPaiFilho);

    let nomeFormatado = toTextSearch(trataRequisicaNome(nome));

    return Promise.all([

        pessoaModel.getPessoaSimplificadoNome_ReceitaNovo_PessoaFisica(nomeFormatado, 'fulltext', dataNascimentoFilho, '<', false)
        //pessoaModel.getPessoaSimplificadoNome_ReceitaNovo_PessoaFisica(nome, 'igual', dataNascimentoFilho, '<', false)
    ])
        .then(pessoas => filtraNaoEncontrados(pessoas))

        // Verifica se encontrou algum resultado, se não achou nada vai buscar na base Receita_full
        //.then(pessoas => (pessoas.length === 0) ? pessoaModel.getPessoaSimplificadoNome_ReceitaFull_PF(nomeFormatado, copiaDataNascimento) : pessoas)
        //.then(pessoas => filtraNaoEncontrados(pessoas))

        .then(pessoas => agrupaEFiltraDuplicados(pessoas))
        .then(pessoas => desagrupa(pessoas))
        .then(pessoas => pessoas.dados)

        .then(possiveisPais => {
            //console.log('getDadosPaiMae: ', JSON.stringify(possiveisPais, null, 2)) ;
            return (possiveisPais.length) ? { possiveisPais, nome } : { possiveisPais: [{ nome }], nome };
        });
}

let procuraEmpresas = function (pessoa: Pessoa) {
    //console.log('procuraEmpresas: ', pessoa);

    return Promise.all([
        empresaModel.getEmpresaSimplificadoSocioPFCPF_ReceitaNovo(pessoa.cpf),
        //empresaModel.getEmpresaSimplificadoSocioPFCPF_ReceitaFull(pessoa.cpf),
    ])

        .then(empresas => filtraNaoEncontrados(empresas))
        .then(empresas => agrupaEFiltraDuplicados(empresas))
        .then(empresas => desagrupa(empresas))
        .then(empresas => empresas.dados)

        .catch(error => console.error('ERROR: procuraEmpresas - ', error));
}

let procuraEmpresasResponsavel = function (pessoa: Pessoa) {
    //console.log('procuraEmpresas: ', pessoa);

    return Promise.all([
        empresaModel.getEmpresaDetalhadoCPFResponsavel_ReceitaNovo_PessoaJuridica(pessoa.cpf),
    ])

        .then(empresas => filtraNaoEncontrados(empresas))
        .then(empresas => agrupaEFiltraDuplicados(empresas))
        .then(empresas => desagrupa(empresas))
        .then(empresas => empresas.dados)

        .catch(error => console.error('ERROR: procuraEmpresasResponsavel - ', error));
}

let procuraSociosPF = function (empresa: Empresa) {

    return Promise.all([
        //socioModel.getSocioPFEmpresaSimplificadoCNPJ_ReceitaFull(empresa.cnpj),
        socioModel.getSocioPFEmpresaSimplificadoCNPJ_ReceitaNovo(empresa.cnpj)
    ])

        .then(pessoas => filtraNaoEncontrados(pessoas))
        .then(pessoas => agrupaEFiltraDuplicados(pessoas))
        .then(pessoas => desagrupa(pessoas))
        .then(pessoas => pessoas.dados)

        // .then(possiveisSocios => joinResultados(possiveisSocios))
        .catch(error => console.error('ERROR: procuraSociosPF - ', error))
}

let procuraSociosPJ = function (empresa: Empresa) {

    return Promise.all([
        //socioModel.getSocioPJEmpresaSimplificadoCNPJ_ReceitaFull(empresa.cnpj),
        socioModel.getSocioPJEmpresaSimplificadoCNPJ_ReceitaNovo(empresa.cnpj)
    ])

        .then(pessoas => filtraNaoEncontrados(pessoas))
        .then(pessoas => agrupaEFiltraDuplicados(pessoas))
        .then(pessoas => desagrupa(pessoas))
        .then(pessoas => pessoas.dados)

        // .then(possiveisSocios => joinResultados(possiveisSocios))
        .catch(error => console.log('ERROR: procuraSociosPJ - ', error))
}

let procuraSociosEstrangeiro = function (empresa: Empresa) {

    return Promise.all([
        socioModel.getSocioEstrangeiroEmpresaSimplificadoCNPJ_ReceitaNovo(empresa.cnpj),
        //socioModel.getSocioPJEmpresaSimplificadoCNPJ_ReceitaNovo(empresa.cnpj)
    ])

        .then(pessoas => filtraNaoEncontrados(pessoas))
        .then(pessoas => agrupaEFiltraDuplicados(pessoas))
        .then(pessoas => desagrupa(pessoas))
        .then(pessoas => pessoas.dados)

        // .then(possiveisSocios => joinResultados(possiveisSocios))
        .catch(error => console.log('ERROR: procuraSociosEstrangeiro - ', error))
}


/**
 * Valida os tipos possiveis de busca para a entidade Pessoa.
 * @param tipoBusca 
 */
export let validaTipoBuscaPessoa = function(tipoBusca: string) {
    if(!tipoBusca) { return false; }

    if(tipoBusca === 'parentesco' ||
        tipoBusca === 'telefones' ||
        tipoBusca === 'empresasresponsavel' ||
        tipoBusca === 'empresassocio' ||
        tipoBusca === 'orgaospublicos' ||
        tipoBusca === 'enderecos' ||
        tipoBusca === 'completa') {
        return true;
    } else {
        return false;
    }
}

/**
 * Valida os tipos possiveis de busca para a entidade Telefone.
 * @param tipoBusca 
 */
export let validaTipoBuscaTelefone = function(tipoBusca: string) {
    if(!tipoBusca) { return false; }

    if(tipoBusca === 'proprietarios' ||
        tipoBusca === 'completa') {
        return true;
    } else {
        return false;
    }
}

/**
 * Valida os tipos possiveis de busca para a entidade Empresa.
 * @param tipoBusca 
 */
export let validaTipoBuscaEmpresa = function(tipoBusca: string) {
    if(!tipoBusca) { return false; }

    if(tipoBusca === 'responsavel' ||
        tipoBusca === 'sociospf' ||
        tipoBusca === 'sociospj' ||
        tipoBusca === 'socios' ||
        tipoBusca === 'filiais' ||
        tipoBusca === 'telefones' ||
        tipoBusca === 'enderecos' ||
        tipoBusca === 'completa') {
        return true;
    } else {
        return false;
    }
}

/**
 * Valida os tipos possiveis de busca para a entidade Endereco.
 * @param tipoBusca 
 */
export let validaTipoBuscaEndereco = function(tipoBusca: string) {
    if(!tipoBusca) { return false; }

    if(tipoBusca === 'pessoas' ||
        tipoBusca === 'empresas' ||
        tipoBusca === 'completa') {
        return true;
    } else {
        return false;
    }
}


export let filtraNodesPorCiclos = function(grafo, cdugestora) {

    const ANALISE_CICLOS_SERVER = `http://localhost:5000`

    let options = {
        method: 'POST',
        uri: `${ANALISE_CICLOS_SERVER}/analiseciclos`,
        body: {
            edges: grafo.edges,
            cdugestora: cdugestora
        },
        timeout: 120000,
        json: true
    };

    const tempoInicial = new BenchmarkService();

    return rp(options)
        .then(data => {
            printTempoExecucao(tempoInicial, 'filtraNodesPorCiclos');

            const nodesAlvo = data.nodes;

            const nodesFiltrados = grafo.nodes.map(node => {
                return (nodesAlvo.indexOf(node.id) !== -1) ? node : null
            }).filter(n => n !== null);

            const edgesFiltrados = grafo.edges.map(edge => {
                return (nodesAlvo.indexOf(edge.origem) !== -1 || nodesAlvo.indexOf(edge.destino)) ? edge : null
            }).filter(e => e !== null);

            return {
                nodes: nodesFiltrados,
                edges: edgesFiltrados
            }
        })
        .catch(error => {console.error(error)});
}