import { Request, Response } from 'express';

import * as telefoneModel from './../../models/telefone.model';
import * as logModel from './../../models/log.model';
import { Log } from './../../schemas/log.schema';

import {
    trataErro,
    agrupaEFiltraDuplicados,
    filtraNaoEncontrados,
    desagrupa,
    trataRequisicaNome,
    getId_Token,
    print,
    limitaNumeroResultados
} from './../../utils';

import {
    API_CODES,
    API_MSGS,
    API_CONFIG,
    LOG_CODES,
    LOG_MSGS
} from './../../config';

import {
    procuraTelefoneSimplificadoCPF,
    procuraTelefoneSimplificadoCNPJ,
    procuraTelefoneSimplificadoTelefone,
    procuraTelefoneSimplificadoNome,
    procuraTelefoneSimplificadoRazaoSocial,
    procuraTelefoneBuscaProfundaTelefone,

} from './telefones.functions';

export let getTelefoneSimplificadoCPF = function (req: Request, res: Response) {
    let cpf = req.params.cpf;
    cpf = cpf.replace(/[^0-9]/g, '');
    let processo = req.query.processo;

    if (cpf && cpf.length === 11) {
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.TELEFONE_SIMPLIFICADO_CPF}: ${cpf}`, user_agent: req.headers['user-agent'], processo: processo }));
        }

        procuraTelefoneSimplificadoCPF(cpf)

            .then(telefones => agrupaEFiltraDuplicados(telefones))
            .then(telefones => desagrupa(telefones))
            .then(telefones => limitaNumeroResultados(telefones))

            .then(telefones => res.status(200).send(telefones))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
};

export let getTelefoneSimplificadoCNPJ = function (req : Request, res : Response){
	let cnpj = req.params.cnpj;
    cnpj = cnpj.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if (cnpj && cnpj.length === 14){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.TELEFONE_SIMPLIFICADO_CNPJ}: ${cnpj}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraTelefoneSimplificadoCNPJ(cnpj)

            .then(telefones => agrupaEFiltraDuplicados(telefones))
            .then(telefones => desagrupa(telefones))
            .then(telefones => limitaNumeroResultados(telefones))

            .then(telefones => res.status(200).send(telefones))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
};

export let getTelefoneSimplificadoTelefone = function (req : Request, res : Response){
	let telefone = req.params.telefone;
    telefone = telefone.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if (telefone && telefone.length >= 4){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.TELEFONE_SIMPLIFICADO_TELEFONE}: ${telefone}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraTelefoneSimplificadoTelefone(telefone)

            .then(telefones => agrupaEFiltraDuplicados(telefones))
            .then(telefones => desagrupa(telefones))
            .then(telefones => limitaNumeroResultados(telefones))

            .then(telefones => res.status(200).send(telefones))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
};

export let getTelefoneBuscaProfundaTelefone = function (req : Request, res : Response){
	let telefone = req.params.telefone;
    telefone = telefone.replace(/[^0-9]/g,'');
    let processo = req.query.processo;

    if (telefone && telefone.length >= 4){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.TELEFONE_BUSCA_PROFUNDA_TELEFONE}: ${telefone}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraTelefoneBuscaProfundaTelefone(telefone)

            .then(telefones => agrupaEFiltraDuplicados(telefones))
            .then(telefones => desagrupa(telefones))
            .then(telefones => limitaNumeroResultados(telefones))

            .then(telefones => res.status(200).send(telefones))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
};

export let getTelefoneSimplificadoNome = function (req : Request, res : Response){
    let nome = req.params.nome;
    let nomeArray = trataRequisicaNome(nome);
    let processo = req.query.processo;

    if (nomeArray.length < 2){
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_NOME_INSUFICIENTE));
    }else{
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.TELEFONE_SIMPLIFICADO_NOME}: ${nome}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraTelefoneSimplificadoNome(nome)

            .then(telefones => agrupaEFiltraDuplicados(telefones))
            .then(telefones => desagrupa(telefones))
            .then(telefones => limitaNumeroResultados(telefones))

            .then(telefones => res.status(200).send(telefones))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)))
    }
};

export let getTelefoneSimplificadoRazaoSocial = function (req : Request, res : Response){
    let razaoSocial = req.params.razaosocial;
    let razaoSocialArray = trataRequisicaNome(razaoSocial);
    let processo = req.query.processo;

    if (razaoSocialArray.length < 2){
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_NOME_INSUFICIENTE));
    }else{
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_PESQUISA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.TELEFONE_SIMPLIFICADO_RAZAO_SOCIAL}: ${razaoSocial}`, user_agent: req.headers['user-agent'], processo: processo}));
        }

        procuraTelefoneSimplificadoRazaoSocial(razaoSocial)

            .then(telefones => agrupaEFiltraDuplicados(telefones))
            .then(telefones => desagrupa(telefones))
            .then(telefones => limitaNumeroResultados(telefones))

            .then(telefones => res.status(200).send(telefones))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)))
    }
};

export let insertTelefoneSimplificado = function (req : Request, res : Response){
    let form = req.body;
    let cpf = form.cpf;
    let telefone = form.telefone;
    cpf = cpf.replace(/[^0-9]/g,'');
    telefone = telefone.replace(/[^0-9]/g,'');

    if (cpf && cpf.length === 11){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_INSERCAO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - TELEFONE: ${cpf}`, user_agent: req.headers['user-agent'], processo: null}));
        }
        const fonte = `OP${(new Date()).getFullYear()}`;

        telefoneModel.insertTelefone_Sispesquisa_Telefones({cpf: cpf, telefone: telefone, fonte: fonte})

            .then(resultado => res.status(200).send(resultado))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
};
