import * as jwt from 'jsonwebtoken';
import * as atob from 'atob';

import {
    Request,
    Response
} from 'express';

import { auth, validaToken } from '../services/auth/auth.service';

import { CadastroUsuario, Usuario } from '../models/schemas';
import * as usuarioModel from '../models/usuario.model';
import * as preusuarioModel from '../models/preusuario.model';
import * as logModel from '../models/log.model';
import { Log } from '../schemas/log.schema';

import {
    trataErro,
    desagrupa,
    print,
    filtraNaoEncontrados,
    validaTrocaSenha,
    validaRecaptcha,
    getId_Token,
    agrupaEFiltraDuplicados
} from '../utils';

import {
    API_CODES,
    API_MSGS,
    API_CONFIG,
    LOG_CODES,
    LOG_MSGS
} from '../config';

export let verificaLogin = async function (req : Request, res : Response){
    const formData = req.body;

     let usuarioRequisicao = new Usuario(formData);
     let usuarioEncontrado;

     if (formData.recaptcha || (API_CONFIG.CFG_ENV === 'test' && req.query.test === 's')) {
        const recaptchaResponse = formData.recaptcha;
        try {
            let valido = await validaRecaptcha(recaptchaResponse);
            if (valido || (API_CONFIG.CFG_ENV === 'test' && req.query.test === 's')) {
                usuarioModel.checaLogin(usuarioRequisicao.login)
                    .then((resultado:any) => {
                        if (resultado.status === API_CODES.CODE_SUCESSO){
                            usuarioEncontrado = resultado.dados;
                            return usuarioEncontrado;
                        } else {
                            throw resultado;
                        }
                    })
                    .then(usuario => auth(usuarioRequisicao, usuario.ad))
                    .then((resultado:any) => {
                        if (resultado.status !== API_CODES.CODE_SUCESSO){
                            throw resultado;
                        } else {
                            return resultado
                        }
                    })
                    .then(() => {
                        return usuarioEncontrado.toToken();
                    })
                    .then(user => {
                        if (API_CONFIG.CFG_ENV == 'production') {
                            logModel.salvaLog(new Log({ ip: req.headers['x-real-ip'], usuario: usuarioEncontrado.login, tipo: LOG_CODES.TIPO_LOGIN, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_CODES.CODE_SUCESSO}: ${API_MSGS.MSG_LOGIN_SUCESSO}`, user_agent: req.headers['user-agent'] }));
                        }

                        let token = jwt.sign({ user: user, ts: Date.now() }, API_CONFIG.SIAP_SENHA_TOKEN, { expiresIn: API_CONFIG.TEMPO_TOKEN });
                        return res.status(200).json({
                            status: API_CODES.CODE_SUCESSO,
                            msg: API_MSGS.MSG_LOGIN_SUCESSO,
                            dados: { token: token }
                        });
                    })
                    .catch(error => {
                        if (error.status && error.status === API_CODES.CODE_FALHA_LOGIN){
                            if (API_CONFIG.CFG_ENV == 'production') {
                                logModel.salvaLog(new Log({ip: req.headers['x-real-ip'], usuario: usuarioRequisicao.login, tipo: LOG_CODES.TIPO_TENTATIVA_LOGIN, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_CODES.CODE_FALHA_LOGIN}: ${error.msg}`, user_agent: req.headers['user-agent']}));
                            }

                            res.status(401).send(trataErro(error.status, error.msg));
                        } else {
                            if (API_CONFIG.CFG_ENV == 'production') {
                                logModel.salvaLog(new Log({ip: req.headers['x-real-ip'], usuario: usuarioRequisicao.login, tipo: LOG_CODES.TIPO_TENTATIVA_LOGIN, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_CODES.CODE_ERRO_500}: ${null}`, user_agent: req.headers['user-agent']}));
                            }

                            res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error));
                        }
                    });

            } else {
                if (API_CONFIG.CFG_ENV == 'production') {
                    logModel.salvaLog(new Log({ ip: req.headers['x-real-ip'], usuario: usuarioRequisicao.login, tipo: LOG_CODES.TIPO_TENTATIVA_LOGIN, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_CODES.CODE_RECAPTCHA_INVALIDO}: 'Não foi possível validar o recaptcha.'`, user_agent: req.headers['user-agent'] }));
                }

                res.status(401).send(trataErro(API_CODES.CODE_RECAPTCHA_INVALIDO, 'Não foi possível validar o recaptcha.'))
            }
        } catch (errValidaRecaptcha){
            if (API_CONFIG.CFG_ENV == 'production') {
                logModel.salvaLog(new Log({ ip: req.headers['x-real-ip'], usuario: usuarioRequisicao.login, tipo: LOG_CODES.TIPO_TENTATIVA_LOGIN, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_CODES.CODE_RECAPTCHA_INVALIDO}: 'Erro na validação do recaptcha.'`, user_agent: req.headers['user-agent'] }));
            }

            res.status(401).send(trataErro(API_CODES.CODE_RECAPTCHA_INVALIDO, 'Erro na validação do recaptcha.'))
        }
     } else {
         if (API_CONFIG.CFG_ENV == 'production') {
             logModel.salvaLog(new Log({ ip: req.headers['x-real-ip'], usuario: usuarioRequisicao.login, tipo: LOG_CODES.TIPO_TENTATIVA_LOGIN, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_CODES.CODE_RECAPTCHA_NAO_ENCONTRADO}: 'Valor para validação do recaptcha não encontrado.'`, user_agent: req.headers['user-agent'] }));
         }

        res.status(401).send(trataErro(API_CODES.CODE_RECAPTCHA_NAO_ENCONTRADO, 'Valor para validação do recaptcha não encontrado.'))
    }
}

export let verificaLoginExterno = async function (req: Request, res: Response){
    const payload_cod = req.body.payload;

    let payload;
    try {
        payload = JSON.parse(atob(payload_cod));
    } catch (err) {
        payload = null;
    }

    if (payload && (payload.token && payload.tiposervidor === 'M') ){
        let adInfo;
        try {
            adInfo =  await validaToken(payload.token);

            let resultado: any = await usuarioModel.checaLogin(adInfo.login) ;
            let usuarioValido = (resultado.status === API_CODES.CODE_SUCESSO) ? true: false;
            if (usuarioValido) {
                if (API_CONFIG.CFG_ENV == 'production') {
                    logModel.salvaLog(new Log({ip: req.headers['x-real-ip'], usuario: adInfo.login, tipo: LOG_CODES.TIPO_LOGIN_EXTERNO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_CODES.CODE_SUCESSO}: ${API_MSGS.MSG_LOGIN_SUCESSO}`, user_agent: req.headers['user-agent']}));
                }

                const user = await resultado.dados.toToken();
                let token = jwt.sign({ user: user, ts: Date.now()}, API_CONFIG.SIAP_SENHA_TOKEN, {expiresIn: API_CONFIG.TEMPO_TOKEN});
                return res.status(200).json({
                    status: API_CODES.CODE_SUCESSO,
                    msg: API_MSGS.MSG_LOGIN_SUCESSO,
                    dados: {token: token}
                });
            } else if(resultado.status === API_CODES.CODE_FALHA_LOGIN && resultado.msg === API_MSGS.MSG_LOGIN_USUARIO_INATIVO) {
                usuarioModel.updateStatusUsuario_Sispesquisa_Usuario(adInfo.login, true)
                    .then(() => {
                        if (API_CONFIG.CFG_ENV == 'production') {
                            logModel.salvaLog(new Log({ip: req.headers['x-real-ip'], usuario: adInfo.login, tipo: LOG_CODES.TIPO_LOGIN_EXTERNO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_CODES.CODE_SUCESSO}: Usuário ativado.`, user_agent: req.headers['user-agent']}));
                            logModel.salvaLog(new Log({ip: req.headers['x-real-ip'], usuario: adInfo.login, tipo: LOG_CODES.TIPO_LOGIN_EXTERNO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_CODES.CODE_SUCESSO}: ${API_MSGS.MSG_LOGIN_SUCESSO}`, user_agent: req.headers['user-agent']}));
                        }
                        return  usuarioModel.checaLogin(adInfo.login);
                    })
                    .then((resultado: any) => {
                        let usuarioValido = (resultado.status === API_CODES.CODE_SUCESSO) ? true: false;
                        if (usuarioValido) {
                            if (API_CONFIG.CFG_ENV == 'production') {
                                logModel.salvaLog(new Log({ip: req.headers['x-real-ip'], usuario: adInfo.login, tipo: LOG_CODES.TIPO_LOGIN_EXTERNO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_CODES.CODE_SUCESSO}: ${API_MSGS.MSG_LOGIN_SUCESSO}`, user_agent: req.headers['user-agent']}));
                            }
                            return resultado.dados.toToken();
                        }
                    })
                    .then(user => {
                        let token = jwt.sign({ user: user, ts: Date.now() }, API_CONFIG.SIAP_SENHA_TOKEN, { expiresIn: API_CONFIG.TEMPO_TOKEN });
                        return res.status(200).json({
                            status: API_CODES.CODE_SUCESSO,
                            msg: API_MSGS.MSG_LOGIN_SUCESSO,
                            dados: { token: token }
                        });
                    })
                    .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)))
            } else {
                // cadastro
                let cadastro_usuario = {
                    nome: payload.nome,
                    nacionalidade: payload.nacionalidade,
                    cpf: payload.cpf,
                    identidade: payload.identidade,
                    orgEmissor: payload.org_emissor,
                    ufOrgEmissor: payload.uf_org_emisso,
                    matricula: payload.matricula,
                    titularidade: payload.titularidade,
                    promotoria: payload.promotoria,
                    email: payload.email,
                    telefone: payload.telefone,
                    dataCadastro: (new Date()).toISOString()
                }
                let resultado:any = await preusuarioModel.insertPreUsuarios_Sispesquisa_CadastroUsuario(cadastro_usuario, 'S');
                let cadastroUsuarioCriado = (resultado.status === API_CODES.CODE_SUCESSO) ? true : false;
                if(cadastroUsuarioCriado) {
                    if (API_CONFIG.CFG_ENV == 'production') {
                        logModel.salvaLog(new Log({ip: req.headers['x-real-ip'], usuario: adInfo.login, tipo: LOG_CODES.TIPO_CREATE_CADASTRO_USUARIO_EXTERNO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_CODES.CODE_SUCESSO}: ${null}`, user_agent: req.headers['user-agent']}));
                    }

                    let usuario = new Usuario({
                        nome: payload.nome,
                        login: adInfo.login,
                        senha: null,
                        email: payload.email,
                        ativo: 'S',
                        administrador: 'N',
                        AD: 'MP',
                        perfil: 'publico',
                        processo: 'S',
                        proximo_logon: 'N',
                        membro: 'S',
                        operacoes: 0,
                        id_cadastrador: 99
                    })
                    let resultado = await usuarioModel.insertUsuario_Sispesquisa_Usuario(usuario);
                    let usuarioCriado = (resultado.status === API_CODES.CODE_SUCESSO) ? true : false;
                    if(usuarioCriado) {
                        if (API_CONFIG.CFG_ENV == 'production') {
                            logModel.salvaLog(new Log({ip: req.headers['x-real-ip'], usuario: usuario.login, tipo: LOG_CODES.TIPO_CREATE_USUARIO_EXTERNO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_CODES.CODE_SUCESSO}: ${null}`, user_agent: req.headers['user-agent']}));
                            logModel.salvaLog(new Log({ip: req.headers['x-real-ip'], usuario: usuario.login, tipo: LOG_CODES.TIPO_LOGIN_EXTERNO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_CODES.CODE_SUCESSO}: ${API_MSGS.MSG_LOGIN_SUCESSO}`, user_agent: req.headers['user-agent']}));
                        }

                        const user = await usuario.toToken();
                        let token = jwt.sign({ user: user, ts: Date.now()}, API_CONFIG.SIAP_SENHA_TOKEN, {expiresIn: API_CONFIG.TEMPO_TOKEN});
                        return res.status(200).json({
                            status: API_CODES.CODE_SUCESSO,
                            msg: API_MSGS.MSG_LOGIN_SUCESSO,
                            dados: {token: token}
                        });
                    } else {
                        if (API_CONFIG.CFG_ENV == 'production') {
                            logModel.salvaLog(new Log({ip: req.headers['x-real-ip'], usuario: usuario.login, tipo: LOG_CODES.TIPO_TENTATIVA_LOGIN_EXTERNO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_CODES.CODE_ERRO_500}: ${API_MSGS.MSG_ACESSO_EXTERNO_FALHA_CADASTRO_AUTOMATICO}`, user_agent: req.headers['user-agent']}));
                        }

                        res.status(401).send(trataErro(API_CODES.CODE_ERRO_500, API_MSGS.MSG_ACESSO_EXTERNO_FALHA_CADASTRO_AUTOMATICO));
                    }
                } else {
                    if (API_CONFIG.CFG_ENV == 'production') {
                        logModel.salvaLog(new Log({ip: req.headers['x-real-ip'], usuario: adInfo.login, tipo: LOG_CODES.TIPO_TENTATIVA_LOGIN_EXTERNO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_CODES.CODE_ERRO_500}: ${API_MSGS.MSG_ACESSO_EXTERNO_FALHA_CADASTRO_AUTOMATICO}`, user_agent: req.headers['user-agent']}));
                    }

                    res.status(401).send(trataErro(API_CODES.CODE_ERRO_500, API_MSGS.MSG_ACESSO_EXTERNO_FALHA_CADASTRO_AUTOMATICO));
                }
            }
        } catch (err) {
            let usuario;
            try{
                usuario = payload.login;
            } catch (err){
                usuario = null;
            }

            // token invalido
            if (API_CONFIG.CFG_ENV == 'production') {
                logModel.salvaLog(new Log({ip: req.headers['x-real-ip'], usuario: usuario, tipo: LOG_CODES.TIPO_TENTATIVA_LOGIN_EXTERNO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_CODES.CODE_PAYLOAD_INVALIDO}: ${API_MSGS.MSG_ACESSO_EXTERNO_TOKEN_INVALIDO}`, user_agent: req.headers['user-agent']}));
            }
            res.status(401).send(trataErro(API_CODES.CODE_PAYLOAD_INVALIDO, API_MSGS.MSG_ACESSO_EXTERNO_TOKEN_INVALIDO));
        }
    } else {
        let usuario;
        try{
            usuario = payload.login;
        } catch (err){
            usuario = null;
        }

        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLog(new Log({ip: req.headers['x-real-ip'], usuario: usuario, tipo: LOG_CODES.TIPO_TENTATIVA_LOGIN_EXTERNO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_CODES.CODE_PAYLOAD_INVALIDO}: ${API_MSGS.MSG_PAYLOAD_INVALIDO}`, user_agent: req.headers['user-agent']}));
        }
        res.status(401).send(trataErro(API_CODES.CODE_PAYLOAD_INVALIDO, API_MSGS.MSG_PAYLOAD_INVALIDO));
    }
}

 export let listaUsuarios = function (req : Request, res : Response){
    if (API_CONFIG.CFG_ENV == 'production') {
        logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_SISTEMA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.USUARIO_LISTA_USUARIOS}`, user_agent: req.headers['user-agent']}));
    }

    Promise.all([usuarioModel.getUsuarios()])
        .then(usuarios => filtraNaoEncontrados(usuarios))
        .then(usuarios => agrupaEFiltraDuplicados(usuarios))
        .then(usuarios => desagrupa(usuarios))

        .then(usuarios => res.status(200).send(usuarios))
        .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
}

export let getUsuarioMe = function (req : Request, res : Response){
    let id = getId_Token(req.headers['authorization']);

    if (id){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: id, tipo: LOG_CODES.TIPO_SISTEMA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.USUARIO_GET_USUARIO}: ${id}`, user_agent: req.headers['user-agent']}));
        }

        Promise.all([usuarioModel.getUsuario(id)])

            .then(usuarios => filtraNaoEncontrados(usuarios))
            .then(usuarios => agrupaEFiltraDuplicados(usuarios))
            .then(usuarios => desagrupa(usuarios))

            .then(usuarios => res.status(200).send(usuarios))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getUsuario = function (req : Request, res : Response){
    let id = req.params.id;

    if (id){
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_SISTEMA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.USUARIO_GET_USUARIO}: ${id}`, user_agent: req.headers['user-agent']}));
        }

        usuarioModel.getUsuario(id)

            .then(usuarios => res.status(200).send(usuarios))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let updateUsuario = function (req : Request, res : Response){
    let id = req.params.id;
    let dados = req.body;

    if (id) {
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_SISTEMA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.USUARIO_ATUALIZA_USUARIO}: ${id}`, user_agent: req.headers['user-agent']}));
        }

        let novoUsuario = new Usuario(dados);
        novoUsuario.ad = dados.ad;
        novoUsuario.proximoLogon = dados.proximoLogon;
        novoUsuario.idCadastrador = dados.idCadastrador;

        usuarioModel.updateUsuario_Sispesquisa_Usuario(novoUsuario)

            .then(usuarios => res.status(200).send(usuarios))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));

    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let updateSenhaUsuario = function (req : Request, res : Response){
    let dados = req.body;

    const resultado = validaTrocaSenha(dados);
    const {status, msg} = resultado;
    if (status !== API_CODES.CODE_SUCESSO) {
        res.status(400).send(resultado);
    } else {
        const id = getId_Token(req.headers['authorization']);

        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: id, tipo: LOG_CODES.TIPO_SISTEMA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.USUARIO_ATUALIZA_SENHA}: ${id}`, user_agent: req.headers['user-agent']}));
        }

        usuarioModel.updateSenhaUsuario_Sispesquisa_Usuario(id, dados.senhanova)
            .then(usuarios => res.status(200).send(usuarios))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    }
}

export let deleteUsuario = function (req : Request, res : Response){
    let id = req.params.id;

    if (id) {
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_SISTEMA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.USUARIO_REMOVE_USUARIO}: ${id}`, user_agent: req.headers['user-agent']}));
        }

        usuarioModel.removeControleAcessoUsuarioID_Sispesquisa_Usuario(id)
            .then(() => usuarioModel.deleteUsuario_Sispesquisa_Usuario(id))
            .then(usuarios => res.status(200).send(usuarios))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));

    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let redefinirSenhaUsuario = function (req : Request, res : Response) {
    let id = req.params.id;

    if (id) {
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_SISTEMA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.USUARIO_REDEFINIR_SENHA}: ${id}`, user_agent: req.headers['user-agent']}));
        }

        let passSize = 8;
        let novaSenha = gerarSenha(passSize);

        usuarioModel.updateSenhaUsuarioReset_Sispesquisa_Usuario(id, novaSenha)
            .then(usuario => res.status(200).send(usuario))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let getControleAcessoUsuario = function (req : Request, res : Response) {
    let id = req.params.id;

    if (id) {
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_SISTEMA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.USUARIO_REDEFINIR_SENHA}: ${id}`, user_agent: req.headers['user-agent']}));
        }

        Promise.all([usuarioModel.getControleAcessoPesquisaUsuario_Sispesquisa_Usuario(id),
                    usuarioModel.getControleAcessoAppsUsuario_Sispesquisa_Usuario(id),
                    usuarioModel.getControleAcessoAnaliseUsuario_Sispesquisa_Usuario(id),
                    usuarioModel.getControleAcessoOperacoesUsuario_Sispesquisa_Usuario(id),
                ])
            .then(res => {
                const dados = Object.assign(res[0], res[1], res[2], res[3]);

                return {
                    status: API_CODES.CODE_SUCESSO,
                    msg: '123',
                    dados: [dados]
                }
            })
            .then(acesso => res.status(200).send(acesso))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));
    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let atualizarControleAcessoUsuario = function (req : Request, res : Response) {
    let id       = req.params.id;
    let controle = req.body;

    if (id && controle) {
        if (API_CONFIG.CFG_ENV == 'production') {
            logModel.salvaLogPorId(new Log({ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_SISTEMA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${LOG_MSGS.USUARIO_REDEFINIR_SENHA}: ${id}`, user_agent: req.headers['user-agent']}));
        }

        let l = [];
        Object.keys(controle).forEach(secao => {
            l = l.concat(controle[secao]);
        })

        const listaPermissoes = l.join(',');

        usuarioModel.updateControleAcessoUsuario_Sispesquisa_Usuario(id, listaPermissoes)
            .then(usuario => res.status(200).send(usuario))
            .catch(error => res.status(500).send(trataErro(API_CODES.CODE_ERRO_500, error)));

    } else {
        res.status(400).send(trataErro(API_CODES.CODE_PARAM_INVALIDO, API_MSGS.MSG_PARAM_INVALIDO));
    }
}

export let gerarSenha = function (length) {
    let chars = "abcdefghijklmnopqrstuvwxyz!@#$%^&*-+ABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890";
    let senha = "";

    for (let x = 0; x < length; x++) {
        let i = Math.floor(Math.random() * chars.length);
        senha += chars.charAt(i);
    }

    return senha;
}