import * as mssql from 'mssql';
import * as rp from 'request-promise-native';

import { connection }  from '../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from '../config';
import { BenchmarkService } from '../services/benchmark.service';
import {
    resultNotFound,
    resultFound,
    logErroBuscaBD,
    printTempoExecucao
} from '../utils';

const fonte = MODEL_PRIORITY['nfce_mapa_calor'].fonte;
const rank  = MODEL_PRIORITY['nfce_mapa_calor'].rank;
const grupo = MODEL_PRIORITY['nfce_mapa_calor'].grupo;

export let getGeocoordenadasMapaCalorCPFPeriodo = function (cpf: string, datainicio: string = null, datafim: string = null){

    const tempoInicial = new BenchmarkService();

    let geoCoordenadasExistentes;
    let geoCoordenadasFaltantes;

    return connection.getConn().request()
        .input('CPF', mssql.Char(11), cpf)
        // .input('DATAINICIO', mssql.Date, datainicio)
        // .input('DATAFIM', mssql.Date, datafim)
        .query(`
            ;WITH NOTAS(CPF, DATAEMISSAO, CNPJ, LAT, LNG)
            AS
            (
                SELECT nr_cpf_cnpj_dest, dhemissao, nr_cnpj, PJ.LAT, PJ.LNG
                FROM SISPESQUISA.dbo.nfce_mapa_calor NF
                    LEFT OUTER JOIN SISPESQUISA.dbo.pj_geocoordenadas PJ ON (NF.nr_cnpj = PJ.CNPJ)
                WHERE nr_cpf_cnpj_dest=@CPF
            )

            SELECT CPF as cpf, DATAEMISSAO as dataEmissao, CNPJ as cnpj, LAT as lat, LNG as lng
            FROM NOTAS
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getGeocoordenadasMapaCalorCPFPeriodo');

            geoCoordenadasExistentes  = result.recordset.filter(d => !!d.lat && !!d.lng);
            geoCoordenadasFaltantes   = result.recordset.filter(d => !d.lat || !d.lng);
            const listaCNPJFaltantes  = geoCoordenadasFaltantes.map(d => d.cnpj);

            if (listaCNPJFaltantes.length !== 0) {
                return getEnderecosEmpresasFaltantes(listaCNPJFaltantes);
            } else {
                return geoCoordenadasExistentes;
            }
        })
        .then((resFaltantesEncontrados: any) => {
            resFaltantesEncontrados = resFaltantesEncontrados.filter(r => r !== null);

            if (geoCoordenadasFaltantes.length !== 0) {
                geoCoordenadasFaltantes = geoCoordenadasFaltantes.map(geo => {
                    const encontrado = resFaltantesEncontrados.filter(r => r.cnpj === geo.cnpj)[0];

                    if (encontrado && encontrado.lat && encontrado.lng) {
                        geo.lat = encontrado.lat;
                        geo.lng = encontrado.lng;

                        return geo;
                    } else {
                        return null;
                    }
                }).filter(r => r !== null);

                return geoCoordenadasExistentes.concat(geoCoordenadasFaltantes);
            } else {
                return geoCoordenadasExistentes;
            }

        }).then(res => {
            const fakeresult = {recordset: res};

            if (res.length === 0)
                return resultNotFound(`Nao foi possivel encontrar dados para ${cpf}.`, fonte);
            else {
                return resultFound(fakeresult, fonte, rank, grupo);
            }
        }).catch(error => logErroBuscaBD(error, `Falha na busca por dados de ${cpf}.`, 'getGeocoordenadasMapaCalorCPFPeriodo'));
};

let getEnderecosEmpresasFaltantes = function (listaCNPJ: Array<string>){

    const tempoInicial = new BenchmarkService();
    const lista = "'" + listaCNPJ.join("','") + "'";

    return connection.getConn().request()
        .query(`
            SELECT DISTINCT PJ.CNPJ as cnpj
                ,PJ.DescricaoTipoLogradouro + ' ' + PJ.Logradouro + ', ' + PJ.Numero + ', ' + PJ.Municipio COLLATE Latin1_General_CI_AS + ', Paraiba' AS addr

            FROM ReceitaNovo.dbo.PessoaJuridica PJ
            WHERE CNPJ IN (${lista})
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getEnderecosEmpresasFaltantes');

            const enderecos = result.recordset;
            return criaListaPromisesBuscaGeoCoordenadas(enderecos);
            // if(result.recordset.length === 0)
            //     return resultNotFound(`Nao foi possivel encontrar enderecos para ${lista}.`, fonte);
            // else {
            // }
        }).catch(error => logErroBuscaBD(error, `Falha na busca pelos enderecos de ${lista}.`, 'getEnderecosEmpresasFaltantes'));
};

let criaListaPromisesBuscaGeoCoordenadas = function (enderecos) {
    return Promise.all(enderecos.map(d => criaPromiseBuscaGeoCoordenadas(d)))
}


// endereco : {cnpj, addr}
let criaPromiseBuscaGeoCoordenadas = function (endereco) {

    const url = `https://maps.googleapis.com/maps/api/geocode/json?address=${endereco.addr}&key=${API_CONFIG.SIAP_GOOGLE_MAPS_API_KEY}`;
    let options = {
        method: 'GET',
        uri: url,
        timeout: 120000,
        json: true
    };

    const tempoInicial = new BenchmarkService();

    return rp(options)
        .then(data => {
            printTempoExecucao(tempoInicial, 'criaPromiseBuscaGeoCoordenadas');

            if(data['status'] === 'OK') {
                const lat = data['results'][0]['geometry']['location']['lat'];
                const lng = data['results'][0]['geometry']['location']['lng'];

                // Preenche no banco se dados foram encontrados!
                if (lat && lng) {
                    const cnpj = endereco.cnpj;
                    insertGeoCoordenadasEmpresaCNPJ(cnpj, lat, lng);

                    return {
                        cnpj: cnpj,
                        lat: lat,
                        lng: lng
                    }
                } else {
                    return null;
                }
            } else {
                return null;
            }
        })
        .catch(error => { console.error(error) });
}

let insertGeoCoordenadasEmpresaCNPJ = function (cnpj: string, lat: number, lng: number) {
    return connection.getConn().request()
        .input('CNPJ', mssql.Char(14), cnpj)
        .input('LAT', mssql.Float, lat)
        .input('LNG', mssql.Float, lng)
        .query(`INSERT INTO SISPESQUISA.dbo.pj_geocoordenadas
                (CNPJ, LAT, LNG, FONTE)
                    VALUES
                (@CNPJ, @LAT, @LNG, 'RECEITANOVO')
        `)
        .then(result => {
            return true;
        }).catch(error => { return logErroBuscaBD(error, `Falha na insercao da geocoordenada.`, 'insertGeoCoordenadasEmpresaCNPJ'); });
}