import { Request } from 'express';
import * as mssql from 'mssql';

import { connection }  from '../services/db.service';
import { Log } from '../schemas/log.schema';
import {
    resultNotFound,
    resultFound,
    logErroBuscaBD
} from '../utils';
import {
    MODEL_PRIORITY,
    API_CODES,
    API_CONFIG,
    API_MSGS
} from '../config';

const fonte   = MODEL_PRIORITY['sispesquisa.logs'].fonte;
const rank    = MODEL_PRIORITY['sispesquisa.logs'].rank;
const grupo   = MODEL_PRIORITY['sispesquisa.logs'].grupo;

export let getLogs = function (){

    return connection.getConn().request()
        .query(`SELECT  ip, usuario, tipo, mensagem, data_hora as dataHora, user_agent as userAgent, processo
                FROM SISPESQUISA.sispesquisa.logs
                ORDER BY data_hora DESC
                OFFSET 0 ROWS FETCH NEXT 100 ROWS ONLY`)
        .then(result => {
            return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelos logs.`, 'getLogs'); });
};

export let moreLogs = function (offset: string){

    return connection.getConn().request()
        .query(`SELECT  ip, usuario, tipo, mensagem, data_hora as dataHora, user_agent as userAgent, processo
                FROM SISPESQUISA.sispesquisa.logs
                ORDER BY data_hora DESC
                OFFSET ${offset} ROWS FETCH NEXT 100 ROWS ONLY`)
        .then(result => {
                return resultFound(result, "", 0, "");
        }).catch( error => {  
            throw new Error(`Falha na busca pelos logs.`);
        });
};

export let salvaLog = function(dados : Log) {

    return connection.getConn().request()
        .input('ip', mssql.NVarChar, dados.ip)
        .input('usuario', mssql.NVarChar, dados.usuario)
        .input('tipo', mssql.NVarChar, dados.tipo)
        .input('mensagem', mssql.NVarChar, dados.mensagem)
        .input('data_hora', mssql.NVarChar, dados.data_hora)
        .input('user_agent', mssql.NVarChar, dados.user_agent)
        .input('processo', mssql.NVarChar, dados.processo)
        .query(`INSERT INTO SISPESQUISA.sispesquisa.logs
                VALUES
                (@ip, @usuario, @tipo, @mensagem, @data_hora,
                @user_agent, @processo)
        `)
        .then(result => {
            return {
                status: API_CODES.CODE_SUCESSO,
                msg: 'Log cadastrado com sucesso.'
            }
        }).catch( error => {logErroBuscaBD(error, `Falha no cadastro do log.`, 'salvaLog')});
}

export let salvaLogPorId = function(dados : Log) {

    return connection.getConn().request()
        .input('ip', mssql.NVarChar, dados.ip)
        .input('usuario_id', mssql.NVarChar, dados.usuario_id )
        .input('tipo', mssql.NVarChar, dados.tipo)
        .input('mensagem', mssql.NVarChar, dados.mensagem)
        .input('data_hora', mssql.NVarChar, dados.data_hora)
        .input('user_agent', mssql.NVarChar, dados.user_agent)
        .input('processo', mssql.NVarChar, dados.processo)
        .query(`INSERT INTO SISPESQUISA.sispesquisa.logs
                    (ip, usuario, tipo, mensagem, data_hora, user_agent, processo)
                SELECT @ip, login, @tipo, @mensagem, @data_hora, @user_agent, @processo
                FROM SISPESQUISA.sispesquisa.usuario
                WHERE id=@usuario_id
        `)
        .then(result => {
            return {
                status: API_CODES.CODE_SUCESSO,
                msg: 'Log cadastrado com sucesso.'
            }
        }).catch( error => {logErroBuscaBD(error, `Falha no cadastro do log.`, 'salvaLogPorId')});
}

export let getRankingUsoSIAP = function (duracao: string, top: string = '100', versao: string = 'v2'){
    
    let filtro_duracao;
    if  (duracao){
        filtro_duracao = `AND data_hora >= DATEADD(Day, -${duracao}, getdate())`;
    } else {
        filtro_duracao = '';
    }

    let filtro_top;
    if (top){
        filtro_top = `TOP ${top}`;
    } else {
        filtro_top = '';
    }

    let filtro_versao;
    if (versao === 'siapv2'){
        filtro_versao = `LIKE 'SIAP2%'
                        AND TIPO !='CADASTRO DE CADASTRO_USUÁRIO'
                        AND TIPO !='TENTATIVA DE LOGIN' `;
    } else if (versao === 'siapv1'){
        filtro_versao = `NOT LIKE 'SIAP2%' `;
    }

    return connection.getConn().request()
        .query(`SELECT ${filtro_top}
                    usuario, COUNT(usuario) as qtd
                FROM SISPESQUISA.sispesquisa.logs
                WHERE mensagem ${filtro_versao}
                    ${filtro_duracao}
                GROUP BY usuario
                ORDER BY qtd desc
        `)
        .then(result => {
            return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelos rankings.`, 'getRankingUsoSIAP'); });
};

export let getRecursosMaisPesquisados_Pesquisa = function (duracao: string){
    
    let filtro_duracao;
    if  (duracao){
        filtro_duracao = `WHERE data_hora >= DATEADD(Day, -${duracao}, getdate())`;
    } else {
        filtro_duracao = '';
    }

    return connection.getConn().request()
        .query(`SELECT recurso, COUNT(recurso) as qtd
                FROM (
                    SELECT usuario,
                        SUBSTRING(mensagem, CHARINDEX(' - ', mensagem)+3,   CHARINDEX(' - ', mensagem, 7) - CHARINDEX(' - ', mensagem)-3) AS recurso, 
                        SUBSTRING(mensagem, CHARINDEX('BUSCA', mensagem)+6, CHARINDEX('POR', mensagem) - CHARINDEX('BUSCA', mensagem)-7) AS tipo, 
                        SUBSTRING(mensagem, CHARINDEX('POR', mensagem)+4,   CHARINDEX(':', mensagem) - CHARINDEX('POR', mensagem)-4) AS chave, 
                        SUBSTRING(mensagem, CHARINDEX(':', mensagem)+2,     LEN(mensagem) - CHARINDEX(':', mensagem) ) AS valor,
                        data_hora
                        
                    FROM SISPESQUISA.sispesquisa.logs
                    WHERE mensagem LIKE 'SIAP2%'
                        AND TIPO='PESQUISA'
                        AND usuario!='DEV'
                ) as TABELA_CONSULTAS
                ${filtro_duracao}
                GROUP BY recurso
                ORDER BY qtd desc
        `)
        .then(result => {
            return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelos recursos usados.`, 'getRecursosMaisPesquisados_Pesquisa'); });
};

export let getRecursosEChavesMaisPesquisados_Pesquisa = function (duracao: string){
    
    let filtro_duracao;
    if  (duracao){
        filtro_duracao = `WHERE data_hora >= DATEADD(Day, -${duracao}, getdate())`;
    } else {
        filtro_duracao = '';
    }

    return connection.getConn().request()
        .query(`SELECT recurso, chave, COUNT(*) as qtd
                FROM (
                    SELECT usuario,
                        SUBSTRING(mensagem, CHARINDEX(' - ', mensagem)+3,   CHARINDEX(' - ', mensagem, 7) - CHARINDEX(' - ', mensagem)-3) AS recurso, 
                        SUBSTRING(mensagem, CHARINDEX('BUSCA', mensagem)+6, CHARINDEX('POR', mensagem) - CHARINDEX('BUSCA', mensagem)-7) AS tipo, 
                        SUBSTRING(mensagem, CHARINDEX('POR', mensagem)+4,   CHARINDEX(':', mensagem) - CHARINDEX('POR', mensagem)-4) AS chave, 
                        SUBSTRING(mensagem, CHARINDEX(':', mensagem)+2,     LEN(mensagem) - CHARINDEX(':', mensagem) ) AS valor,
                        data_hora
                        
                    FROM SISPESQUISA.sispesquisa.logs
                    WHERE mensagem LIKE 'SIAP2%'
                        AND TIPO='PESQUISA'
                        AND usuario!='DEV'
                ) as TABELA_CONSULTAS
                ${filtro_duracao}
                GROUP BY recurso, chave
                ORDER BY recurso, qtd desc
        `)
        .then(result => {
            return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelos recursos e chaves mais usados.`, 'getRecursosEChavesMaisPesquisados_Pesquisa'); });
};


export let getCPFsMaisPesquisados_Pesquisa = function (duracao: string, top: string = '100'){
    
    let filtro_duracao;
    if  (duracao){
        filtro_duracao = `AND data_hora >= DATEADD(Day, -${duracao}, getdate())`;
    } else {
        filtro_duracao = '';
    }

    let filtro_top = `TOP ${top}`;

    return connection.getConn().request()
        .query(`SELECT RESULTADO.CPF as cpf, PF.Nome as nome, RESULTADO.QTD as qtd
                FROM (
                SELECT ${filtro_top}
                    TABELA_CONSULTAS.valor as CPF, COUNT(*) as QTD
                    FROM (
                        SELECT usuario,
                            SUBSTRING(mensagem, CHARINDEX(' - ', mensagem)+3,   CHARINDEX(' - ', mensagem, 7) - CHARINDEX(' - ', mensagem)-3) AS recurso, 
                            SUBSTRING(mensagem, CHARINDEX('BUSCA', mensagem)+6, CHARINDEX('POR', mensagem) - CHARINDEX('BUSCA', mensagem)-7) AS tipo, 
                            SUBSTRING(mensagem, CHARINDEX('POR', mensagem)+4,   CHARINDEX(':', mensagem) - CHARINDEX('POR', mensagem)-4) AS chave, 
                            SUBSTRING(mensagem, CHARINDEX(':', mensagem)+2,     LEN(mensagem) - CHARINDEX(':', mensagem) ) AS valor,
                            data_hora
                        
                        FROM SISPESQUISA.sispesquisa.logs
                        WHERE mensagem LIKE 'SIAP2%'
                            AND TIPO='PESQUISA'
                            AND usuario!='DEV'
                    ) as TABELA_CONSULTAS
                    WHERE TABELA_CONSULTAS.chave='CPF'
                        ${filtro_duracao}
                    GROUP BY TABELA_CONSULTAS.valor
                    ORDER BY QTD DESC
                ) AS RESULTADO
                LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica AS PF
                    ON (RESULTADO.CPF  = PF.CPF)
        `)
        .then(result => {
            return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelos CPFs mais usados.`, 'getCPFsMaisPesquisados_Pesquisa'); });
};

export let getCNPJsMaisPesquisados_Pesquisa = function (duracao: string, top: string = '100'){
    
    let filtro_duracao;
    if  (duracao){
        filtro_duracao = `AND data_hora >= DATEADD(Day, -${duracao}, getdate())`;
    } else {
        filtro_duracao = '';
    }

    const filtro_top = `TOP ${top}`;

    return connection.getConn().request()
        .query(`SELECT RESULTADO.CNPJ as cnpj, PJ.NomeFantasia as nomeFantasia, PJ.RazaoSocial as razaoSocial, RESULTADO.QTD as qtd
                FROM (
                SELECT ${filtro_top}
                    TABELA_CONSULTAS.valor as CNPJ, COUNT(*) as QTD
                    FROM (
                        SELECT usuario,
                            SUBSTRING(mensagem, CHARINDEX(' - ', mensagem)+3,   CHARINDEX(' - ', mensagem, 7) - CHARINDEX(' - ', mensagem)-3) AS recurso, 
                            SUBSTRING(mensagem, CHARINDEX('BUSCA', mensagem)+6, CHARINDEX('POR', mensagem) - CHARINDEX('BUSCA', mensagem)-7) AS tipo, 
                            SUBSTRING(mensagem, CHARINDEX('POR', mensagem)+4,   CHARINDEX(':', mensagem) - CHARINDEX('POR', mensagem)-4) AS chave, 
                            SUBSTRING(mensagem, CHARINDEX(':', mensagem)+2,     LEN(mensagem) - CHARINDEX(':', mensagem) ) AS valor,
                            data_hora
                        
                        FROM SISPESQUISA.sispesquisa.logs
                        WHERE mensagem LIKE 'SIAP2%'  
                            AND TIPO='PESQUISA'
                            AND usuario!='DEV'
                    ) as TABELA_CONSULTAS
                    WHERE TABELA_CONSULTAS.chave='CNPJ'
                        ${filtro_duracao}
                    GROUP BY TABELA_CONSULTAS.valor
                    ORDER BY QTD DESC
                ) AS RESULTADO
                LEFT OUTER JOIN ReceitaNovo.dbo.PessoaJuridica AS PJ
                    ON (RESULTADO.CNPJ = PJ.CNPJ)
                
        `)
        .then(result => {
            return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelos CNPJs mais usados.`, 'getCNPJsMaisPesquisados_Pesquisa'); });
};

export let getValoresMaisPesquisados_Pesquisa = function (duracao: string, top: string = '100', multiusuario: string = null){
    
    let filtro_duracao;
    if  (duracao){
        filtro_duracao = `AND data_hora >= DATEADD(Day, -${duracao}, getdate())`;
    } else {
        filtro_duracao = '';
    }

    let filtro_multiusuario;
    if  (multiusuario){
        filtro_multiusuario = `HAVING COUNT(DISTINCT usuario) > 1`;
    } else {
        filtro_multiusuario = '';
    }

    const filtro_top = `TOP ${top}`;

    return connection.getConn().request()
        .query(`SELECT ${filtro_top}
                    chave, valor, count(*) AS qtd
                FROM (
                    SELECT usuario, 
                                --mensagem, 
                                SUBSTRING(mensagem, CHARINDEX(' - ', mensagem)+3,   CHARINDEX(' - ', mensagem, 7) - CHARINDEX(' - ', mensagem)-3) AS recurso, 
                                SUBSTRING(mensagem, CHARINDEX('BUSCA', mensagem)+6, CHARINDEX('POR', mensagem) - CHARINDEX('BUSCA', mensagem)-7) AS tipo, 
                                SUBSTRING(mensagem, CHARINDEX('POR', mensagem)+4,   CHARINDEX(':', mensagem) - CHARINDEX('POR', mensagem)-4) AS chave, 
                                SUBSTRING(mensagem, CHARINDEX(':', mensagem)+2,     LEN(mensagem) - CHARINDEX(':', mensagem) ) AS valor,
                                data_hora
                            
                        FROM SISPESQUISA.sispesquisa.logs
                        WHERE mensagem LIKE 'SIAP2%'
                            AND TIPO='PESQUISA'
                            AND usuario!='DEV'
                            ${filtro_duracao}
                ) AS RESULTADO
                GROUP BY chave, valor
                ${filtro_multiusuario}
                ORDER BY qtd DESC
        `)
        .then(result => {
            return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelos valores mais usados.`, 'getValoresMaisPesquisados_Pesquisa'); });
};

export let getUsuariosQuePesquisaram_Pesquisa = function (duracao: string, dados = null){
    
    let filtro_duracao;
    if  (duracao){
        filtro_duracao = `AND data_hora >= DATEADD(Day, -${duracao}, getdate())`;
    } else {
        filtro_duracao = '';
    }

    return connection.getConn().request()
        .input('chave', mssql.NVarChar, dados.chave)
        .input('valor', mssql.NVarChar, dados.valor)
        .query(`SELECT 
                    usuario, count(*) as qtd
                FROM (
                    SELECT usuario,
                            SUBSTRING(mensagem, CHARINDEX(' - ', mensagem)+3,   CHARINDEX(' - ', mensagem, 7) - CHARINDEX(' - ', mensagem)-3) AS recurso, 
                            SUBSTRING(mensagem, CHARINDEX('BUSCA', mensagem)+6, CHARINDEX('POR', mensagem) - CHARINDEX('BUSCA', mensagem)-7) AS tipo, 
                            SUBSTRING(mensagem, CHARINDEX('POR', mensagem)+4,   CHARINDEX(':', mensagem) - CHARINDEX('POR', mensagem)-4) AS chave, 
                            SUBSTRING(mensagem, CHARINDEX(':', mensagem)+2,     LEN(mensagem) - CHARINDEX(':', mensagem) ) AS valor,
                            data_hora
                            
                        FROM SISPESQUISA.sispesquisa.logs
                        WHERE mensagem LIKE 'SIAP2%'
                            AND TIPO='PESQUISA'
                            AND usuario!='DEV'
                            ${filtro_duracao}
                ) AS RESULTADO
                WHERE chave=@chave
                    AND valor=@valor
                GROUP BY usuario
                ORDER BY qtd DESC
        `)
        .then(result => {
            return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelos valores mais usados.`, 'getValoresMaisPesquisados_Pesquisa'); });
};

export let getTokensValidos = function () {

    const duracaoToken = API_CONFIG.TEMPO_TOKEN;

    return connection.getConn().request()
        .query(`SELECT 
                    ip,
                    usuario,
                    tipo,
                    data_hora as dataHora,
                    DATEADD(SECOND, ${duracaoToken}, data_hora) as validade,
                    user_agent

                FROM SISPESQUISA.sispesquisa.logs
                WHERE (tipo='LOGIN EXTERNO' OR tipo='LOGIN')
                    AND data_hora >= DATEADD(SECOND, -${duracaoToken}, getdate())
                ORDER BY data_hora desc
        `)
        .then(result => {
            return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos tokens validos.`, 'getTokensValidos'); });
}

export let getQuantidadePesquisas = function (duracao: string) {

    const duracaoToken = API_CONFIG.TEMPO_TOKEN;
    return connection.getConn().request()
        .query(`SELECT 
                    DATEFROMPARTS ( YEAR(data_hora),  MONTH(data_hora), DAY(data_hora) ) as data,
                    count(*) as qtd
                    
                FROM SISPESQUISA.sispesquisa.logs
                WHERE tipo='PESQUISA' AND data_hora >= DATEADD(Day, -${duracao}, getdate())
                GROUP BY YEAR(data_hora), MONTH(data_hora), DAY(data_hora)
                ORDER BY YEAR(data_hora) desc, MONTH(data_hora) desc, DAY(data_hora) desc
        `)
        .then(result => {
            return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela quantidade de pesquisas.`, 'getQuantidadePesquisas'); });
}

export let getQuantidadeLogins = function (duracao: string) {

    const duracaoToken = API_CONFIG.TEMPO_TOKEN;
    return connection.getConn().request()
        .query(`SELECT 
                    DATEFROMPARTS ( YEAR(data_hora),  MONTH(data_hora), DAY(data_hora) ) as data,
                    count(*) as qtd
                    
                FROM SISPESQUISA.sispesquisa.logs
                WHERE (tipo='LOGIN EXTERNO' OR tipo='LOGIN') AND data_hora >= DATEADD(Day, -${duracao}, getdate())
                GROUP BY YEAR(data_hora), MONTH(data_hora), DAY(data_hora)
                ORDER BY YEAR(data_hora) desc, MONTH(data_hora) desc, DAY(data_hora) desc
        `)
        .then(result => {
            return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela quantidade de logins.`, 'getQuantidadeLogins'); });
}

export let getCPFsNaoEncontrados = function () {

    return connection.getConn().request()
        .query(`
            ;WITH CPFS_NAO_ENCONTRADOS(CPF)
            AS
            (   
                SELECT DISTINCT SUBSTRING(mensagem, CHARINDEX('CPF: ', mensagem)+5, LEN(mensagem)) AS cpf
                FROM SISPESQUISA.sispesquisa.logs
                WHERE mensagem LIKE 'SIAP2 - CPF:%'
                    AND TIPO='REGISTRO NAO ENCONTRADO'
            )
            , EXISTE_NA_BASE(CPF)
            AS
            (	
                SELECT C.CPF as cpf
                FROM CPFS_NAO_ENCONTRADOS C
                    INNER JOIN SISPESQUISA.sispesquisa.pessoas PF ON (C.CPF = PF.CPF)	
                
                UNION

                SELECT C.CPF as cpf
                FROM CPFS_NAO_ENCONTRADOS C
                    INNER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (C.CPF = PF.CPF)
            )
            , RESULTADO(CPF)
            AS
            (
                SELECT CPF as cpf
                FROM CPFS_NAO_ENCONTRADOS

                EXCEPT

                SELECT CPF as cpf
                FROM EXISTE_NA_BASE
            )

            SELECT CPF as cpf, 'cpf' as tipo
            FROM RESULTADO
        `)
        .then(result => {
            return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos cpfs inexistentes.`, 'getCPFsNaoEncontrados'); });
}

export let getCNPJsNaoEncontrados = function () {

    return connection.getConn().request()
        .query(`
            ;WITH CNPJS_NAO_ENCONTRADOS(CNPJ)
            AS
            (   
                SELECT DISTINCT SUBSTRING(mensagem, CHARINDEX('CNPJ: ', mensagem)+6, LEN(mensagem)) AS cpf
                FROM SISPESQUISA.sispesquisa.logs
                WHERE mensagem LIKE 'SIAP2 - CNPJ:%'
                    AND TIPO='REGISTRO NAO ENCONTRADO'
            )

            SELECT C.CNPJ as cnpj, 'cnpj' as tipo
            FROM CNPJS_NAO_ENCONTRADOS C
                LEFT OUTER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (C.CNPJ = PJ.CNPJ)
            WHERE PJ.CNPJ IS NULL
        `)
        .then(result => {
            return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos cnpjs inexistentes.`, 'getCNPJsNaoEncontrados'); });
}



//https://www.robtex.com/api/
//https://freeapi.robtex.com/ipquery/186.215.248.218
//http://ipinfo.io/186.215.248.218

//https://github.com/IonicaBizau/node-ipinfo
//https://github.com/bluesmoon/node-geoip