import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { MODEL_PRIORITY, API_CONFIG } from './../../config';
import {
    resultNotFound,
    modelAddWhereTipoComparacao,
    modelAddWhereDataNascimento,
    modelAddWherePrimeiroNome,
    resultFound,
    removerAcentos,
    logErroBuscaBD
} from '../../utils';

import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['receita_full'];

const fonte  = MODEL_PRIORITY['receita_full.pf'].fonte;
const rank   = MODEL_PRIORITY['receita_full.pf'].rank;
const grupo  = MODEL_PRIORITY['receita_full.pf'].grupo;

export let getPessoaSimplificadoNome_ReceitaFull_PF = function (nome: string,
                                                                tipoComparacao='fulltext',
                                                                dataNascimentoFilho=undefined,
                                                                travaPrimeiroNome=false){

    let condicao = modelAddWhereTipoComparacao('Nome', '@nome_parameter', tipoComparacao);
    let c1 = modelAddWhereDataNascimento('DataNascimento', '@dt_nasc_filho_parameter', dataNascimentoFilho);
    let c2 = modelAddWherePrimeiroNome('Nome', '@primeiro_nome_parameter', travaPrimeiroNome);
    condicao += (c1) ? c1 : '';
    condicao += (c2) ? c2 : '';

    let primeiroNome = removerAcentos(nome.split(" ")[0]);

    return connection.getConn().request()
        .input('nome_parameter', mssql.VarChar, nome)
        .input('primeiro_nome_parameter', mssql.VarChar, primeiroNome + ' %')
        .input('dt_nasc_filho_parameter', mssql.Date, dataNascimentoFilho)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPF as cpf,
                        UPPER(Nome) as nome,
                        UPPER(NomeMae) as nomeMae,
                        UPPER(Municipio) as municipio,
                        UPPER(UF) as uf,
                        '${fonte_dado}' as fonte,
                        CASE
                            WHEN isdate(DataNascimento) = 1 THEN cast(DataNascimento as date)
                            ELSE NULL
                        END AS dataNascimento

                FROM Receita_Full.Receita.PF
                ${condicao}
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${nome}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${nome}.`, 'getPessoaSimplificadoNome_ReceitaFull_PF') });

};

export let getPessoaSimplificadoTitulo_ReceitaFull_PF = function (titulo: string){

    return connection.getConn().request()
        .input('titulo_parameter', mssql.VarChar, titulo)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPF as cpf,
                        UPPER(Nome) as nome,
                        UPPER(NomeMae) as nomeMae,
                        UPPER(Municipio) as municipio,
                        UPPER(UF) as uf,
                        '${fonte_dado}' as fonte,
                        CASE
                            WHEN isdate(DataNascimento) = 1 THEN cast(DataNascimento as date)
                            ELSE NULL
                        END AS dataNascimento,
                        CASE
                            WHEN RIGHT(TituloEleitor, 12) = '000000000000' THEN NULL
                            ELSE RIGHT(TituloEleitor, 12)
                        END AS tituloEleitor

                FROM Receita_Full.Receita.PF
                WHERE RIGHT(TituloEleitor, 12)=@titulo_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${titulo}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${titulo}.`, 'getPessoaSimplificadoTitulo_ReceitaFull_PF'); });
}

export let getPessoaNomeMae_ReceitaFull_PF = function (nomeMae: string, tipoComparacao='fulltext'){

    let condicao = modelAddWhereTipoComparacao('NomeMae', '@nome_mae_parameter', tipoComparacao);

    return connection.getConn().request()
        .input('nome_mae_parameter', mssql.VarChar, nomeMae)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPF as cpf,
                        UPPER(Nome) as nome,
                        UPPER(NomeMae) as nomeMae,
                        UPPER(Municipio) as municipio,
                        UPPER(UF) as uf,
                        '${fonte_dado}' as fonte,
                        CASE
                            WHEN isdate(DataNascimento) = 1 THEN cast(DataNascimento as date)
                            ELSE NULL
                        END AS dataNascimento

                FROM Receita_Full.Receita.PF
                ${condicao}
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${nomeMae}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${nomeMae}.`, 'getPessoaNomeMae_ReceitaFull_PF'); });
}

export let getPessoaDetalhadoCPF_ReceitaFull_PF = function (cpf: String){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.VarChar, cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPF as cpf,
                        UPPER(TipoSituacaoCadastralPF.Descricao) as situacaoCadastral,
                        '${fonte_dado}' as fonte,
                        UPPER(Nome) as nome,
                        UPPER(NomeMae) as nomeMae,
                        CASE
                            WHEN AnoObito = 0 THEN NULL
                            ELSE AnoObito
                        END as anoObito,
                        CASE
                            WHEN isdate(DataAtualizacao) = 1 THEN cast(DataAtualizacao as date)
                            ELSE NULL
                        END AS dataAtualizacao,
                        CASE
                            WHEN isdate(DataNascimento) = 1 THEN cast(DataNascimento as date)
                            ELSE NULL
                        END AS dataNascimento,
                        CASE
                            WHEN (Estrangeiro) = 0 THEN 'NÃO'
                            ELSE 'SIM'
                        END AS estrangeiro,
                        CASE
                            WHEN (ResidenteExterior) = 0 THEN 'NÃO'
                            ELSE 'SIM'
                        END AS residenteExterior,
                        UPPER(NomePaisExterior) as nomePaisExterior,
                        CASE
                            WHEN RIGHT(TituloEleitor, 12) = '000000000000' THEN NULL
                            ELSE RIGHT(TituloEleitor, 12)
                        END AS tituloEleitor,
                        UPPER(TipoSexo.Descricao) as sexo,
                        UPPER(TipoNaturezaOcupacao.Descricao) as naturezaOcupacao,
                        CASE
                            WHEN ExercicioOcupacao = 0 THEN NULL
                            ELSE ExercicioOcupacao
                        END as anoExercicioOcupacao,
                        UPPER(TipoOcupacaoPrincipal.Tipo) as tipoOcupacaoPrincipal,
                        UPPER(TipoOcupacaoPrincipal.Descricao) as ocupacaoPrincipal

                FROM Receita_Full.Receita.PF
                    INNER JOIN Receita_Full.Receita.TipoSexo ON (PF.Sexo = TipoSexo.Sexo)
                    INNER JOIN Receita_Full.Receita.TipoSituacaoCadastralPF ON (PF.SituacaoCadastral = TipoSituacaoCadastralPF.SituacaoCadastral)
                    LEFT OUTER JOIN Receita_Full.Receita.TipoNaturezaOcupacao ON (PF.NaturezaOcupacao = TipoNaturezaOcupacao.NaturezaOcupacao)
                    LEFT OUTER JOIN Receita_Full.Receita.TipoOcupacaoPrincipal ON (PF.OcupacaoPrincipal = TipoOcupacaoPrincipal.OcupacaoPrincipal)
                WHERE CPF=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cpf}.`, 'getPessoaDetalhadoCPF_ReceitaFull_PF'); });
};

export let getPessoaSimplificadoCPF_ReceitaFull_PF = function (cpf: String){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.VarChar, cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPF as cpf,
                        UPPER(Nome) as nome,
                        UPPER(NomeMae) as nomeMae,
                        UPPER(Municipio) as municipio,
                        UPPER(UF) as uf,
                        '${fonte_dado}' as fonte,
                        CASE
                            WHEN isdate(DataNascimento) = 1 THEN cast(DataNascimento as date)
                            ELSE NULL
                        END AS dataNascimento

                FROM Receita_Full.Receita.PF
                WHERE CPF=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${cpf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cpf}.`, 'getPessoaSimplificadoCPF_ReceitaFull_PF'); });
};

export let getPessoaTelefone_ReceitaFull_PF = function (telefone: string){
    return connection.getConn().request()
        .input('telefone_parameter', mssql.VarChar, telefone)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPF as cpf,
                        UPPER(PF.Nome) as nome,
                        UPPER(NomeMae) as nomeMae,
                        UPPER(Municipio) as municipio,
                        UPPER(UF) as uf,
                        CONCAT(DDD,Telefone) as telefone,
                        '${fonte_dado}' as fonte

                FROM Receita_Full.Receita.PF
                WHERE Telefone=@telefone_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa com telefone ${telefone}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa com telefone ${telefone}.`, 'getPessoaTelefone_ReceitaFull_PF'); });
}