import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import { Pessoa } from '../schemas';
import { MODEL_PRIORITY, API_CONFIG } from './../../config';
import {
    trataResultado,
    resultNotFound,
    resultFound,
    modelAddWhereTipoComparacao,
    modelAddWhereDataNascimento,
    modelAddWherePrimeiroNome,
    removerAcentos,
    logErroBuscaBD
} from '../../utils';

import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['sispesquisa.pessoas'];

const fontePessoas   = MODEL_PRIORITY['sispesquisa.pessoas'].fonte;
const rankPessoas    = MODEL_PRIORITY['sispesquisa.pessoas'].rank;
const grupoPessoas   = MODEL_PRIORITY['sispesquisa.pessoas'].grupo;

export let getPessoaDetalhadoCPF_Sispesquisa_Pessoas = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.NChar(11), cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPF as cpf,
                        UPPER(SituacaoCadastral) as situacaoCadastral,
                        UPPER(Nome) as nome,
                        UPPER(NomeMae) as nomeMae,
                        UPPER(Sexo) as sexo,
                        cast(DataNascimento as date) as dataNascimento,
                        cast(DataAtualizacao as date) as dataAtualizacao,
                        AnoObito as anoObito,
                        Estrangeiro as estrangeiro,
                        NomePaisExterior as nomePaisExterior,
                        ResidenteExterior as residenteExterior,
                        RIGHT(TituloEleitor,12) as tituloEleitor,
                        '${fonte_dado}' as fonte

                FROM SISPESQUISA.sispesquisa.pessoas
                WHERE cpf=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${cpf}.`, fontePessoas);
            else
                return resultFound(result, fontePessoas, rankPessoas, grupoPessoas);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cpf}.`, 'getPessoaDetalhadoCPF_Sispesquisa_Pessoas'); });
};

export let getPessoaSimplificadoCPF_Sispesquisa_Pessoas = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.NChar(11), cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        pessoas.CPF as cpf,
                        UPPER(pessoas.Nome) as nome,
                        UPPER(pessoas.NomeMae) as nomeMae,
                        UPPER(PF.Municipio) as municipio,
                        UPPER(PF.UF) as uf,
                        cast(pessoas.DataNascimento as date) as dataNascimento,
                        '${fonte_dado}' as fonte

                FROM SISPESQUISA.sispesquisa.pessoas
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica AS PF ON (PF.CPF = pessoas.CPF)
                WHERE pessoas.cpf=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${cpf}.`, fontePessoas);
            else
                return resultFound(result, fontePessoas, rankPessoas, grupoPessoas);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cpf}.`, 'getPessoaSimplificadoCPF_Sispesquisa_Pessoas'); });
};

export let getPessoaSimplificadoTitulo_Sispesquisa_Pessoas = function (titulo: string){

        return connection.getConn().request()
            .input('titulo_parameter', mssql.NVarChar, titulo)
            .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                            pessoas.CPF as cpf,
                            UPPER(pessoas.Nome) as nome,
                            UPPER(pessoas.NomeMae) as nomeMae,
                            UPPER(PF.Municipio) as municipio,
                            UPPER(PF.UF) as uf,
                            cast(pessoas.DataNascimento as date) as dataNascimento,
                            '${fonte_dado}' as fonte

                    FROM SISPESQUISA.sispesquisa.pessoas
                        LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica AS PF ON (PF.CPF = pessoas.CPF)
                    WHERE pessoas.TituloEleitor=@titulo_parameter
            `)
            .then(result => {
                if(result.recordset.length === 0)
                    return resultNotFound(`Nao foi possivel encontrar a pessoa ${titulo}.`, fontePessoas);
                else
                    return resultFound(result, fontePessoas, rankPessoas, grupoPessoas);
            }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${titulo}.`, 'getPessoaSimplificadoTitulo_Sispesquisa_Pessoas'); });
}

// Buscas pelo Nome
export let getPessoaSimplificadoNome_Sispesquisa_Pessoas = function (nome: string,
                                                                    tipoComparacao='fulltext',
                                                                    dataNascimentoFilho=undefined,
                                                                    travaPrimeiroNome=false){

    let condicao = modelAddWhereTipoComparacao('pessoas.Nome', '@nome_parameter', tipoComparacao);
    let c1 = modelAddWhereDataNascimento('DN', '@dt_nasc_filho_parameter', dataNascimentoFilho);
    let c2 = modelAddWherePrimeiroNome('Nome', '@primeiro_nome_parameter', travaPrimeiroNome);
    condicao += (c1) ? c1 : '';
    condicao += (c2) ? c2 : '';

    let primeiroNome = removerAcentos(nome.split(" ")[0]);

    return connection.getConn().request()
        .input('nome_parameter', mssql.NVarChar, nome)
        .input('primeiro_nome_parameter', mssql.NVarChar, primeiroNome + ' %')
        .input('dt_nasc_filho_parameter', mssql.Date, dataNascimentoFilho)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        pessoas.CPF as cpf,
                        UPPER(pessoas.Nome) as nome,
                        UPPER(pessoas.NomeMae) as nomeMae,
                        UPPER(PF.Municipio) as municipio,
                        UPPER(PF.UF) as uf,
                        cast(pessoas.DataNascimento as date) as dataNascimento,
                        '${fonte_dado}' as fonte

                FROM SISPESQUISA.sispesquisa.pessoas
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica AS PF ON (PF.CPF = pessoas.CPF)
                ${condicao}
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${nome}.`, fontePessoas);
            else
                return resultFound(result, fontePessoas, rankPessoas, grupoPessoas);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${nome}.`, 'getPessoaSimplificadoNome_Sispesquisa_Pessoas'); });
};


export let getPessoaNomeMae_Sispesquisa_Pessoas = function (nomeMae: string, tipoComparacao='fulltext'){

    let condicao = modelAddWhereTipoComparacao('pessoas.NomeMae', '@nome_mae_parameter', tipoComparacao);

    return connection.getConn().request()
        .input('nome_mae_parameter', mssql.NVarChar, nomeMae)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        pessoas.CPF as cpf,
                        UPPER(pessoas.Nome) as nome,
                        UPPER(pessoas.NomeMae) as nomeMae,
                        UPPER(PF.Municipio) as municipio,
                        UPPER(PF.UF) as uf,
                        cast(pessoas.DataNascimento as date) as dataNascimento,
                        '${fonte_dado}' as fonte

                FROM SISPESQUISA.sispesquisa.pessoas
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica AS PF ON (PF.CPF = pessoas.CPF)
                ${condicao}
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${nomeMae}.`, fontePessoas);
            else
                return resultFound(result, fontePessoas, rankPessoas, grupoPessoas);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${nomeMae}.`, 'getPessoaNomeMae_Sispesquisa_Pessoas'); });
}