import * as mssql from 'mssql';

import { connection }  from '../../services/db.service';
import {
    resultNotFound,
    resultFound,
    logErroBuscaBD,
    modelAddWhereTipoComparacao,
    modelAddWhereDataNascimento,
} from '../../utils';

import { MODEL_PRIORITY, API_CONFIG } from './../../config';

import { SIGLA_FONTE } from './../_fontes';

const fonte_dado = SIGLA_FONTE['tse'];

const fontePessoas   = MODEL_PRIORITY['tse.pessoa'].fonte;
const rankPessoas    = MODEL_PRIORITY['tse.pessoa'].rank;
const grupoPessoas   = MODEL_PRIORITY['tse.pessoa'].grupo;

export let getPessoaDetalhadoCPF_TSE_Eleitor = function (cpf: string){

    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(11), cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPF_RF AS cpf,
                        NOM_ELEITOR as nome,
                        NOM_MAE as nomeMae,
                        NOM_PAI as nomePai,
                        DAT_NASC as dataNascimento,
                        PF.Municipio as municipio,
                        PF.UF as uf,
                        NUM_INSCRICAO as tituloEleitor,
                        '${fonte_dado}' as fonte

                FROM TSE.dbo.ELEITOR
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica AS PF ON (PF.CPF = ELEITOR.CPF_RF )
                WHERE CPF_RF=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${cpf}.`, fontePessoas);
            else
                return resultFound(result, fontePessoas, rankPessoas, grupoPessoas);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cpf}.`, 'getPessoaDetalhadoCPF_TSE_Eleitor'); });
};

export let getPessoaSimplificadoCPF_TSE_Eleitor = function (cpf: string){
    return connection.getConn().request()
        .input('cpf_parameter', mssql.Char(11), cpf)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPF_RF AS cpf,
                        NOM_ELEITOR as nome,
                        NOM_MAE as nomeMae,
                        NOM_PAI as nomePai,
                        DAT_NASC as dataNascimento,
                        PF.Municipio as municipio,
                        PF.UF as uf,
                        '${fonte_dado}' as fonte

                FROM TSE.dbo.ELEITOR
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica AS PF ON (PF.CPF = ELEITOR.CPF_RF )
                WHERE ELEITOR.CPF_RF=@cpf_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${cpf}.`, fontePessoas);
            else
                return resultFound(result, fontePessoas, rankPessoas, grupoPessoas);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${cpf}.`, 'getPessoaSimplificadoCPF_TSE_Eleitor');  });
};

export let getPessoaSimplificadoTitulo_TSE_Eleitor = function (titulo: string){
    return connection.getConn().request()
        .input('titulo_parameter', mssql.Char(12), titulo)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPF_RF AS cpf,
                        NOM_ELEITOR as nome,
                        NOM_MAE as nomeMae,
                        NOM_PAI as nomePai,
                        DAT_NASC as dataNascimento,
                        PF.Municipio as municipio,
                        PF.UF as uf,
                        '${fonte_dado}' as fonte

                FROM TSE.dbo.ELEITOR
                    LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica AS PF ON (PF.CPF = ELEITOR.CPF_RF )
                WHERE ELEITOR.NUM_INSCRICAO=@titulo_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa ${titulo}.`, fontePessoas);
            else
                return resultFound(result, fontePessoas, rankPessoas, grupoPessoas);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa ${titulo}.`, 'getPessoaSimplificadoTitulo_TSE_Eleitor'); });
};

export let getPessoaSimplificadoNomePai_TSE_Eleitor = function (nomePai: string,
                                                                tipoComparacao='fulltext',
                                                                dataNascimento=undefined,
                                                                dataComparacao = '<',
                                                                fazerJoinReceita= undefined){

    let condicao = modelAddWhereTipoComparacao('NOM_PAI', '@nomePai', tipoComparacao);
    let c1 = modelAddWhereDataNascimento('DAT_NASC', '@dataNascimento', dataNascimento, dataComparacao);
    condicao += (c1) ? c1 : '';

    let camposJoinReceita;
    let joinReceita;

    if (fazerJoinReceita){
        camposJoinReceita = `, PF.Municipio as municipio, PF.UF as uf, CASE
                                                                        WHEN PF.Sexo = 1 THEN 'Masculino'
                                                                        ELSE 'Feminino'
                                                                        END sexo `;
        joinReceita = `LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica AS PF ON (PF.CPF = ELEITOR.CPF_RF )`;
    } else {
        camposJoinReceita = ``;
        joinReceita = ``;
    }

    return connection.getConn().request()
        .input('nomePai', mssql.VarChar, nomePai)
        .input('dataNascimento', mssql.Date, dataNascimento)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPF_RF AS cpf,
                        NOM_ELEITOR as nome,
                        NOM_MAE as nomeMae,
                        NOM_PAI as nomePai,
                        DAT_NASC as dataNascimento,
                        '${fonte_dado}' as fonte
                        ${camposJoinReceita}

                FROM TSE.dbo.ELEITOR
                ${joinReceita}
                ${condicao}
                `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa com pai ${nomePai}.`, fontePessoas);
            else
                return resultFound(result, fontePessoas, rankPessoas, grupoPessoas);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa com pai ${nomePai}.`, 'getPessoaSimplificadoNomePai_TSE_Eleitor');});
};

export let getPessoaSimplificadoNomeMae_TSE_Eleitor = function (nomeMae: string,
                                                                tipoComparacao='fulltext',
                                                                dataNascimento=undefined,
                                                                dataComparacao = '<',
                                                                fazerJoinReceita = undefined){

    let condicao = modelAddWhereTipoComparacao('NOM_MAE', '@nomeMae', tipoComparacao);
    let c1 = modelAddWhereDataNascimento('DAT_NASC', '@dataNascimento', dataNascimento, dataComparacao);
    condicao += (c1) ? c1 : '';

    let camposJoinReceita;
    let joinReceita;
    if (fazerJoinReceita) {
        camposJoinReceita = `, PF.Municipio as municipio, PF.UF as uf, CASE
                                                                        WHEN PF.Sexo = 1 THEN 'Masculino'
                                                                        ELSE 'Feminino'
                                                                        END sexo `;
        joinReceita = `LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica AS PF ON (PF.CPF = ELEITOR.CPF_RF )`;
    } else {
        camposJoinReceita = ``;
        joinReceita = ``;
    }

    return connection.getConn().request()
        .input('nomeMae', mssql.VarChar, nomeMae)
        .input('dataNascimento', mssql.Date, dataNascimento)
        .query(`SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                        CPF_RF AS cpf,
                        NOM_ELEITOR as nome,
                        NOM_MAE as nomeMae,
                        NOM_PAI as nomePai,
                        DAT_NASC as dataNascimento,
                        '${fonte_dado}' as fonte
                        ${camposJoinReceita}

                FROM TSE.dbo.ELEITOR
                ${joinReceita}
                ${condicao}
                `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a pessoa com mãe ${nomeMae}.`, fontePessoas);
            else
                return resultFound(result, fontePessoas, rankPessoas, grupoPessoas);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pela pessoa com mãe ${nomeMae}.`, 'getPessoaSimplificadoNomeMae_TSE_Eleitor');});
};
