import * as mssql from 'mssql';

import { BenchmarkService } from './../../services/benchmark.service';
import { connection } from '../../services/db.service';
import { MODEL_PRIORITY } from './../../config';

import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD,
    printTempoExecucao
} from '../../utils';

export let getNodesResponsavelEmpresaCNPJ = function (listaCnpj: Array<string>, tipobusca: string = 'completa'){

    if (tipobusca !== 'responsavel' && tipobusca !== 'completa') { return []; }
    if (listaCnpj.length === 0) { return []; }

    const lista = "'" + listaCnpj.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH PESSOA_RESPONSAVEL_SEM_PAI(CPF, NOME, NomeMae, DataNascimento, Sexo, Municipio, UF)
            AS
            (
                SELECT CPF, NOME, NomeMae, DataNascimento, UPPER(Sexo.Descricao), PF.Municipio, PF.UF
                FROM ReceitaNovo.dbo.PessoaJuridica PJ
                    INNER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (PJ.CpfResponsavel = PF.cpf)
                    INNER JOIN ReceitaNovo.dbo.Sexo ON (PF.Sexo = Sexo.SexoID)
                WHERE PJ.CNPJ IN (${lista})
            )
            ,PESSOA_RESPONSAVEL(CPF, NOME, NomeMae, NomePai, DataNascimento, Sexo, Municipio, UF)
            AS
            (
                SELECT A.CPF, A.NOME, A.NomeMae, E.NOM_PAI, A.DataNascimento, A.Sexo, A.Municipio, A.UF
                FROM PESSOA_RESPONSAVEL_SEM_PAI A
                    LEFT OUTER JOIN TSE.dbo.ELEITOR E ON (A.CPF = E.CPF_RF)
            )

            SELECT CPF as id, 'pessoa' as entidade, CPF as cpf, NOME as nome, NomeMae as nomeMae, NomePai as nomePai, DataNascimento as dataNascimento, Sexo as sexo, Municipio as municipio, UF as uf
            FROM PESSOA_RESPONSAVEL
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getNodesResponsavelEmpresaCNPJ');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${listaCnpj}.`, 'getNodesResponsavelEmpresaCNPJ'); });
};

export let getEdgesResponsavelEmpresaCNPJ = function (listaCnpj: Array<string>, tipobusca: string = 'completa') {

    if (tipobusca !== 'responsavel' && tipobusca !== 'completa') { return []; }
    if (listaCnpj.length === 0) { return []; }

    const lista = "'" + listaCnpj.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            SELECT PJ.CNPJ as origem, 'responsavel' as relacao, PJ.CpfResponsavel as destino
            FROM ReceitaNovo.dbo.PessoaJuridica PJ
            WHERE PJ.CNPJ IN (${lista}) AND PJ.CpfResponsavel <> '00000000000'
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getEdgesResponsavelEmpresaCNPJ');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${listaCnpj}.`, 'getEdgesResponsavelEmpresaCNPJ'); });
};

export let getNodesEnderecosEmpresaCNPJ = function (listaCnpj: Array<string>, tipobusca: string = 'completa'){

    if (tipobusca !== 'enderecos' && tipobusca !== 'completa') { return []; }
    if (listaCnpj.length === 0) { return []; }

    const lista = "'" + listaCnpj.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH ENDERECOS(TIPOLOGRADOURO, LOGRADOURO, NUMERO, COMPLEMENTO, BAIRRO, CEP, MUNICIPIO, UF)
            AS
            (
                SELECT TRIM(DescricaoTipoLogradouro), TRIM(Logradouro), TRIM(Numero), TRIM(Complemento),
                    TRIM(Bairro), TRIM(Cep), TRIM(Municipio), TRIM(UF)
                FROM ReceitaNovo.dbo.PessoaJuridica
                WHERE CNPJ IN (${lista})
            )
            SELECT CONCAT(LOGRADOURO,'-', NUMERO) as id, 'endereco' as entidade, TIPOLOGRADOURO as tipoLogradouro,
                LOGRADOURO as logradouro, NUMERO as numero, COMPLEMENTO as complemento, BAIRRO as bairro,
                CEP as cep, MUNICIPIO as municipio, UF as uf
            FROM ENDERECOS
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getNodesEnderecosEmpresaCNPJ');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${listaCnpj}.`, 'getNodesEnderecosEmpresaCNPJ'); });
};

export let getEdgesEnderecosEmpresaCNPJ = function (listaCnpj: Array<string>, tipobusca: string = 'completa') {

    if (tipobusca !== 'enderecos' && tipobusca !== 'completa') { return []; }
    if (listaCnpj.length === 0) { return []; }

    const lista = "'" + listaCnpj.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH ENDERECOS(CNPJ, LOGRADOURO, NUMERO)
            AS
            (
                SELECT TRIM(CNPJ), TRIM(Logradouro), TRIM(Numero)
                FROM ReceitaNovo.dbo.PessoaJuridica
                WHERE CNPJ IN (${lista})
            )
            SELECT CNPJ as origem, 'encontra-se' as relacao, CONCAT(LOGRADOURO, '-', NUMERO) as destino
            FROM ENDERECOS
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getEdgesEnderecosEmpresaCNPJ');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${listaCnpj}.`, 'getEdgesEnderecosEmpresaCNPJ'); });
};

export let getNodesSociosPJEmpresaCNPJ = function (listaCnpj: Array<string>, tipobusca: string = 'completa'){

    if (tipobusca !== 'sociospj' && tipobusca !== 'socios' && tipobusca !== 'completa') { return []; }
    if (listaCnpj.length === 0) { return []; }

    const lista = "'" + listaCnpj.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH EMPRESAS_ALVO(CNPJ, RAZAOSOCIAL, NOMEFANTASIA, DTINICIOATIVIDADE, DTSITUACAOCADASTRAL, MUNICIPIO, UF)
            AS
            (
                SELECT PJ.CNPJ, PJ.RazaoSocial, PJ.NomeFantasia, PJ.DataInicioAtividade, PJ.DataSituacaoCadastral, PJ.Municipio, PJ.UF
                FROM ReceitaNovo.dbo.PessoaJuridica PJ
                WHERE CNPJ IN (${lista})
            )
            ,TEM_SOCIEDADE(CNPJ, RAZAOSOCIAL, NOMEFANTASIA, DTINICIOATIVIDADE, DTSITUACAOCADASTRAL, MUNICIPIO, UF)
            AS
            (
                SELECT PJ.CNPJ, PJ.RazaoSocial, PJ.NomeFantasia, PJ.DataInicioAtividade, PJ.DataSituacaoCadastral, PJ.Municipio, PJ.UF
                FROM EMPRESAS_ALVO E
                    INNER JOIN ReceitaNovo.dbo.Socio S ON (E.CNPJ = S.CpfSocio)
                    INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (S.Cnpj = PJ.CNPJ)
                WHERE S.IdentificadorSocio=1
            )
            ,SOCIOS(CNPJ, RAZAOSOCIAL, NOMEFANTASIA, DTINICIOATIVIDADE, DTSITUACAOCADASTRAL, MUNICIPIO, UF)
            AS
            (
                SELECT PJ.CNPJ, PJ.RazaoSocial, PJ.NomeFantasia, PJ.DataInicioAtividade, PJ.DataSituacaoCadastral, PJ.Municipio, PJ.UF
                FROM EMPRESAS_ALVO E
                    INNER JOIN ReceitaNovo.dbo.Socio S ON (E.CNPJ = S.CNPJ)
                    INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (S.CpfSocio = PJ.CNPJ)
                WHERE S.IdentificadorSocio=1
            )
            ,EMPRESAS(CNPJ, RAZAOSOCIAL, NOMEFANTASIA, DTINICIOATIVIDADE, DTSITUACAOCADASTRAL, MUNICIPIO, UF)
            AS
            (
                SELECT * FROM EMPRESAS_ALVO
                UNION
                SELECT * FROM TEM_SOCIEDADE
                UNION
                SELECT * FROM SOCIOS
            )

            SELECT CNPJ as id, 'empresa' as entidade, CNPJ as cnpj, RAZAOSOCIAL as razaoSocial, NOMEFANTASIA as nomeFantasia, cast(DTINICIOATIVIDADE as date) as dataInicioAtividade, MUNICIPIO as municipio, UF as uf
            FROM EMPRESAS
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getNodesSociosPJEmpresaCNPJ');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${listaCnpj}.`, 'getNodesSociosPJEmpresaCNPJ'); });
};

export let getEdgesSociosPJEmpresaCNPJ = function (listaCnpj: Array<string>, tipobusca: string = 'completa') {

    if (tipobusca !== 'sociospj' && tipobusca !== 'socios' && tipobusca !== 'completa') { return []; }

    const lista = "'" + listaCnpj.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH EMPRESAS_ALVO(CNPJ, RAZAOSOCIAL, NOMEFANTASIA, DTINICIOATIVIDADE, DTSITUACAOCADASTRAL, MUNICIPIO, UF)
            AS
            (
                SELECT PJ.CNPJ, PJ.RazaoSocial, PJ.NomeFantasia, PJ.DataInicioAtividade, PJ.DataSituacaoCadastral, PJ.Municipio, PJ.UF
                FROM ReceitaNovo.dbo.PessoaJuridica PJ
                WHERE CNPJ IN (${lista})
            )
            ,TEM_SOCIEDADE(ORIGEM, RELACAO, DESTINO)
            AS
            (
                SELECT E.CNPJ as origem, 'tem participacao' as relacao, S.CNPJ as destino
                FROM EMPRESAS_ALVO E
                    INNER JOIN ReceitaNovo.dbo.Socio S ON (E.CNPJ = S.CpfSocio)
                WHERE S.IdentificadorSocio=1
            )
            ,SOCIOS(ORIGEM, RELACAO, DESTINO)
            AS
            (
                SELECT E.CNPJ as origem, 'tem como socio' as relacao, S.CpfSocio as destino
                FROM EMPRESAS_ALVO E
                    INNER JOIN ReceitaNovo.dbo.Socio S ON (E.CNPJ = S.CNPJ)
                WHERE S.IdentificadorSocio=1
            )
            ,VINCULOS(ORIGEM, RELACAO, DESTINO)
            AS
            (
                SELECT * FROM TEM_SOCIEDADE
                UNION
                SELECT * FROM SOCIOS
            )

            SELECT ORIGEM as origem, RELACAO as relacao, DESTINO as destino
            FROM VINCULOS
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getEdgesSociosPJEmpresaCNPJ');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${listaCnpj}.`, 'getEdgesSociosPJEmpresaCNPJ'); });
};

export let getNodesSociosPFEmpresaCNPJ = function (listaCnpj: Array<string>, tipobusca: string = 'completa'){

    if (tipobusca !== 'sociospf' && tipobusca !== 'socios' && tipobusca !== 'completa') { return []; }
    if (listaCnpj.length === 0) { return []; }

    const lista = "'" + listaCnpj.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH EMPRESAS_ALVO(CNPJ, RAZAOSOCIAL, NOMEFANTASIA, DTINICIOATIVIDADE, DTSITUACAOCADASTRAL, MUNICIPIO, UF)
            AS
            (
                SELECT PJ.CNPJ, PJ.RazaoSocial, PJ.NomeFantasia, PJ.DataInicioAtividade, PJ.DataSituacaoCadastral, PJ.Municipio, PJ.UF
                FROM ReceitaNovo.dbo.PessoaJuridica PJ
                WHERE CNPJ IN (${lista})
            )
            ,SOCIOS(CPF, NOME, NomeMae, NomePai, DataNascimento, Sexo, Municipio, UF)
            AS
            (
                SELECT PF.CPF, PF.NOME, PF.NomeMae, EL.NOM_PAI, PF.DataNascimento, UPPER(Sexo.Descricao), PF.Municipio, PF.UF
                FROM EMPRESAS_ALVO E
                    INNER JOIN ReceitaNovo.dbo.Socio S ON (E.CNPJ = S.CNPJ)
                    INNER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (PF.CPF = RIGHT(S.CpfSocio,11))
                    LEFT OUTER JOIN TSE.dbo.ELEITOR EL ON (PF.CPF = EL.CPF_RF)
                    INNER JOIN ReceitaNovo.dbo.Sexo ON (PF.Sexo = Sexo.SexoID)
                WHERE S.IdentificadorSocio=2
            )

            SELECT CPF as id, 'pessoa' as entidade, CPF as cpf, NOME as nome, NomeMae as nomeMae, NomePai as nomePai, DataNascimento as dataNascimento, Sexo as sexo, Municipio as municipio, UF as uf
            FROM SOCIOS
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getNodesSociosPFEmpresaCNPJ');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${listaCnpj}.`, 'getNodesSociosPFEmpresaCNPJ'); });
};

export let getEdgesSociosPFEmpresaCNPJ = function (listaCnpj: Array<string>, tipobusca: string = 'completa') {

    if (tipobusca !== 'sociospf' && tipobusca !== 'socios' && tipobusca !== 'completa') { return []; }

    const lista = "'" + listaCnpj.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH EMPRESAS_ALVO(CNPJ, RAZAOSOCIAL, NOMEFANTASIA, DTINICIOATIVIDADE, DTSITUACAOCADASTRAL, MUNICIPIO, UF)
            AS
            (
                SELECT PJ.CNPJ, PJ.RazaoSocial, PJ.NomeFantasia, PJ.DataInicioAtividade, PJ.DataSituacaoCadastral, PJ.Municipio, PJ.UF
                FROM ReceitaNovo.dbo.PessoaJuridica PJ
                WHERE CNPJ IN (${lista})
            )
            ,SOCIOS(ORIGEM, RELACAO, DESTINO)
            AS
            (
                SELECT E.CNPJ as origem, 'tem como socio' as relacao, RIGHT(S.CpfSocio,11) as destino
                FROM EMPRESAS_ALVO E
                    INNER JOIN ReceitaNovo.dbo.Socio S ON (E.CNPJ = S.CNPJ)
                WHERE S.IdentificadorSocio=2
            )

            SELECT ORIGEM as origem, RELACAO as relacao, DESTINO as destino
            FROM SOCIOS
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getEdgesSociosPFEmpresaCNPJ');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${listaCnpj}.`, 'getEdgesSociosPFEmpresaCNPJ'); });
};

export let getNodesFiliaisEmpresaCNPJ = function (listaCnpj: Array<string>, tipobusca: string = 'completa'){

    if (tipobusca !== 'filiais' && tipobusca !== 'completa') { return []; }
    if (listaCnpj.length === 0) { return []; }

    const lista = "'" + listaCnpj.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`
            ;WITH EMPRESAS_MATRIZ(CNPJ, MATRIZFILIAL)
            AS
            (
                SELECT LEFT(PJ.CNPJ, 10), PJ.IdentificadorMatrizFilial
                FROM ReceitaNovo.dbo.PessoaJuridica PJ
                WHERE CNPJ IN (${lista}) AND IdentificadorMatrizFilial = 1
            )
            ,FILIAIS(CNPJ, RAZAOSOCIAL, NOMEFANTASIA, DTINICIOATIVIDADE, DTSITUACAOCADASTRAL, MUNICIPIO, UF)
            AS
            (
                SELECT PJ.CNPJ, PJ.RazaoSocial, PJ.NomeFantasia, PJ.DataInicioAtividade, PJ.DataSituacaoCadastral, PJ.Municipio, PJ.UF
                FROM EMPRESAS_MATRIZ E
                    INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (E.CNPJ = LEFT(PJ.CNPJ, 10))
                WHERE PJ.IdentificadorMatrizFilial <> 1
            )

            SELECT CNPJ as id, 'empresa' as entidade, CNPJ as cnpj, RAZAOSOCIAL as razaoSocial, NOMEFANTASIA as nomeFantasia, cast(DTINICIOATIVIDADE as date) as dataInicioAtividade, MUNICIPIO as municipio, UF as uf
            FROM FILIAIS
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getNodesFiliaisEmpresaCNPJ');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${listaCnpj}.`, 'getNodesFiliaisEmpresaCNPJ'); });
};

export let getEdgesFiliaisEmpresaCNPJ = function (listaCnpj: Array<string>, tipobusca: string = 'completa') {

    if (tipobusca !== 'filiais' && tipobusca !== 'completa') { return []; }

    const lista = "'" + listaCnpj.join("','") + "'";
    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .query(`

            ;WITH EMPRESAS_MATRIZ(INICIO_CNPJ, CNPJ)
            AS
            (
                SELECT LEFT(PJ.CNPJ, 10), PJ.CNPJ
                FROM ReceitaNovo.dbo.PessoaJuridica PJ
                WHERE CNPJ IN (${lista}) AND IdentificadorMatrizFilial = 1
            )
            ,VINCULOS(ORIGEM, RELACAO, DESTINO)
            AS
            (
                SELECT E.CNPJ, 'matriz', PJ.CNPJ
                FROM EMPRESAS_MATRIZ E
                    INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (E.INICIO_CNPJ = LEFT(PJ.CNPJ, 10))
                WHERE PJ.IdentificadorMatrizFilial <> 1
            )

            SELECT ORIGEM as origem, RELACAO as relacao, DESTINO as destino
            FROM VINCULOS
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getEdgesFiliaisEmpresaCNPJ');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${listaCnpj}.`, 'getEdgesFiliaisEmpresaCNPJ'); });
};