import * as mssql from 'mssql';

import { BenchmarkService } from './../../services/benchmark.service';
import { connection } from '../../services/db.service';
import { MODEL_PRIORITY } from './../../config';

import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD,
    printTempoExecucao
} from '../../utils';

/**
 * Retorna as entidades 'Pessoa' que possuem esse endereco.
 * @param cpf
 * @param tipobusca
 */
export let getNodesPessoasEndereco = function (logradouro: string, numero: string, municipio: string, tipobusca: string = 'completa'){

    if (tipobusca !== 'pessoas' && tipobusca !== 'completa') { return []; }

    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .input('LOGRADOURO', mssql.VarChar(120), logradouro)
        .input('NUMERO', mssql.VarChar(12), numero)
        .input('MUNICIPIO', mssql.VarChar(50), municipio)
        .query(`
            ;WITH PESSOAS(CPF, NOME, NomeMae, DataNascimento, Sexo, Municipio, UF)
            AS
            (
                SELECT PF.CPF, NOME, NomeMae, DataNascimento, Sexo, Municipio, UF
                FROM ReceitaNovo.dbo.PessoaFisica PF
                WHERE CONTAINS(Logradouro, @LOGRADOURO)
                    AND NumeroLogradouro = @NUMERO
                    AND Municipio = @MUNICIPIO
            )
            ,DADOS_COM_PAI(CPF, NOME, NomeMae, NomePai, DataNascimento, Sexo, Municipio, UF)
            AS
            (
                SELECT P.CPF, P.NOME, P.NomeMae, E.NOM_PAI, P.DataNascimento, P.Sexo, P.Municipio, P.UF
                FROM PESSOAS P
                    LEFT OUTER JOIN TSE.dbo.ELEITOR E ON (P.CPF = E.CPF_RF)
            )

            SELECT CPF as id, 'pessoa' as entidade, CPF as cpf, NOME as nome, NomeMae as nomeMae, NomePai as nomePai, DataNascimento as dataNascimento, Sexo as sexo, Municipio as municipio, UF as uf
            FROM DADOS_COM_PAI
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getNodesPessoasEndereco');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${logradouro}.`, 'getNodesPessoasEndereco'); });
};

export let getEdgesPessoasEndereco = function (logradouro: string, numero: string, municipio: string, tipobusca: string = 'completa') {

    if (tipobusca !== 'pessoas' && tipobusca !== 'completa') { return []; }

    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .input('LOGRADOURO', mssql.VarChar(120), logradouro)
        .input('NUMERO', mssql.VarChar(12), numero)
        .input('MUNICIPIO', mssql.VarChar(50), municipio)
        .query(`
            ;WITH ENDERECOS(CPF, LOGRADOURO, NUMERO)
            AS
            (
                SELECT TRIM(CPF), TRIM(Logradouro), TRIM(NumeroLogradouro)
                FROM ReceitaNovo.dbo.PessoaFisica
                WHERE CONTAINS(Logradouro, @LOGRADOURO)
                    AND NumeroLogradouro = @NUMERO
                    AND Municipio = @MUNICIPIO
            )
            SELECT CONCAT(LOGRADOURO, '-', NUMERO) as origem, 'residencia' as relacao, cpf as destino
            FROM ENDERECOS
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getEdgesPessoasEndereco');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos edges de ${logradouro}.`, 'getEdgesPessoasEndereco'); });
};

/**
 * Retorna as entidades 'Empresa' que possuem esse endereco.
 * @param cpf
 * @param tipobusca
 */
export let getNodesEmpresasEndereco = function (logradouro: string, numero: string, municipio: string, tipobusca: string = 'completa'){

    if (tipobusca !== 'empresas' && tipobusca !== 'completa') { return []; }

    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .input('LOGRADOURO', mssql.VarChar(120), logradouro)
        .input('NUMERO', mssql.VarChar(12), numero)
        .input('MUNICIPIO', mssql.VarChar(50), municipio)
        .query(`
            ;WITH EMPRESAS(CNPJ, RAZAOSOCIAL, NOMEFANTASIA, DTINICIOATIVIDADE, DTSITUACAOCADASTRAL, MUNICIPIO, UF)
            AS
            (
                SELECT PJ.CNPJ, PJ.RazaoSocial, PJ.NomeFantasia, PJ.DataInicioAtividade, PJ.DataSituacaoCadastral, PJ.Municipio, PJ.UF
                FROM ReceitaNovo.dbo.PessoaJuridica PJ
                WHERE CONTAINS(Logradouro, @LOGRADOURO)
                    AND Numero = @NUMERO
                    AND Municipio = @MUNICIPIO
            )
            SELECT CNPJ as id, 'empresa' as entidade, CNPJ as cnpj, RAZAOSOCIAL as razaoSocial, NOMEFANTASIA as nomeFantasia, cast(DTINICIOATIVIDADE as date) as dataInicioAtividade, MUNICIPIO as municipio, UF as uf
            FROM EMPRESAS
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getNodesEmpresasEndereco');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${logradouro}.`, 'getNodesEmpresasEndereco'); });
};

export let getEdgesEmpresasEndereco = function (logradouro: string, numero: string, municipio: string, tipobusca: string = 'completa') {

    if (tipobusca !== 'empresas' && tipobusca !== 'completa') { return []; }

    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .input('LOGRADOURO', mssql.VarChar(120), logradouro)
        .input('NUMERO', mssql.VarChar(12), numero)
        .input('MUNICIPIO', mssql.VarChar(50), municipio)
        .query(`
            ;WITH ENDERECOS(CNPJ, LOGRADOURO, NUMERO)
            AS
            (
                SELECT TRIM(CNPJ), TRIM(Logradouro), TRIM(Numero)
                FROM ReceitaNovo.dbo.PessoaJuridica PJ
                WHERE CONTAINS(Logradouro, @LOGRADOURO)
                    AND Numero = @NUMERO
                    AND Municipio = @MUNICIPIO
            )
            SELECT CONCAT(LOGRADOURO, '-', NUMERO) as origem, 'abriga' as relacao, cnpj as destino
            FROM ENDERECOS
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getEdgesEmpresasEndereco');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos edges de ${logradouro}.`, 'getEdgesEmpresasEndereco'); });
};
