import * as mssql from 'mssql';

import { BenchmarkService } from './../../services/benchmark.service';
import { connection } from '../../services/db.service';
import { MODEL_PRIORITY } from './../../config';

import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD,
    printTempoExecucao
} from '../../utils';

export let getNodesOrgaoPublicoCdUgestoraMunicipal = function (cdUgestora: string, esfera: string, tipobusca: string = 'completa'){

    if (tipobusca !== 'orgaospublicos' && tipobusca !== 'completa' && esfera.toUpperCase() !== 'M') { return []; }

    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .input('CDUGESTORA', mssql.Char(6), cdUgestora)
        .query(`
            ;WITH UGESTORA(CDUGESTORA, UGESTORA, MUNICIPIO)
            AS
            (
                SELECT TOP 1 cd_Ugestora, UPPER(de_Ugestora), UPPER(M.no_Municipio)
                FROM SAGRES_CONTROLE.dbo.Codigo_Ugestora UG
                    LEFT OUTER JOIN SAGRES_CONTROLE.dbo.Codigo_Municipios M ON (M.cd_Municipio = UG.cd_Municipio)
                WHERE cd_Ugestora=@CDUGESTORA
                ORDER BY dt_Ano DESC
            )

            SELECT CAST(CDUGESTORA as varchar) as id, 'orgaopublico' as entidade, UGESTORA as uGestora, 'M' as esfera, MUNICIPIO as municipio
            FROM UGESTORA
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getNodesOrgaoPublicoCdUgestoraMunicipal');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${cdUgestora}.`, 'getNodesOrgaoPublicoCdUgestoraMunicipal'); });
};

export let getNodesEmpresasPagasCdUgestoraMunicipal = function (cdUgestora: string, anoInicial: string, anoFinal: string, tipobusca: string = 'completa'){

    if (tipobusca !== 'empresaspagas' && tipobusca !== 'completa') { return []; }

    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .input('CDUGESTORA', mssql.Char(6), cdUgestora)
        .input('ANO_INICIAL', mssql.Char(4), anoInicial)
        .input('ANO_FINAL', mssql.Char(4), anoFinal)
        .query(`
            ;WITH EMPRESAS_PAGAS(CNPJ)
            AS
            (
                SELECT DISTINCT E.cd_Credor
                FROM SAGRES_MUNICIPAL.dbo.Empenhos E
                    INNER JOIN SAGRES_MUNICIPAL.dbo.PAGAMENTOS P ON (E.cd_Ugestora = P.cd_Ugestora) AND (E.dt_Ano = P.dt_Ano) AND (E.cd_UnidOrcamentaria = P.cd_UnidOrcamentaria) AND (E.nu_Empenho = P.nu_Empenho)
                WHERE E.cd_UGestora=@CDUGESTORA and E.dt_Ano BETWEEN @ANO_INICIAL AND @ANO_FINAL AND tp_Credor=2
            )
            ,INFO_EMPRESAS(CNPJ, RAZAOSOCIAL, NOMEFANTASIA, DTINICIOATIVIDADE, MUNICIPIO, UF)
            AS
            (
                SELECT PJ.CNPJ, PJ.RazaoSocial, PJ.NomeFantasia, CAST(PJ.DataInicioAtividade as date), PJ.Municipio, PJ.UF
                FROM EMPRESAS_PAGAS E
                    INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (E.CNPJ = PJ.CNPJ)
            )

            SELECT CNPJ as id, 'empresa' as entidade, CNPJ as cnpj, RAZAOSOCIAL as razaoSocial, NOMEFANTASIA as nomeFantasia, DTINICIOATIVIDADE as dataInicioAtividade, MUNICIPIO as municipio, UF as uf
            FROM INFO_EMPRESAS
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getNodesEmpresasPagasCdUgestoraMunicipal');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${cdUgestora}.`, 'getNodesEmpresasPagasCdUgestoraMunicipal'); });
};

export let getEdgesEmpresasPagasCdUgestoraMunicipal = function (cdUgestora: string, anoInicial: string, anoFinal: string, tipobusca: string = 'completa') {

    if (tipobusca !== 'empresaspagas' && tipobusca !== 'completa') { return []; }

    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .input('CDUGESTORA', mssql.Char(6), cdUgestora)
        .input('ANO_INICIAL', mssql.Char(4), anoInicial)
        .input('ANO_FINAL', mssql.Char(4), anoFinal)
        .query(`
            ;WITH EMPRESAS_PAGAS(CDUGESTORA, CNPJ, ANO, EMPENHADO, PAGO)
            AS
            (
                SELECT E.cd_ugestora, E.cd_Credor, E.dt_ano, SUM(E.vl_Empenho), SUM(P.vl_Pagamento)
                FROM SAGRES_MUNICIPAL.dbo.Empenhos E
                    INNER JOIN SAGRES_MUNICIPAL.dbo.PAGAMENTOS P ON (E.cd_Ugestora = P.cd_Ugestora) AND (E.dt_Ano = P.dt_Ano) AND (E.cd_UnidOrcamentaria = P.cd_UnidOrcamentaria) AND (E.nu_Empenho = P.nu_Empenho)
                WHERE E.cd_UGestora=@CDUGESTORA and E.dt_Ano BETWEEN @ANO_INICIAL AND @ANO_FINAL AND tp_Credor=2
                GROUP BY E.cd_ugestora, E.cd_Credor, E.dt_Ano
            )

            SELECT CONCAT(CDUGESTORA, '-', ANO, '-', CNPJ) as id, CAST(CDUGESTORA as varchar) as origem, CNPJ as destino, 'PAGOU' as relacao, ANO as ano, EMPENHADO as empenhado, PAGO as pago
            FROM EMPRESAS_PAGAS
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getEdgesEmpresasPagasCdUgestoraMunicipal');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${cdUgestora}.`, 'getEdgesEmpresasPagasCdUgestoraMunicipal'); });
};

export let getNodesServidoresCdUgestoraMunicipal = function (cdUgestora: string, anoInicial: string, anoFinal: string, tipobusca: string = 'completa'){

    if (tipobusca !== 'servidores' && tipobusca !== 'completa') { return []; }

    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .input('CDUGESTORA', mssql.Char(6), cdUgestora)
        .input('ANO_INICIAL', mssql.Char(4), anoInicial)
        .input('ANO_FINAL', mssql.Char(4), anoFinal)
        .query(`
            ;WITH SERVIDORES(CPF)
            AS
            (
                SELECT FP.cd_CPF
                FROM SAGRES_MUNICIPAL.dbo.FolhaPagamento FP
                WHERE FP.cd_UGestora=@CDUGESTORA AND FP.dt_Ano BETWEEN @ANO_INICIAL AND @ANO_FINAL

                UNION

                SELECT FP.cd_CPF
                FROM SAGRES_MUNICIPAL.dbo.FolhaPagamento_N FP
                WHERE FP.cd_UGestora=@CDUGESTORA AND FP.dt_Ano BETWEEN @ANO_INICIAL AND @ANO_FINAL

                UNION

                SELECT FP.cd_CPF
                FROM SAGRES_MUNICIPAL.dbo.FolhaPagamento_N_2017 FP
                WHERE FP.cd_UGestora=@CDUGESTORA AND FP.dt_Ano BETWEEN @ANO_INICIAL AND @ANO_FINAL
            )
            ,PESSOA(CPF, NOME, NomeMae, NomePai, DataNascimento, Sexo, Municipio, UF)
            AS
            (
                SELECT PF.CPF, PF.Nome, PF.NomeMae, E.NOM_PAI, PF.DataNascimento, UPPER(Sexo.Descricao), PF.Municipio, PF.UF
                FROM SERVIDORES S
                    INNER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (S.CPF = PF.CPF)
                    INNER JOIN ReceitaNovo.dbo.Sexo ON (PF.Sexo = Sexo.SexoID)
                    LEFT OUTER JOIN TSE.dbo.ELEITOR E ON (S.CPF = E.CPF_RF)
            )

            SELECT CPF as id, 'pessoa' as entidade, CPF as cpf, NOME as nome, NomeMae as nomeMae, NomePai as nomePai, DataNascimento as dataNascimento, Sexo as sexo, Municipio as municipio, UF as uf
            FROM PESSOA
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getNodesServidoresCdUgestoraMunicipal');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${cdUgestora}.`, 'getNodesServidoresCdUgestoraMunicipal'); });
};

export let getEdgesServidoresCdUgestoraMunicipal = function (cdUgestora: string, anoInicial: string, anoFinal: string, tipobusca: string = 'completa') {

    if (tipobusca !== 'servidores' && tipobusca !== 'completa') { return []; }

    const tempoInicial = new BenchmarkService();

    return connection.getConn().request()
        .input('CDUGESTORA', mssql.Char(6), cdUgestora)
        .input('ANO_INICIAL', mssql.Char(4), anoInicial)
        .input('ANO_FINAL', mssql.Char(4), anoFinal)
        .query(`
            ;WITH SERVIDORES(CPF, CDCARGO, CARGO, CDUGESTORA)
            AS
            (
                SELECT FP.cd_CPF, FP.cd_Cargo, C.de_Cargo, FP.cd_UGestora
                FROM SAGRES_MUNICIPAL.dbo.FolhaPagamento FP
                OUTER APPLY (
                    SELECT TOP 1 de_Cargo
                    FROM SAGRES_MUNICIPAL.dbo.Cargos C
                    WHERE C.cd_Cargo=FP.cd_Cargo AND C.cd_UGestora=FP.cd_UGestora
                    ORDER BY C.dt_Ano DESC
                ) C
                WHERE FP.cd_UGestora=@CDUGESTORA AND FP.dt_Ano BETWEEN @ANO_INICIAL AND @ANO_FINAL

                UNION

                SELECT FP.cd_CPF, FP.cd_Cargo, C.de_Cargo, FP.cd_UGestora
                FROM SAGRES_MUNICIPAL.dbo.FolhaPagamento_N FP
                OUTER APPLY (
                    SELECT TOP 1 de_Cargo
                    FROM SAGRES_MUNICIPAL.dbo.Cargos C
                    WHERE C.cd_Cargo=FP.cd_Cargo AND C.cd_UGestora=FP.cd_UGestora
                    ORDER BY C.dt_Ano DESC
                ) C
                WHERE FP.cd_UGestora=@CDUGESTORA AND FP.dt_Ano BETWEEN @ANO_INICIAL AND @ANO_FINAL

                UNION

                SELECT FP.cd_CPF, FP.cd_Cargo, C.de_Cargo, FP.cd_UGestora
                FROM SAGRES_MUNICIPAL.dbo.FolhaPagamento_N_2017 FP
                OUTER APPLY (
                    SELECT TOP 1 de_Cargo
                    FROM SAGRES_MUNICIPAL.dbo.Cargos C
                    WHERE C.cd_Cargo=FP.cd_Cargo AND C.cd_UGestora=FP.cd_UGestora
                    ORDER BY C.dt_Ano DESC
                ) C
                WHERE FP.cd_UGestora=@CDUGESTORA AND FP.dt_Ano BETWEEN @ANO_INICIAL AND @ANO_FINAL
            )

            SELECT CONCAT(CDUGESTORA, '-', CDCARGO, '-', CPF) as id, CAST(CDUGESTORA as varchar) as origem, CPF as destino, CARGO as relacao
            FROM SERVIDORES
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getEdgesServidoresCdUgestoraMunicipal');

            return result.recordset.map(resultado => trataResultado(resultado));
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos nodes de ${cdUgestora}.`, 'getEdgesServidoresCdUgestoraMunicipal'); });
};
