import * as mssql from 'mssql';

import { connection } from './../../services/db.service';
import { BenchmarkService } from './../../services/benchmark.service';
import { MODEL_PRIORITY } from './../../config';

import {
    resultNotFound,
    resultFound,
    logErroBuscaBD,
    trataResultadosCruzamentoSisobi,
    insereAtributoTipologia,
    printTempoExecucao
} from './../../utils';

const fonte = MODEL_PRIORITY['tipologias.cacafantasmas'].fonte;
const rank  = MODEL_PRIORITY['tipologias.cacafantasmas'].rank;
const grupo = MODEL_PRIORITY['tipologias.cacafantasmas'].grupo;

const tipologias = {
    'fp_sisobi':                    { codigo: 't1', peso: 5},
    'empenho_sisobi':               { codigo: 't2', peso: 5 },

    'fp_enderecosrf':               { codigo: 't3', peso: 2 },
    'fp_enderecosre':               { codigo: 't4', peso: 1 },

    'fp_sociopj':                   { codigo: 't5', peso: 2 },
    'fp_responsavelpj':             { codigo: 't6', peso: 2 },

    'fp_extravinculo_publico':      { codigo: 't7', peso: 4 },
    'fp_caged':                     { codigo: 't8', peso: 2 },

    'fp_analfabetos':               { codigo: 't9', peso: 5 },
    'fp_bolsafamilia':              { codigo: 't10', peso: 5 },
    'fp_bolsafamilia_veiculos':     { codigo: 't11', peso: 5 },

    'fp_siaf':                      { codigo: 't12', peso: 1 },

    'fp_tipologia_tcu':             { codigo: 't13', peso: 0.2 },
    'fp_tipologia_tcu_doadores':    { codigo: 't14', peso: 10 },
    'fp_rf_cpf_inexistente':        { codigo: 't15', peso: 100 },

    'fp_filiacaoregular':           { codigo: 't16', peso: 5 },
    'fp_filiacaocancelada':         { codigo: 't17', peso: 5 },

    'fp_cadicon':                   { codigo: 't18', peso: 5 },
    'fp_ceis':                      { codigo: 't19', peso: 5 },



    // 'fp_sociopj_empenho':           { codigo: 't7', peso: 10 },
    // 'fp_veiculos':                  { codigo: 't20', peso: 1 },
    // 'fp_sociopj_caged':             { codigo: 't21', peso: 50 },
}

export let getTipologiaSimplificadaFP_SISOBI = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.VarChar, dtInicio)
        .input('DT_FINAL', mssql.VarChar, dtFim)
        .input('UGESTORA', mssql.VarChar, ugestora)
        .input('PESO_TIPOLOGIA', mssql.Int, tipologias['fp_sisobi'].peso)
        .query(`
            ;WITH SERVIDORES(CPF, MESANO)
            AS
            (
                SELECT DISTINCT FP.cd_CPF, FP.MESANO_ORDENADO
                FROM MISC.dbo.FolhaPagamento_Servidores FP
                WHERE FP.cd_UGestora = @UGESTORA
                    AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
            )

            ,SERVIDORES_COM_OBITO(CPF, NOMEFALECIDO, DATAOBITO, QTD, PIORPAGAMENTO)
            AS
            (
                SELECT S.CPF, O.Falecido, O.Obito, COUNT(*) as QTD, MAX(DATEDIFF(M, O.Obito, S.MESANO + '01'))
                FROM Sisobi.dbo.Obito O
                    INNER JOIN SERVIDORES S ON (S.CPF = O.CPF_RF)
                WHERE S.MESANO+'01' > O.Obito
                GROUP BY S.CPF, O.Falecido, O.Obito
            )
            , RESULTADO(CPF, NOME, NOMEFALECIDO, DATAOBITO, SCORE)
            AS
            (
                SELECT S.CPF, NOME, NOMEFALECIDO, DATAOBITO, QTD * @PESO_TIPOLOGIA + PIORPAGAMENTO/12
                FROM SERVIDORES_COM_OBITO  S
                    INNER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (S.CPF = PF.CPF)
            )

            SELECT CPF as cpf, NOME as nome, NOMEFALECIDO as nomeFalecido, SCORE as score
            FROM RESULTADO
            ORDER BY SCORE DESC
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaSimplificadaFP_SISOBI');

            result.recordset = trataResultadosCruzamentoSisobi(result.recordset);
            result.recordset = insereAtributoTipologia(result.recordset, tipologias['fp_sisobi'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias FP-SISOBI para ${ugestora}.`, fonte);
            else{
                return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaSimplificadaFP_SISOBI'); });
}

export let getTipologiaSimplificadaFP_ENDERECOSRF = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.VarChar, dtInicio)
        .input('DT_FINAL', mssql.VarChar, dtFim)
        .input('UGESTORA', mssql.VarChar, ugestora)
        .input('PESO_TIPOLOGIA', mssql.Int, tipologias['fp_enderecosrf'].peso)
        .query(`
            ;WITH SERVIDORES(CPF)
            AS
            (
                SELECT DISTINCT FP.cd_CPF
                FROM MISC.dbo.FolhaPagamento_Servidores FP
                WHERE FP.cd_UGestora = @UGESTORA
                    AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
            )

            ,UG(CD_UGESTORA, MUNICIPIO, MICROREGIAO)
            AS
            (
                SELECT DISTINCT G.cd_Ugestora, M.no_Municipio, MMM.MICROREGIAO
                FROM SAGRES_CONTROLE.dbo.Codigo_Ugestora G
                    INNER JOIN SAGRES_CONTROLE.dbo.Codigo_Municipios M ON (G.cd_Municipio = M.cd_Municipio)
                    LEFT OUTER JOIN SISPESQUISA.dbo.MUNICIPIO_MESOMICROREGIAO MMM ON (MMM.MUNICIPIO COLLATE Latin1_General_CI_AI = M.no_Municipio COLLATE Latin1_General_CI_AI)
            )

            ,ENDERECOS_SERVIDORES(CPF, NOME, S_MUNICIPIO, S_UF, S_MICRORREGIAO, UG_MICRORREGIAO, CUSTO)
            AS
            (
                SELECT  S.CPF, PF.Nome, PF.Municipio, PF.UF, MMM.MICROREGIAO, UG.MICROREGIAO,
                        CASE
                            WHEN ISNULL(CUSTO.custo, -1) = -1 THEN 20
                            ELSE Custo.custo + 1
                        END as custo

                FROM ReceitaNovo.dbo.PessoaFisica PF
                    INNER JOIN SERVIDORES S ON (PF.CPF = S.CPF)
                    LEFT OUTER JOIN SISPESQUISA.dbo.MUNICIPIO_MESOMICROREGIAO MMM ON (MMM.MUNICIPIO COLLATE Latin1_General_CI_AI = PF.MUNICIPIO AND PF.UF = 'PB')
                    LEFT OUTER JOIN UG ON (UG.CD_UGESTORA = @UGESTORA)
                    LEFT OUTER JOIN SISPESQUISA.dbo.tabela_custo_microregioes CUSTO ON (CUSTO.origem = MMM.MICROREGIAO COLLATE Latin1_General_CI_AI AND CUSTO.destino = UG.MICROREGIAO COLLATE Latin1_General_CI_AI)
                WHERE CUSTO.custo > 0 OR CUSTO.custo IS NULL
            )

            SELECT CPF as cpf, NOME as nome, CUSTO * @PESO_TIPOLOGIA as score
            FROM ENDERECOS_SERVIDORES
            ORDER BY 3 DESC
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaSimplificadaFP_ENDERECOSRF');
            result.recordset = insereAtributoTipologia(result.recordset, tipologias['fp_enderecosrf'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias FP-EnderecosRF para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaSimplificadaFP_ENDERECOSRF'); });
}

export let getTipologiaSimplificadaFP_ENDERECOSRE = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.VarChar, dtInicio)
        .input('DT_FINAL', mssql.VarChar, dtFim)
        .input('UGESTORA', mssql.VarChar, ugestora)
        .input('PESO_TIPOLOGIA', mssql.Int, tipologias['fp_enderecosre'].peso)
        .query(`
           ;WITH SERVIDORES(CPF)
            AS
            (
                SELECT DISTINCT FP.cd_CPF
                FROM MISC.dbo.FolhaPagamento_Servidores FP
                WHERE FP.cd_UGestora = @UGESTORA
                    AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
            )

            ,UG(CD_UGESTORA, MUNICIPIO, MICROREGIAO)
            AS
            (
                SELECT DISTINCT G.cd_Ugestora, M.no_Municipio, MMM.MICROREGIAO
                FROM SAGRES_CONTROLE.dbo.Codigo_Ugestora G
                    INNER JOIN SAGRES_CONTROLE.dbo.Codigo_Municipios M ON (G.cd_Municipio = M.cd_Municipio)
                    LEFT OUTER JOIN SISPESQUISA.dbo.MUNICIPIO_MESOMICROREGIAO MMM ON (MMM.MUNICIPIO COLLATE Latin1_General_CI_AI = M.no_Municipio COLLATE Latin1_General_CI_AI)
            )

            ,ENDERECOS_SERVIDORES(CPF, S_MUNICIPIO, S_UF, S_MICRORREGIAO, UG_MICRORREGIAO, CUSTO)
            AS
            (
                SELECT  S.CPF, E.Municipio, E.UF, MMM.MICROREGIAO, UG.MICROREGIAO,
                        CASE
                            WHEN ISNULL(CUSTO.custo, -1) = -1 THEN 20
                            ELSE Custo.custo + 1
                        END as custo

                FROM SERVIDORES S
                    INNER JOIN SISPESQUISA.sispesquisa.enderecos E ON (S.CPF COLLATE Latin1_General_CI_AI = E.cpf_cnpj)
                    LEFT OUTER JOIN SISPESQUISA.dbo.MUNICIPIO_MESOMICROREGIAO MMM ON (MMM.MUNICIPIO COLLATE Latin1_General_CI_AI = E.MUNICIPIO AND E.UF = 'PB')
                    LEFT OUTER JOIN UG ON (UG.CD_UGESTORA = @UGESTORA)
                    LEFT OUTER JOIN SISPESQUISA.dbo.tabela_custo_microregioes CUSTO ON (CUSTO.origem = MMM.MICROREGIAO COLLATE Latin1_General_CI_AI AND CUSTO.destino = UG.MICROREGIAO COLLATE Latin1_General_CI_AI)
                WHERE (CUSTO.custo > 0 OR CUSTO.custo IS NULL)
                    AND NOT(E.UF = 'PB' AND MMM.MICROREGIAO IS NULL )
            )

            ,RESULTADO(CPF, MICRORREGIAO, UF, CUSTO, QTD)
            AS
            (
                SELECT CPF, S_MICRORREGIAO, S_UF, CUSTO, COUNT(*) AS QTD
                FROM ENDERECOS_SERVIDORES E
                GROUP BY CPF, S_MUNICIPIO, S_MICRORREGIAO, S_UF, CUSTO
            )

            SELECT R.CPF as cpf, PF.Nome as nome, SUM(CUSTO * QTD * @PESO_TIPOLOGIA) as score
            FROM RESULTADO R
                INNER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (R.CPF = PF.CPF)
            GROUP BY R.CPF, PF.Nome
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaSimplificadaFP_ENDERECOSRE');

            result.recordset = insereAtributoTipologia(result.recordset, tipologias['fp_enderecosre'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias FP-EnderecosRE para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaSimplificadaFP_ENDERECOSRE'); });
}

export let getTipologiaSimplificadaEmpenhos_SISOBI = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.VarChar, dtInicio)
        .input('DT_FINAL', mssql.VarChar, dtFim)
        .input('UGESTORA', mssql.VarChar, ugestora)
        .input('PESO_TIPOLOGIA', mssql.Int, tipologias['empenho_sisobi'].peso)
        .query(`
            ;WITH EMPENHOS_COM_OBITO(CPF, NOME, NOMEFALECIDO, DATAOBITO, QTD, PIOREMPENHO)
            AS
            (
                SELECT RIGHT(E.cd_Credor,11), E.no_Credor, O.Falecido, O.Obito, COUNT(*) as QTD, MAX(DATEDIFF(M, O.Obito, E.dt_Empenho))
                FROM SAGRES_MUNICIPAL.dbo.Empenhos E
                    INNER JOIN SAGRES_MUNICIPAL.dbo.PAGAMENTOS P ON (E.cd_Ugestora = P.cd_Ugestora) AND (E.dt_Ano = P.dt_Ano) AND (E.cd_UnidOrcamentaria = P.cd_UnidOrcamentaria) AND (E.nu_Empenho = P.nu_Empenho)
                    INNER JOIN Sisobi.dbo.Obito O ON (RIGHT(E.cd_Credor, 11) = O.CPF_RF)

                WHERE E.cd_UGestora = @UGESTORA
                    AND E.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
                    AND E.tp_Credor = 1
                    AND E.dt_Empenho > O.Obito
                    AND O.CPF_RF <> '00000000000'
                    AND E.cd_Credor <> '00000000000191'

                GROUP BY E.cd_Credor, E.no_Credor, O.Falecido, O.Obito
            )

            , RESULTADO(CPF, NOME, NOMEFALECIDO, DATAOBITO, SCORE)
            AS
            (
                SELECT CPF, NOME, NOMEFALECIDO, DATAOBITO, QTD * @PESO_TIPOLOGIA + PIOREMPENHO/12
                FROM EMPENHOS_COM_OBITO
            )

            SELECT CPF as cpf, NOME as nome, NOMEFALECIDO as nomeFalecido, SCORE as score
            FROM RESULTADO
            ORDER BY SCORE DESC
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaSimplificadaEmpenhos_SISOBI');

            result.recordset = trataResultadosCruzamentoSisobi(result.recordset);
            result.recordset = insereAtributoTipologia(result.recordset, tipologias['empenho_sisobi'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias Empenhos-SISOBI para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaSimplificadaEmpenhos_SISOBI'); });
}

export let getTipologiaSimplificadaFP_SocioPJ = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.VarChar, dtInicio)
        .input('DT_FINAL', mssql.VarChar, dtFim)
        .input('UGESTORA', mssql.VarChar, ugestora)
        .input('PESO_TIPOLOGIA', mssql.Int, tipologias['fp_sociopj'].peso)
        .query(`
            ;WITH SERVIDORES(CPF)
            AS
            (
                SELECT DISTINCT FP.cd_CPF
                FROM MISC.dbo.FolhaPagamento_Servidores FP
                WHERE FP.cd_UGestora = @UGESTORA
                        AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
            )

            ,UG(CD_UGESTORA, MUNICIPIO, MICROREGIAO)
            AS
            (
                SELECT DISTINCT G.cd_Ugestora, M.no_Municipio, MMM.MICROREGIAO
                FROM SAGRES_CONTROLE.dbo.Codigo_Ugestora G
                    INNER JOIN SAGRES_CONTROLE.dbo.Codigo_Municipios M ON (G.cd_Municipio = M.cd_Municipio)
                    LEFT OUTER JOIN SISPESQUISA.dbo.MUNICIPIO_MESOMICROREGIAO MMM ON (MMM.MUNICIPIO COLLATE Latin1_General_CI_AI = M.no_Municipio COLLATE Latin1_General_CI_AI)
            )

            ,SERVIDORES_SOCIOS(CPF, CNPJ, RAZAOSOCIAL, MUNICIPIO, UF, MICRORREGIAO, CUSTO)
            AS
            (
                SELECT DISTINCT
                    SERV.CPF, PJ.CNPJ, PJ.RazaoSocial, PJ.Municipio, PJ.UF, MMM.MICROREGIAO,
                    CASE
                        WHEN ISNULL(CUSTO.custo, -1) = -1 THEN 20
                        ELSE Custo.custo + 1
                    END as custo

                FROM ReceitaNovo.dbo.Socio S
                    INNER JOIN SERVIDORES SERV ON (('000' + SERV.CPF) = S.CpfSocio AND S.IdentificadorSocio = 2)
                    INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (PJ.CNPJ = S.Cnpj)
                    LEFT OUTER JOIN SISPESQUISA.dbo.MUNICIPIO_MESOMICROREGIAO MMM ON (MMM.MUNICIPIO COLLATE Latin1_General_CI_AI = PJ.MUNICIPIO AND PJ.UF = 'PB')
                    LEFT OUTER JOIN UG ON (UG.CD_UGESTORA = @UGESTORA)
                    LEFT OUTER JOIN SISPESQUISA.dbo.tabela_custo_microregioes CUSTO ON (CUSTO.origem = MMM.MICROREGIAO COLLATE Latin1_General_CI_AI AND CUSTO.destino = UG.MICROREGIAO COLLATE Latin1_General_CI_AI)
            )
            ,RESULTADO(CPF, MICRORREGIAO, UF, CUSTO, QTD)
            AS
            (
                SELECT CPF, MICRORREGIAO, UF, CUSTO, COUNT(*) AS QTD
                FROM SERVIDORES_SOCIOS S
                GROUP BY CPF, MICRORREGIAO, UF, CUSTO
            )

            SELECT PF.CPF as cpf, PF.Nome as nome, SUM(CUSTO * QTD * @PESO_TIPOLOGIA) as score
            FROM RESULTADO
                LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (RESULTADO.CPF = PF.CPF)
            GROUP BY PF.CPF, PF.Nome
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaSimplificadaFP_SocioPJ');

            result.recordset = insereAtributoTipologia(result.recordset, tipologias['fp_sociopj'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias Empenhos-SISOBI para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaSimplificadaFP_SocioPJ'); });
}

export let getTipologiaSimplificadaFP_ResponsavelPJ = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.VarChar, dtInicio)
        .input('DT_FINAL', mssql.VarChar, dtFim)
        .input('UGESTORA', mssql.VarChar, ugestora)
        .input('PESO_TIPOLOGIA', mssql.Int, tipologias['fp_responsavelpj'].peso)
        .query(`
            ;WITH SERVIDORES(CPF)
            AS
            (
                SELECT DISTINCT FP.cd_CPF
                FROM SAGRES_MUNICIPAL.dbo.FolhaPagamento_N FP
                WHERE FP.cd_UGestora = @UGESTORA
                    AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL

                UNION ALL

                SELECT DISTINCT FP.cd_CPF
                FROM SAGRES_MUNICIPAL.dbo.FolhaPagamento_N_2017 FP
                WHERE FP.cd_UGestora = @UGESTORA
                    AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
            )

            ,UG(CD_UGESTORA, MUNICIPIO, MICROREGIAO)
            AS
            (
                SELECT DISTINCT G.cd_Ugestora, M.no_Municipio, MMM.MICROREGIAO
                FROM SAGRES_CONTROLE.dbo.Codigo_Ugestora G
                    INNER JOIN SAGRES_CONTROLE.dbo.Codigo_Municipios M ON (G.cd_Municipio = M.cd_Municipio)
                    LEFT OUTER JOIN SISPESQUISA.dbo.MUNICIPIO_MESOMICROREGIAO MMM ON (MMM.MUNICIPIO COLLATE Latin1_General_CI_AI = M.no_Municipio COLLATE Latin1_General_CI_AI)
            )

            ,SERVIDORES_RESPONSAVEIS_EMPRESA(CPF, CNPJ, RAZAOSOCIAL, MUNICIPIO, UF, MICRORREGIAO, CUSTO)
            AS
            (
                SELECT DISTINCT
                    SERV.CPF, PJ.CNPJ, PJ.RazaoSocial, PJ.Municipio,  PJ.UF, MMM.MICROREGIAO,
                    CASE
                        WHEN ISNULL(CUSTO.custo, -1) = -1 THEN 20
                        ELSE Custo.custo + 1
                    END as custo
                FROM ReceitaNovo.dbo.PessoaJuridica PJ
                    INNER JOIN SERVIDORES SERV ON (SERV.CPF = PJ.CpfResponsavel)
                    LEFT OUTER JOIN SISPESQUISA.dbo.MUNICIPIO_MESOMICROREGIAO MMM ON (MMM.MUNICIPIO COLLATE Latin1_General_CI_AI = PJ.MUNICIPIO AND PJ.UF = 'PB')
                    LEFT OUTER JOIN UG ON (UG.CD_UGESTORA = @UGESTORA)
                    LEFT OUTER JOIN SISPESQUISA.dbo.tabela_custo_microregioes CUSTO ON (CUSTO.origem = MMM.MICROREGIAO COLLATE Latin1_General_CI_AI AND CUSTO.destino = UG.MICROREGIAO COLLATE Latin1_General_CI_AI)
            )

            ,RESULTADO(CPF, MICRORREGIAO, UF, CUSTO, QTD)
            AS
            (
                SELECT CPF, MICRORREGIAO, UF, CUSTO, COUNT(*) AS QTD
                FROM SERVIDORES_RESPONSAVEIS_EMPRESA S
                GROUP BY CPF, MICRORREGIAO, UF, CUSTO
            )

            SELECT PF.CPF as cpf, PF.Nome as nome, SUM(CUSTO * QTD * @PESO_TIPOLOGIA) as score
            FROM RESULTADO
                LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (RESULTADO.CPF = PF.CPF)
            GROUP BY PF.CPF, PF.Nome
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaSimplificadaFP_ResponsavelPJ');

            result.recordset = insereAtributoTipologia(result.recordset, tipologias['fp_responsavelpj'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias FP-ResponsavelPJ para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaSimplificadaFP_ResponsavelPJ'); });
}

export let getTipologiaSimplificadoFP_ExtraVinculos = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.VarChar, dtInicio)
        .input('DT_FINAL', mssql.VarChar, dtFim)
        .input('UGESTORA', mssql.VarChar, ugestora)
        .input('PESO_TIPOLOGIA', mssql.Int, tipologias['fp_extravinculo_publico'].peso)
        .query(`
            ;WITH SERVIDORES_DO_ORGAO(CPF, ANO)
            AS
            (
                SELECT DISTINCT FP.cd_CPF, FP.dt_Ano
                FROM MISC.dbo.FolhaPagamento_Servidores FP
                WHERE FP.cd_UGestora = @UGESTORA
                        AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
            )

            ,UG(CD_UGESTORA, UGESTORA, MUNICIPIO, MICROREGIAO)
            AS
            (
                SELECT DISTINCT G.cd_Ugestora, UPPER(G.de_Ugestora), UPPER(M.no_Municipio), MMM.MICROREGIAO
                FROM SAGRES_CONTROLE.dbo.Codigo_Ugestora G
                    INNER JOIN SAGRES_CONTROLE.dbo.Codigo_Municipios M ON (G.cd_Municipio = M.cd_Municipio)
                    LEFT OUTER JOIN SISPESQUISA.dbo.MUNICIPIO_MESOMICROREGIAO MMM ON (MMM.MUNICIPIO COLLATE Latin1_General_CI_AI = M.no_Municipio COLLATE Latin1_General_CI_AI)
            )

            ,EXTRA_VINCULOS(CPF, ANO, CD_UGESTORA)
            AS
            (
                SELECT DISTINCT FP.cd_CPF, FP.dt_Ano, FP.cd_UGestora
                FROM MISC.dbo.FolhaPagamento_Servidores FP
                    INNER JOIN SERVIDORES_DO_ORGAO S ON (FP.cd_CPF = S.CPF AND FP.dt_Ano = ANO)
                WHERE FP.cd_UGestora <> @UGESTORA
                    AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
            )

            ,SERVIDORES_COM_EXTRAVINCULOS(CPF, ANO, CD_UGESTORA, UGESTORA, MUNICIPIO, MICRORREGIAO)
            AS
            (
                SELECT E.CPF, E.ANO, E.CD_UGESTORA, UG.UGESTORA, UG.MUNICIPIO, UG.MICROREGIAO
                FROM EXTRA_VINCULOS E
                    LEFT OUTER JOIN UG ON (E.CD_UGESTORA = UG.CD_UGESTORA)
            )

            SELECT	S.CPF as cpf, PF.Nome as nome, SUM((CUSTO.custo+1) * @PESO_TIPOLOGIA) as score
            FROM SERVIDORES_COM_EXTRAVINCULOS S
                INNER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (S.CPF = PF.CPF)
                LEFT OUTER JOIN UG ON (UG.CD_UGESTORA = @UGESTORA)
                LEFT OUTER JOIN SISPESQUISA.dbo.tabela_custo_microregioes CUSTO ON (CUSTO.origem = S.MICRORREGIAO COLLATE Latin1_General_CI_AI AND CUSTO.destino = UG.MICROREGIAO COLLATE Latin1_General_CI_AI)
            GROUP BY S.CPF, PF.Nome
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaSimplificadoFP_ExtraVinculos');

            result.recordset = insereAtributoTipologia(result.recordset, tipologias['fp_extravinculo_publico'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias FP-ExtraVinculo para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaSimplificadoFP_ExtraVinculos'); });
}

export let getTipologiaSimplificadoFP_SocioPJ_Empenhos = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.VarChar, dtInicio)
        .input('DT_FINAL', mssql.VarChar, dtFim)
        .input('UGESTORA', mssql.VarChar, ugestora)
        .input('PESO_TIPOLOGIA', mssql.Int, tipologias['fp_sociopj_empenho'].peso)
        .query(`
            ;WITH SERVIDORES(CPF, ANO)
            AS
            (
                SELECT DISTINCT FP.cd_CPF, FP.dt_Ano
                FROM MISC.dbo.FolhaPagamento_Servidores FP
                WHERE FP.cd_UGestora = @UGESTORA
                        AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
            )

            ,UG(CD_UGESTORA, MUNICIPIO, MICROREGIAO)
            AS
            (
                SELECT DISTINCT G.cd_Ugestora, M.no_Municipio, MMM.MICROREGIAO
                FROM SAGRES_CONTROLE.dbo.Codigo_Ugestora G
                    INNER JOIN SAGRES_CONTROLE.dbo.Codigo_Municipios M ON (G.cd_Municipio = M.cd_Municipio)
                    LEFT OUTER JOIN SISPESQUISA.dbo.MUNICIPIO_MESOMICROREGIAO MMM ON (MMM.MUNICIPIO COLLATE Latin1_General_CI_AI = M.no_Municipio COLLATE Latin1_General_CI_AI)
            )

            ,SERVIDORES_SOCIOS(ANO, CPF, CNPJ, RAZAOSOCIAL, MUNICIPIO, UF, MICRORREGIAO, CUSTO)
            AS
            (
                SELECT
                    SERV.ANO, SERV.CPF, PJ.CNPJ, PJ.RazaoSocial, PJ.Municipio, PJ.UF, MMM.MICROREGIAO,
                    CASE
                        WHEN ISNULL(CUSTO.custo, -1) = -1 THEN 20
                        ELSE Custo.custo + 1
                    END as custo

                FROM ReceitaNovo.dbo.Socio S
                    INNER JOIN SERVIDORES SERV ON (('000' + SERV.CPF) = S.CpfSocio AND S.IdentificadorSocio = 2)
                    INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (PJ.CNPJ = S.Cnpj)
                    LEFT OUTER JOIN SISPESQUISA.dbo.MUNICIPIO_MESOMICROREGIAO MMM ON (MMM.MUNICIPIO COLLATE Latin1_General_CI_AI = PJ.MUNICIPIO AND PJ.UF = 'PB')
                    LEFT OUTER JOIN UG ON (UG.CD_UGESTORA = @UGESTORA)
                    LEFT OUTER JOIN SISPESQUISA.dbo.tabela_custo_microregioes CUSTO ON (CUSTO.origem = MMM.MICROREGIAO COLLATE Latin1_General_CI_AI AND CUSTO.destino = UG.MICROREGIAO COLLATE Latin1_General_CI_AI)
            )

            ,EMPRESAS_EMPENHO(CPF,  CNPJ, RAZAOSOCIAL, MUNICIPIO, UF, MICRORREGIAO, CUSTO, EMPENHADO, PAGAMENTO, QTD)
            AS
            (
                SELECT	S.CPF, S.CNPJ, S.RAZAOSOCIAL, S.MUNICIPIO, S.UF, S.MICRORREGIAO, S.CUSTO,
                        SUM(E.vl_Empenho) AS EMPENHADO, SUM(P.vl_Pagamento) AS PAGAMENTO, COUNT(*) AS QTD

                FROM SERVIDORES_SOCIOS S
                    INNER JOIN SAGRES_MUNICIPAL.dbo.Empenhos E ON (S.CNPJ = E.cd_Credor AND S.ANO = E.dt_Ano)
                    INNER JOIN SAGRES_MUNICIPAL.dbo.PAGAMENTOS P ON (E.cd_Ugestora = P.cd_Ugestora) AND (E.dt_Ano = P.dt_Ano) AND (E.cd_UnidOrcamentaria = P.cd_UnidOrcamentaria) AND (E.nu_Empenho = P.nu_Empenho)

                WHERE E.cd_UGestora = @UGESTORA
                    AND E.tp_Credor = 2
                    AND E.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL

                GROUP BY S.CPF, S.CNPJ, S.RAZAOSOCIAL, S.MUNICIPIO, S.UF, S.MICRORREGIAO, S.CUSTO
            )


            ,RESULTADO(CPF, NOME, CNPJ, RAZAOSOCIAL, MUNICIPIO, UF, MICRORREGIAO, CUSTO, QTD, FX_PAGO)
            AS
            (
                SELECT E.CPF, PF.Nome, CNPJ, RAZAOSOCIAL, E.MUNICIPIO, E.UF, MICRORREGIAO, CUSTO, QTD,
                        CASE
                            WHEN PAGAMENTO > 0 AND PAGAMENTO <= 10000 THEN 1
                            WHEN PAGAMENTO > 10000 AND PAGAMENTO <= 100000 THEN 2
                            WHEN PAGAMENTO > 100000 AND PAGAMENTO <= 250000 THEN 3
                            WHEN PAGAMENTO > 250000 AND PAGAMENTO <= 500000 THEN 4
                            WHEN PAGAMENTO > 500000 AND PAGAMENTO <= 750000 THEN 5
                            WHEN PAGAMENTO > 750000 AND PAGAMENTO <= 1000000 THEN 6
                            WHEN PAGAMENTO > 1000000 THEN 7
                            ELSE 0
                        END AS FX_PAGO
                FROM EMPRESAS_EMPENHO E
                    INNER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (E.CPF = PF.CPF)
            )

            SELECT CPF as cpf, NOME as nome, @PESO_TIPOLOGIA * ((FX_PAGO+1) * (FX_PAGO+1) + (QTD/CUSTO)) as score
            FROM RESULTADO
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaSimplificadoFP_SocioPJ_Empenhos');

            result.recordset = insereAtributoTipologia(result.recordset, tipologias['fp_sociopj_empenho'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias FP-SocioPJ-ExtraVinculo para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaSimplificadoFP_SocioPJ_Empenhos'); });
}

export let getTipologiaSimplificadoFP_SocioPJ_Caged = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.VarChar, dtInicio)
        .input('DT_FINAL', mssql.VarChar, dtFim)
        .input('UGESTORA', mssql.VarChar, ugestora)
        .input('PESO_TIPOLOGIA', mssql.Int, tipologias['fp_sociopj_caged'].peso)
        .query(`
            ;WITH SERVIDORES(CPF)
            AS
            (
                SELECT DISTINCT FP.cd_CPF
                FROM MISC.dbo.FolhaPagamento_Servidores FP
                WHERE FP.cd_UGestora = @UGESTORA
                    AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
            )

            ,UG(CD_UGESTORA, MUNICIPIO, MICROREGIAO)
            AS
            (
                SELECT DISTINCT G.cd_Ugestora, M.no_Municipio, MMM.MICROREGIAO
                FROM SAGRES_CONTROLE.dbo.Codigo_Ugestora G
                    INNER JOIN SAGRES_CONTROLE.dbo.Codigo_Municipios M ON (G.cd_Municipio = M.cd_Municipio)
                    LEFT OUTER JOIN SISPESQUISA.dbo.MUNICIPIO_MESOMICROREGIAO MMM ON (MMM.MUNICIPIO COLLATE Latin1_General_CI_AI = M.no_Municipio COLLATE Latin1_General_CI_AI)
            )

            ,SERVIDORES_SOCIOS(CPF, CNPJ, RAZAOSOCIAL, MUNICIPIO, UF, MICRORREGIAO, CUSTO)
            AS
            (
                SELECT DISTINCT
                    SERV.CPF, PJ.CNPJ, PJ.RazaoSocial, PJ.Municipio, PJ.UF, MMM.MICROREGIAO,
                    CASE
                        WHEN ISNULL(CUSTO.custo, -1) = -1 THEN 20
                        ELSE Custo.custo + 1
                    END as custo

                FROM ReceitaNovo.dbo.Socio S
                    INNER JOIN SERVIDORES SERV ON (('000' + SERV.CPF) = S.CpfSocio AND S.IdentificadorSocio = 2)
                    INNER JOIN ReceitaNovo.dbo.PessoaJuridica PJ ON (PJ.CNPJ = S.Cnpj)
                    LEFT OUTER JOIN SISPESQUISA.dbo.MUNICIPIO_MESOMICROREGIAO MMM ON (MMM.MUNICIPIO COLLATE Latin1_General_CI_AI = PJ.MUNICIPIO AND PJ.UF = 'PB')
                    LEFT OUTER JOIN UG ON (UG.CD_UGESTORA = @UGESTORA)
                    LEFT OUTER JOIN SISPESQUISA.dbo.tabela_custo_microregioes CUSTO ON (CUSTO.origem = MMM.MICROREGIAO COLLATE Latin1_General_CI_AI AND CUSTO.destino = UG.MICROREGIAO COLLATE Latin1_General_CI_AI)
            )

            ,EMPRESAS_FUNCIONARIOS_CAGED(CPF, CNPJ, RAZAOSOCIAL, MUNICIPIO, UF, MICRORREGIAO, CUSTO, QTD_EMPREGADOS_CAGED)
            AS
            (
                SELECT S.CPF, S.CNPJ, S.RAZAOSOCIAL, S.MUNICIPIO, S.UF, S.MICRORREGIAO,
                        S.CUSTO, COUNT(DISTINCT E.CPF) as QTD_EMPREGADOS_CAGED
                FROM SERVIDORES_SOCIOS S
                    LEFT OUTER JOIN CAGED.dbo.Empregados E ON (S.CNPJ = E.CNPJCEI)
                WHERE (E.Competencia BETWEEN @DT_INICIAL AND @DT_FINAL) OR E.Competencia IS NULL
                GROUP BY S.CPF, S.CNPJ, S.RAZAOSOCIAL, S.MUNICIPIO, S.UF, S.MICRORREGIAO, S.CUSTO

            )

            ,RESULTADO(CPF, NOME, CUSTO, QTD_EMPRESAS_FANTASMAS)
            AS
            (
                SELECT E.CPF, PF.Nome, CUSTO, COUNT(CNPJ)
                FROM EMPRESAS_FUNCIONARIOS_CAGED E
                    INNER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (PF.CPF = E.CPF)
                WHERE QTD_EMPREGADOS_CAGED = 0
                GROUP BY E.CPF, Pf.NOME, CUSTO
            )

            SELECT CPF as cpf, NOME as nome, SUM((@PESO_TIPOLOGIA * QTD_EMPRESAS_FANTASMAS)/CUSTO) as score
            FROM RESULTADO
            GROUP BY CPF, NOME
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaSimplificadoFP_SocioPJ_Caged');

            result.recordset = insereAtributoTipologia(result.recordset, tipologias['fp_sociopj_caged'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias FP-SocioPJ-Caged para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaSimplificadoFP_SocioPJ_Caged'); });
}

export let getTipologiaSimplificadoFP_BolsaFamilia = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.VarChar, dtInicio)
        .input('DT_FINAL', mssql.VarChar, dtFim)
        .input('UGESTORA', mssql.VarChar, ugestora)
        .input('PESO_TIPOLOGIA', mssql.Int, tipologias['fp_bolsafamilia'].peso)
        .query(`
            ;WITH SERVIDORES(CPF)
            AS
            (
                SELECT FP.cd_CPF
                FROM MISC.dbo.FolhaPagamento_Servidores FP
                WHERE FP.cd_UGestora = @UGESTORA
                    AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
            )
            ,SERVIDORES_BOLSA_FAMILIA(CPF)
            AS
            (
                SELECT DISTINCT BF.NUM_CPF_CADUNICO
                FROM SERVIDORES S
                    INNER JOIN BolsaFamilia.dbo.DGI_BENEF_BOLSA_FAMILIA_TCU BF ON (S.CPF = BF.NUM_CPF_CADUNICO)
            )


            SELECT S.CPF as cpf, PF.Nome as nome, 1 as score
            FROM SERVIDORES_BOLSA_FAMILIA S
                LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (S.CPF = PF.CPF)
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaSimplificadoFP_BolsaFamilia');

            result.recordset = insereAtributoTipologia(result.recordset, tipologias['fp_bolsafamilia'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias FP-BolsaFamilia para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaSimplificadoFP_BolsaFamilia'); });
}

export let _getTipologiaSimplificadoFP_BolsaFamilia = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.VarChar, dtInicio)
        .input('DT_FINAL', mssql.VarChar, dtFim)
        .input('UGESTORA', mssql.VarChar, ugestora)
        .input('PESO_TIPOLOGIA', mssql.Int, tipologias['fp_bolsafamilia'].peso)
        .query(`
            ;WITH SERVIDORES(CPF, ANO)
            AS
            (
                SELECT DISTINCT FP.cd_CPF, FP.dt_Ano
                FROM MISC.dbo.FolhaPagamento_Servidores FP
                WHERE FP.cd_UGestora = @UGESTORA
                    AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
            )

            ,BOLSA_FAMILIA(NIS, ANO)
            AS
            (
                SELECT NISTitular, 2014 as ANO FROM BolsaFamilia.dbo.VW_BolsaFamilia_PB_2014
                    UNION
                SELECT NISTitular, 2015 as ANO FROM BolsaFamilia.dbo.VW_BolsaFamilia_PB_2015
                    UNION
                SELECT NISTitular, 2016 as ANO FROM BolsaFamilia.dbo.VW_BolsaFamilia_PB_2016
            )


            SELECT S.CPF as cpf, PF.Nome as nome, 1 as score
            FROM SERVIDORES S
                INNER JOIN Cadastro_Unico.dbo.bloco_00 CAD ON (CAD.CPF = S.CPF)
                INNER JOIN BOLSA_FAMILIA BF ON (BF.NIS = CAD.NIS AND S.ANO = BF.ANO)
                LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (S.CPF = PF.CPF)
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaSimplificadoFP_BolsaFamilia');

            result.recordset = insereAtributoTipologia(result.recordset, tipologias['fp_bolsafamilia'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias FP-BolsaFamilia para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaSimplificadoFP_BolsaFamilia'); });
}

export let getTipologiaSimplificadoFP_SIAF = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.VarChar, dtInicio)
        .input('DT_FINAL', mssql.VarChar, dtFim)
        .input('UGESTORA', mssql.VarChar, ugestora)
        .input('PESO_TIPOLOGIA', mssql.Int, tipologias['fp_siaf'].peso)
        .query(`
            ;WITH SERVIDORES(CPF)
            AS
            (
                SELECT DISTINCT FP.cd_CPF
                FROM MISC.dbo.FolhaPagamento_Servidores FP
                WHERE FP.cd_UGestora = @UGESTORA
                    AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
            )

            ,SERVIDOR_CONTAS_BANCARIAS(CPF, BANCO, AGENCIA, CONTA, DIGITO, DESCRICAO)
            AS
            (
                SELECT S.CPF as cpf, CC.banco, CC.agencia, CC.conta, CC.digito, CC.ds_conta
                FROM SERVIDORES S
                    INNER JOIN SIAF.siaf.credores C ON (C.nu_cnpj_cpf = S.CPF)
                    INNER JOIN SIAF.siaf.credores_contas CC ON (C.cd = CC.cd_credor)
            )

            ,RESULTADO(CPF, BANCO, AGENCIA, CONTA, QTD)
            AS
            (
                SELECT S.CPF, S.BANCO, S.AGENCIA, S.CONTA, COUNT(cd_credor) AS QTD
                FROM SERVIDOR_CONTAS_BANCARIAS S
                    INNER JOIN SIAF.siaf.credores_contas CC ON (CC.banco = S.BANCO AND CC.agencia = S.AGENCIA AND CC.conta = S.CONTA AND CC.digito = S.DIGITO)
                GROUP BY S.CPF, S.BANCO, S.AGENCIA, S.CONTA
            )

            SELECT PF.CPF as cpf, PF.Nome as nome, QTD as score
            FROM RESULTADO
                LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (RESULTADO.CPF = PF.CPF)
            WHERE QTD > 1
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaSimplificadoFP_SIAF');

            result.recordset = insereAtributoTipologia(result.recordset, tipologias['fp_siaf'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias FP-SIAF para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaSimplificadoFP_SIAF'); });
}

export let getTipologiaSimplificadoFP_TipologiasTCU = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.VarChar, dtInicio)
        .input('DT_FINAL', mssql.VarChar, dtFim)
        .input('UGESTORA', mssql.VarChar, ugestora)
        .input('PESO_TIPOLOGIA', mssql.Float, tipologias['fp_tipologia_tcu'].peso)
        .query(`
            ;WITH SERVIDORES(CPF)
            AS
            (
                SELECT DISTINCT FP.cd_CPF
                FROM MISC.dbo.FolhaPagamento_Servidores FP
                WHERE FP.cd_UGestora = @UGESTORA
                    AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
            )

            SELECT S.CPF as cpf, Credor as nome, CAST(TotalPesoTipologia * @PESO_TIPOLOGIA AS INT) as score
            FROM SERVIDORES S
                INNER JOIN Tipologias.dbo.MATRIZ_COMPLETA_PF T ON (S.CPF = T.CPF)
            WHERE T.TotalOcorrencias > 0
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaSimplificadoFP_TipologiasTCU');

            result.recordset = insereAtributoTipologia(result.recordset, tipologias['fp_tipologia_tcu'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias FP-TipologiasTCU para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaSimplificadoFP_TipologiasTCU'); });
}

export let getTipologiaSimplificadoFP_TipologiasTCU_Doadores = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.VarChar, dtInicio)
        .input('DT_FINAL', mssql.VarChar, dtFim)
        .input('UGESTORA', mssql.VarChar, ugestora)
        .input('PESO_TIPOLOGIA', mssql.Int, tipologias['fp_tipologia_tcu_doadores'].peso)
        .query(`
            ;WITH SERVIDORES(CPF)
            AS
            (
                SELECT DISTINCT FP.cd_CPF
                FROM MISC.dbo.FolhaPagamento_Servidores FP
                WHERE FP.cd_UGestora = @UGESTORA
                    AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
            )
            , RESULTADO(CPF, NOME, FAIXA)
            AS
            (
                SELECT S.CPF as cpf, Credor as nome,
                    CASE
                        WHEN T.DoacaoEleitoral < 5000 THEN 1
                        WHEN T.DoacaoEleitoral >= 5000 AND T.DoacaoEleitoral < 10000 THEN 2
                        WHEN T.DoacaoEleitoral >= 10000 AND T.DoacaoEleitoral < 50000 THEN 3
                        WHEN T.DoacaoEleitoral >= 50000 AND T.DoacaoEleitoral < 100000 THEN 4
                        WHEN T.DoacaoEleitoral >= 100000 AND T.DoacaoEleitoral < 500000 THEN 5
                        WHEN T.DoacaoEleitoral >= 500000 AND T.DoacaoEleitoral < 1000000 THEN 6
                        ELSE 7
                    END as score

                FROM SERVIDORES S
                    INNER JOIN Tipologias.dbo.MATRIZ_COMPLETA_PF T ON (S.CPF = T.CPF)
                WHERE T.T27 <> 0 OR T.T28 <> 0 OR T.T29 <> 0 OR T.T30 <> 0 OR T.T31 <> 0
            )

            SELECT CPF as cpf, NOME as nome, @PESO_TIPOLOGIA * FAIXA as score
            FROM RESULTADO
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaSimplificadoFP_TipologiasTCU_Doadores');

            result.recordset = insereAtributoTipologia(result.recordset, tipologias['fp_tipologia_tcu_doadores'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias FP-TipologiasTCU-Doadores para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaSimplificadoFP_TipologiasTCU_Doadores'); });
}

export let getTipologiaSimplificadoFP_Analfabetos = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.VarChar, dtInicio)
        .input('DT_FINAL', mssql.VarChar, dtFim)
        .input('UGESTORA', mssql.VarChar, ugestora)
        .input('PESO_TIPOLOGIA', mssql.Int, tipologias['fp_analfabetos'].peso)
        .query(`
            ;WITH SERVIDORES(CPF)
            AS
            (
                SELECT DISTINCT FP.cd_CPF
                FROM MISC.dbo.FolhaPagamento_Servidores FP
                WHERE FP.cd_UGestora = @UGESTORA
                    AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
            )

            SELECT S.CPF as cpf, NOME_ELEITOR as nome, @PESO_TIPOLOGIA as score
            FROM SERVIDORES S
                INNER JOIN SISPESQUISA.dbo.Analfabetos_tre_PB_2017 A ON (S.CPF = A.CPF)
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaSimplificadoFP_Analfabetos');

            result.recordset = insereAtributoTipologia(result.recordset, tipologias['fp_analfabetos'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias FP-Analfabetos para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaSimplificadoFP_Analfabetos'); });
}

export let getTipologiaSimplificadoFP_CAGED = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.VarChar, dtInicio)
        .input('DT_FINAL', mssql.VarChar, dtFim)
        .input('UGESTORA', mssql.VarChar, ugestora)
        .input('PESO_TIPOLOGIA', mssql.Int, tipologias['fp_caged'].peso)
        .query(`
            ;WITH SERVIDORES(CPF, ANO)
            AS
            (
                SELECT DISTINCT FP.cd_CPF, FP.dt_Ano
                FROM MISC.dbo.FolhaPagamento_Servidores FP
                WHERE FP.cd_UGestora = @UGESTORA
                    AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
            )

            ,SERVIDORES_COM_VINCULOS_PRIVADOS(CPF, ANO, QTD)
            AS
            (
                SELECT E.CPF, E.ANO, COUNT(DISTINCT E.CNPJCEI) AS QTD
                FROM SERVIDORES S
                    INNER JOIN CAGED.dbo.Empregados E  ON (S.CPF = E.CPF AND S.ANO = E.Ano)
                GROUP BY E.CPF, E.ANO
            )

            SELECT PF.CPF as cpf, PF.Nome as nome, SUM(QTD * @PESO_TIPOLOGIA) as score
            FROM SERVIDORES_COM_VINCULOS_PRIVADOS S
                LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (S.CPF = PF.CPF)
            GROUP BY PF.CPF, PF.Nome
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaSimplificadoFP_CAGED');

            result.recordset = insereAtributoTipologia(result.recordset, tipologias['fp_caged'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias FP-CAGED para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaSimplificadoFP_CAGED'); });
}

export let getTipologiaSimplificadoFP_Veiculos = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.VarChar, dtInicio)
        .input('DT_FINAL', mssql.VarChar, dtFim)
        .input('UGESTORA', mssql.VarChar, ugestora)
        .input('PESO_TIPOLOGIA', mssql.Int, tipologias['fp_veiculos'].peso)
        .query(`
            ;WITH SERVIDORES(CPF)
            AS
            (
                SELECT DISTINCT FP.cd_CPF
                FROM MISC.dbo.FolhaPagamento_Servidores FP
                WHERE FP.cd_UGestora = @UGESTORA
                    AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
            )

            ,SERVIDORES_COM_CARROS(CPF, QTD)
            AS
            (
                SELECT S.CPF, COUNT(*) AS QTD
                FROM Renavam_2014.dbo.renavam_2016_08 R
                    INNER JOIN SERVIDORES S ON (S.CPF = R.NUMERO_DOCUMENTO_PROPRIETARIO)
                GROUP BY CPF
            )

            SELECT CPF AS cpf,
                CASE
                    WHEN QTD <3 THEN 1
                    WHEN QTD >=3 AND QTD < 5 THEN 2
                    WHEN QTD >= 5 THEN 3
                END AS score
            FROM SERVIDORES_COM_CARROS S
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaSimplificadoFP_Veiculos');

            result.recordset = insereAtributoTipologia(result.recordset, tipologias['fp_veiculos'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias FP-Veiculos para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaSimplificadoFP_Veiculos'); });
}

export let getTipologiaSimplificadoFP_BolsaFamilia_Veiculos = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.VarChar, dtInicio)
        .input('DT_FINAL', mssql.VarChar, dtFim)
        .input('UGESTORA', mssql.VarChar, ugestora)
        .input('PESO_TIPOLOGIA', mssql.Int, tipologias['fp_bolsafamilia_veiculos'].peso)
        .query(`
            ;WITH SERVIDORES(CPF, ANO)
            AS
            (
                SELECT DISTINCT FP.cd_CPF, FP.dt_Ano
                FROM MISC.dbo.FolhaPagamento_Servidores FP
                WHERE FP.cd_UGestora = @UGESTORA
                    AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
            )

            ,BOLSA_FAMILIA(NIS, ANO)
            AS
            (
                SELECT NISTitular, 2014 as ANO FROM BolsaFamilia.dbo.VW_BolsaFamilia_PB_2014
                    UNION
                SELECT NISTitular, 2015 as ANO FROM BolsaFamilia.dbo.VW_BolsaFamilia_PB_2015
                    UNION
                SELECT NISTitular, 2016 as ANO FROM BolsaFamilia.dbo.VW_BolsaFamilia_PB_2016
            )


            SELECT S.CPF as cpf, PF.Nome as nome, @PESO_TIPOLOGIA * COUNT(*) as score
            FROM SERVIDORES S
                INNER JOIN Cadastro_Unico.dbo.bloco_00 CAD ON (CAD.CPF = S.CPF)
                INNER JOIN BOLSA_FAMILIA BF ON (BF.NIS = CAD.NIS AND S.ANO = BF.ANO)
                LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (S.CPF = PF.CPF)
                INNER JOIN Renavam_2014.dbo.renavam_2016_08 R ON (S.CPF = R.NUMERO_DOCUMENTO_PROPRIETARIO)
            GROUP BY S.CPF, PF.NOME
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaSimplificadoFP_BolsaFamilia_Veiculos');

            result.recordset = insereAtributoTipologia(result.recordset, tipologias['fp_bolsafamilia_veiculos'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias FP-BolsaFamilia-Veiculos para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaSimplificadoFP_BolsaFamilia_Veiculos'); });
}

export let getTipologiaSimplificadoFP_CPFInexistente = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.VarChar, dtInicio)
        .input('DT_FINAL', mssql.VarChar, dtFim)
        .input('UGESTORA', mssql.VarChar, ugestora)
        .input('PESO_TIPOLOGIA', mssql.Int, tipologias['fp_rf_cpf_inexistente'].peso)
        .query(`
            ;WITH SERVIDORES(CPF)
            AS
            (
                SELECT DISTINCT FP.cd_CPF
                FROM MISC.dbo.FolhaPagamento_Servidores FP
                WHERE FP.cd_UGestora = @UGESTORA
                    AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
            )

            SELECT S.CPF, @PESO_TIPOLOGIA as score
            FROM SERVIDORES S
                LEFT OUTER JOIN ReceitaNovo.dbo.PessoaFisica PF ON (S.CPF = PF.CPF)
            WHERE PF.CPF IS NULL
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaSimplificadoFP_CPFInexistente');

            result.recordset = insereAtributoTipologia(result.recordset, tipologias['fp_rf_cpf_inexistente'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias FP-CPFInexistente para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaSimplificadoFP_CPFInexistente'); });
}

export let getTipologiaSimplificadoFP_FiliacaoPartidariaRegular = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.VarChar, dtInicio)
        .input('DT_FINAL', mssql.VarChar, dtFim)
        .input('UGESTORA', mssql.VarChar, ugestora)
        .input('PESO_TIPOLOGIA', mssql.Int, tipologias['fp_filiacaoregular'].peso)
        .query(`
            ;WITH SERVIDORES(CPF)
            AS
            (
                SELECT DISTINCT FP.cd_CPF
                FROM MISC.dbo.FolhaPagamento_Servidores FP
                WHERE FP.cd_UGestora = @UGESTORA
                    AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
            )

            SELECT S.CPF as cpf, MAX(F.nome) as nome, COUNT(*) * @PESO_TIPOLOGIA as score
            FROM SERVIDORES S
                INNER JOIN SISPESQUISA.dbo.filiados_pb F ON (S.CPF = F.CPF)
            WHERE situacaoRegistro='REGULAR'
            GROUP BY S.CPF
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaSimplificadoFP_FiliacaoPartidariaRegular');

            result.recordset = insereAtributoTipologia(result.recordset, tipologias['fp_filiacaoregular'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias FP-FiliacaoPartidariaRegular para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaSimplificadoFP_FiliacaoPartidariaRegular'); });
}

export let getTipologiaSimplificadoFP_FiliacaoPartidariaCancelada = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.VarChar, dtInicio)
        .input('DT_FINAL', mssql.VarChar, dtFim)
        .input('UGESTORA', mssql.VarChar, ugestora)
        .input('PESO_TIPOLOGIA', mssql.Int, tipologias['fp_filiacaocancelada'].peso)
        .query(`
            ;WITH SERVIDORES(CPF)
            AS
            (
                SELECT DISTINCT FP.cd_CPF
                FROM MISC.dbo.FolhaPagamento_Servidores FP
                WHERE FP.cd_UGestora = @UGESTORA
                    AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
            )

            SELECT S.CPF as cpf, MAX(F.nome) as nome, COUNT(*) * @PESO_TIPOLOGIA as score
            FROM SERVIDORES S
                INNER JOIN SISPESQUISA.dbo.filiados_pb F ON (S.CPF = F.CPF)
            WHERE situacaoRegistro IN ('CANCELADO', 'DESFILIADO')
            GROUP BY S.CPF
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaSimplificadoFP_FiliacaoPartidariaCancelada');

            result.recordset = insereAtributoTipologia(result.recordset, tipologias['fp_filiacaocancelada'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias FP-FiliacaoPartidariaCancelada para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaSimplificadoFP_FiliacaoPartidariaCancelada'); });
}

export let getTipologiaSimplificadoFP_Cadicon = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.VarChar, dtInicio)
        .input('DT_FINAL', mssql.VarChar, dtFim)
        .input('UGESTORA', mssql.VarChar, ugestora)
        .input('PESO_TIPOLOGIA', mssql.Int, tipologias['fp_cadicon'].peso)
        .query(`
            ;WITH SERVIDORES(CPF)
            AS
            (
                SELECT DISTINCT FP.cd_CPF
                FROM MISC.dbo.FolhaPagamento_Servidores FP
                WHERE FP.cd_UGestora = @UGESTORA
                    AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
            )

            SELECT S.CPF as cpf, UPPER(C.NOME_RESPONSAVEL) as nome, COUNT(*) * @PESO_TIPOLOGIA as score
            FROM SERVIDORES S
                INNER JOIN Transparencia.dbo.cadicon_15_03_2018 C ON (S.CPF = C.NUM_CPF)
            GROUP BY S.CPF, C.NOME_RESPONSAVEL
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaSimplificadoFP_Cadicon');

            result.recordset = insereAtributoTipologia(result.recordset, tipologias['fp_cadicon'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias FP-Cadicon para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaSimplificadoFP_Cadicon'); });
}

export let getTipologiaSimplificadoFP_CEIS = function (ugestora: string, dtInicio: string, dtFim: string) {

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('DT_INICIAL', mssql.VarChar, dtInicio)
        .input('DT_FINAL', mssql.VarChar, dtFim)
        .input('UGESTORA', mssql.VarChar, ugestora)
        .input('PESO_TIPOLOGIA', mssql.Int, tipologias['fp_ceis'].peso)
        .query(`
            ;WITH SERVIDORES(CPF)
            AS
            (
                SELECT DISTINCT FP.cd_CPF
                FROM MISC.dbo.FolhaPagamento_Servidores FP
                WHERE FP.cd_UGestora = @UGESTORA
                    AND FP.MESANO_ORDENADO BETWEEN @DT_INICIAL AND @DT_FINAL
            )

            SELECT S.CPF as cpf, UPPER(C.Nome_Informado_pelo_Orgao_Sancionador) as nome, COUNT(*) * @PESO_TIPOLOGIA as score
            FROM SERVIDORES S
                INNER JOIN Transparencia.dbo.ceis_15_03_2018 C  ON (S.CPF = C.CPF_CNPJ_Sancionado)
            GROUP BY S.CPF, C.Nome_Informado_pelo_Orgao_Sancionador
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTipologiaSimplificadoFP_CEIS');

            result.recordset = insereAtributoTipologia(result.recordset, tipologias['fp_ceis'].codigo);

            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar tipologias FP-CEIS para ${ugestora}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pela tipologias de ${ugestora}.`, 'getTipologiaSimplificadoFP_CEIS'); });
}
