import * as md5 from 'md5';
import * as mssql from 'mssql';

import { connection }  from '../services/db.service';
import { Usuario, CadastroUsuario } from './schemas';

import {
    MODEL_PRIORITY,
    API_CODES,
    API_MSGS
} from '../config';
import {
    resultNotFound,
    resultFound,
    logErroBuscaBD,
    trataErro,
    agrupaControleAcessos
} from '../utils';

const fonte  = MODEL_PRIORITY['sispesquisa.usuario'].fonte;
const rank   = MODEL_PRIORITY['sispesquisa.usuario'].rank;
const grupo  = MODEL_PRIORITY['sispesquisa.usuario'].grupo;


export let checaLogin = function (login: string){
    return connection.getConn().request()
        .input('login_parameter', mssql.NVarChar, login)
        .query(`SELECT *
                FROM SISPESQUISA.sispesquisa.usuario
                WHERE login=@login_parameter`)
        .then(result => {
            if (result.recordset.length === 0){
                // Usuario inexistente
                return trataErro(API_CODES.CODE_FALHA_LOGIN, API_MSGS.MSG_FALHA_LOGIN);
            }

            let usuarioEncontrado = new Usuario(result.recordset[0]);

            // Usuario inativo
            if (!usuarioEncontrado.estaAtivo()){
                return trataErro(API_CODES.CODE_FALHA_LOGIN, API_MSGS.MSG_LOGIN_USUARIO_INATIVO);
            }

            return {
                status: API_CODES.CODE_SUCESSO,
                dados: usuarioEncontrado
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo usuario.`, 'checaLogin'); });
};

export let verificaLoginSenha = function (usuario: Usuario){

    return connection.getConn().request()
        .input('login_parameter', mssql.NVarChar, usuario.login)
        .input('senha_parameter', mssql.NVarChar, md5(usuario.senha))
        .query(`SELECT id
                FROM SISPESQUISA.sispesquisa.usuario
                WHERE login=@login_parameter and senha=@senha_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0){
                return trataErro(API_CODES.CODE_FALHA_LOGIN, API_MSGS.MSG_FALHA_LOGIN);
            } else {
                return trataErro(API_CODES.CODE_SUCESSO, API_MSGS.MSG_LOGIN_SUCESSO);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo usuario.`, 'verificaLoginSenha'); });
}

export let getUsuarios = function (){

    return connection.getConn().request()
        .query(`SELECT  id, nome, login, email, ativo, administrador, AD as ad, perfil,
                        proximo_logon as proximoLogon, processo, membro, id_cadastrador as idCadastrador, ISNULL(Operacoes,0) as operacoes

                FROM SISPESQUISA.sispesquisa.usuario
        `)
        .then(result => {
            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar usuarios.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch( error => { return logErroBuscaBD(error, `Falha na busca pelos usuarios.`, 'getUsuarios')});
}

export let getUsuario = function (id: string){

    return connection.getConn().request()
        .input('id_parameter', mssql.NVarChar, id)
        .query(`SELECT  id, nome, login, email, ativo, administrador, AD as ad, perfil,
                        proximo_logon as proximoLogon, processo, membro, id_cadastrador as idCadastrador, ISNULL(Operacoes,0) as operacoes

                FROM SISPESQUISA.sispesquisa.usuario
                WHERE id=@id_parameter
        `)
        .then(result => {
            if(result.recordset.length === 0){
                return resultNotFound(`Nao foi possivel encontrar usuarios.`, fonte);
                //throw new Error(`Nao foi possivel encontrar o usuário ${id}.`)
            }
            else{
                return resultFound(result, fonte, rank, grupo);
                //return {
                    //status: 'OK',
                    //obj: result.recordset.map(resultado => trataResultado(resultado))
                //}
            }
        })
        .catch( error => { return logErroBuscaBD(error, `Falha na busca pelo usuário ${id}.`, 'getUsuario')});
 }

 export let insertUsuario_Sispesquisa_Usuario = function(usuario: Usuario){

     return connection.getConn().request()
        .input('nome', mssql.NVarChar, usuario.nome)
        .input('login', mssql.NVarChar, usuario.login)
        .input('email', mssql.NVarChar, usuario.email)
        .input('ativo', mssql.NVarChar, 'S')
        .input('administrador', mssql.NVarChar, usuario.administrador)
        .input('AD', mssql.NVarChar, usuario.ad)
        .input('perfil', mssql.NVarChar, usuario.perfil)
        .input('senha', mssql.NVarChar, usuario.senha)
        .input('proximo_logon', mssql.NVarChar, usuario.proximoLogon)
        .input('processo', mssql.NVarChar, usuario.processo)
        .input('membro', mssql.NVarChar, usuario.membro)
        .input('operacoes', mssql.Bit, usuario.operacoes)
        .input('id_cadastrador', mssql.NVarChar, usuario.idCadastrador)
        .query(`INSERT INTO SISPESQUISA.sispesquisa.usuario
                (nome, login, email, ativo, administrador, AD, perfil, senha,
                proximo_logon, processo, membro, id_cadastrador, Operacoes)
                VALUES
                (@nome, @login, @email, @ativo, @administrador, @AD, @perfil, @senha,
                @proximo_logon, @processo, @membro, @id_cadastrador, @operacoes)
        `)
        .then(result => {
            return {
                status: API_CODES.CODE_SUCESSO,
                msg: 'Usuário cadastrado com sucesso.'
            }
        }).catch( error => { return logErroBuscaBD(error, `Falha no cadastro do usuario.`, 'insertUsuario_Sispesquisa_Usuario'); });
 }

 export let removeUsuarioLogin_Sispesquisa_Usuario = function(usuario: Usuario){

     return connection.getConn().request()
        .input('LOGIN', mssql.NVarChar, usuario.login)
        .query(`
            DELETE U
            FROM SISPESQUISA.sispesquisa.usuario U
            WHERE U.login = @LOGIN
        `)
        .then(result => {
            return {
                status: API_CODES.CODE_SUCESSO,
                msg: 'Usuário removido com sucesso.'
            }
        }).catch( error => { return logErroBuscaBD(error, `Falha na remocao do usuario ${usuario.login}.`, 'removeUsuarioLogin_Sispesquisa_Usuario'); });
 }

 export let updateUsuario_Sispesquisa_Usuario = function(usuario: Usuario){
    return connection.getConn().request()
        .input('id_parameter', mssql.NVarChar, usuario.id)
        .input('nome_parameter', mssql.NVarChar, usuario.nome)
        .input('login_parameter', mssql.NVarChar, usuario.login)
        .input('email_parameter', mssql.NVarChar, usuario.email)
        .input('ativo_parameter', mssql.NVarChar, usuario.ativo)
        .input('administrador_parameter', mssql.NVarChar, usuario.administrador)
        .input('AD_parameter', mssql.NVarChar, usuario.ad)
        .input('perfil_parameter', mssql.NVarChar, usuario.perfil)
        .input('proximo_logon_parameter', mssql.NVarChar, usuario.proximoLogon)
        .input('processo_parameter', mssql.NVarChar, usuario.processo)
        .input('membro_parameter', mssql.NVarChar, usuario.membro)
        .input('id_cadastrador_parameter', mssql.NVarChar, usuario.idCadastrador)
        .input('operacoes_parameter', mssql.Bit, usuario.operacoes)
        .query(`UPDATE SISPESQUISA.sispesquisa.usuario
                SET
                    nome=@nome_parameter,
                    login=@login_parameter,
                    email=@email_parameter,
                    ativo=@ativo_parameter,
                    administrador=@administrador_parameter,
                    AD=@AD_parameter,
                    perfil=@perfil_parameter,
                    proximo_logon=@proximo_logon_parameter,
                    processo=@processo_parameter,
                    membro=@membro_parameter,
                    id_cadastrador=@id_cadastrador_parameter,
                    Operacoes=@operacoes_parameter
                WHERE id=@id_parameter
        `)
        .then(result => {
            return {
                status: API_CODES.CODE_SUCESSO,
                msg: 'Usuário atualizado com sucesso.'
            }
        }).catch( error => { return logErroBuscaBD(error, `Falha na atualização do usuario.`, 'updateUsuario_Sispesquisa_Usuario'); });
 }

 export let updateSenhaUsuario_Sispesquisa_Usuario = function(id, senhanova){
    return connection.getConn().request()
        .input('id_parameter', mssql.NVarChar, id)
        .input('senha_parameter', mssql.NVarChar, md5(senhanova))
        .query(`UPDATE SISPESQUISA.sispesquisa.usuario
                SET
                    senha=@senha_parameter,
                    proximo_logon='N'
                WHERE id=@id_parameter
        `)
        .then(result => {
            return {
                status: API_CODES.CODE_SUCESSO,
                msg: 'Usuário atualizado com sucesso.'
            }
        }).catch( error => { return logErroBuscaBD(error, `Falha na atualização do usuario.`, 'updateSenhaUsuario_Sispesquisa_Usuario'); });
 }

 export let updateStatusUsuario_Sispesquisa_Usuario = function(login, status){
    return connection.getConn().request()
        .input('login_parameter', mssql.NVarChar, login)
        .input('ativo_parameter', mssql.NVarChar, (status) ? 'S' : 'N')
        .query(`UPDATE SISPESQUISA.sispesquisa.usuario
                SET
                    ativo=@ativo_parameter
                WHERE login=@login_parameter
        `)
        .then(result => {
            return {
                status: API_CODES.CODE_SUCESSO,
                msg: 'Usuário atualizado com sucesso.'
            }
        }).catch( error => { return logErroBuscaBD(error, `Falha na atualização do usuario.`, 'updateStatusUsuario_Sispesquisa_Usuario'); });
 }

 export let deleteUsuario_Sispesquisa_Usuario = function(id: string){
    return connection.getConn().request()
        .input('id_parameter', mssql.NVarChar, id)
        .query(`DELETE FROM SISPESQUISA.sispesquisa.usuario
                WHERE id=@id_parameter
        `)
        .then(result => {
            return {
                status: API_CODES.CODE_SUCESSO,
                msg: 'Usuário removido com sucesso.'
            }
        }).catch( error => { return logErroBuscaBD(error, `Falha na remocao do usuario.`, 'deleteUsuario_Sispesquisa_Usuario'); });
 }

 /**
  * Atualiza a senha do usuário e força a mudança da senha(pelo próprio usuário) no próximo login.
  *
  * @param id
  * @param novaSenha
  */
 export let updateSenhaUsuarioReset_Sispesquisa_Usuario = function(id, novaSenha){
    return connection.getConn().request()
        .input('id_parameter', mssql.NVarChar, id)
        .input('senha_parameter', mssql.NVarChar, md5(novaSenha))
        .query(`UPDATE SISPESQUISA.sispesquisa.usuario
                SET
                    senha=@senha_parameter,
                    proximo_logon='S'
                WHERE id=@id_parameter
        `)
        .then(result => {
            return {
                status: API_CODES.CODE_SUCESSO,
                msg: `Senha redefinida com sucesso! Nova senha: <b>${novaSenha}</b>`
            }
        }).catch( error => { return logErroBuscaBD(error, `Falha ao tentar redefinir a senha do usuário.`, 'updateSenhaUsuarioReset_Sispesquisa_Usuario'); });
 }


export let getControleAcessoPesquisaUsuario_Sispesquisa_Usuario = function (id: number) {

    return connection.getConn().request()
        .input('ID', mssql.Int, id)
        .query(`
                SELECT I.ID_SECAO, I.ID_ITEM
                FROM SISPESQUISA.SISPESQUISA.USUARIO_ITENS UI
                    INNER JOIN SISPESQUISA.sispesquisa.ITENS I ON (UI.ID_ITEM = I.ID_ITEM)
                    INNER JOIN SISPESQUISA.sispesquisa.SECOES S ON (I.ID_SECAO = S.ID_SECAO)
                WHERE UI.ID_USUARIO = @ID AND S.DESCRICAO = 'PESQUISA'
        `)
        .then(result => {
            if (result.recordset.length === 0)
                return {};
                // return resultNotFound(`Nao foi possivel encontrar permissoes nessa secao para ${id}.`, fonte);
            else {
                const dados = result.recordset;

                return agrupaControleAcessos(dados);
                // return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelas permissoes de ${id}.`, 'getControleAcessoPesquisaUsuario_Sispesquisa_Usuario'); });
}

export let getControleAcessoAppsUsuario_Sispesquisa_Usuario = function (id: number) {

    return connection.getConn().request()
        .input('ID', mssql.Int, id)
        .query(`
                SELECT I.ID_SECAO, I.ID_ITEM
                FROM SISPESQUISA.SISPESQUISA.USUARIO_ITENS UI
                    INNER JOIN SISPESQUISA.sispesquisa.ITENS I ON (UI.ID_ITEM = I.ID_ITEM)
                    INNER JOIN SISPESQUISA.sispesquisa.SECOES S ON (I.ID_SECAO = S.ID_SECAO)
                WHERE UI.ID_USUARIO = @ID AND S.DESCRICAO = 'APPS'
        `)
        .then(result => {
            if (result.recordset.length === 0)
                return {};
                // return resultNotFound(`Nao foi possivel encontrar permissoes nessa secao para ${id}.`, fonte);
            else {
                const dados = result.recordset;
                return agrupaControleAcessos(dados);
                // return resultFound(result, fonte, rank, grupo);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelas permissoes de ${id}.`, 'getControleAcessoAppsUsuario_Sispesquisa_Usuario'); });
}

export let getControleAcessoAnaliseUsuario_Sispesquisa_Usuario = function (id: number) {

    return connection.getConn().request()
        .input('ID', mssql.Int, id)
        .query(`
                SELECT I.ID_SECAO, I.ID_ITEM
                FROM SISPESQUISA.SISPESQUISA.USUARIO_ITENS UI
                    INNER JOIN SISPESQUISA.sispesquisa.ITENS I ON (UI.ID_ITEM = I.ID_ITEM)
                    INNER JOIN SISPESQUISA.sispesquisa.SECOES S ON (I.ID_SECAO = S.ID_SECAO)
                WHERE UI.ID_USUARIO = @ID AND S.DESCRICAO = 'ANALISE'
        `)
        .then(result => {
            if (result.recordset.length === 0)
                return {};
            else {
                const dados = result.recordset;
                return agrupaControleAcessos(dados);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelas permissoes de ${id}.`, 'getControleAcessoAnaliseUsuario_Sispesquisa_Usuario'); });
}

export let getControleAcessoOperacoesUsuario_Sispesquisa_Usuario = function (id: number) {

    return connection.getConn().request()
        .input('ID', mssql.Int, id)
        .query(`
                SELECT I.ID_SECAO, I.ID_ITEM
                FROM SISPESQUISA.SISPESQUISA.USUARIO_ITENS UI
                    INNER JOIN SISPESQUISA.sispesquisa.ITENS I ON (UI.ID_ITEM = I.ID_ITEM)
                    INNER JOIN SISPESQUISA.sispesquisa.SECOES S ON (I.ID_SECAO = S.ID_SECAO)
                WHERE UI.ID_USUARIO = @ID AND S.DESCRICAO = 'OPERACOES'
        `)
        .then(result => {
            if (result.recordset.length === 0)
                return {};
            else {
                const dados = result.recordset;
                return agrupaControleAcessos(dados);
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelas permissoes de ${id}.`, 'getControleAcessoOperacoesUsuario_Sispesquisa_Usuario'); });
}

export let insertControleAcessoUsuario_Sispesquisa_Usuario = function (usuario: Usuario) {

    let permissoes;
    switch(usuario.perfil) {
        case 'privado':
            permissoes = `
                S.DESCRICAO='PESQUISA' OR
                (
                    S.DESCRICAO = 'APPS' AND
                    (
                        I.DESCRICAO = 'RELACIONAMENTOS' OR
                        I.DESCRICAO = 'RELATORIO INTEGRADO' OR
                        I.DESCRICAO = 'ERB TRACKER' OR
                        I.DESCRICAO = 'TIPORANK'
                    )
                )
            `;
            break;
        case 'publico':
        default:
            permissoes = `
                S.DESCRICAO='PESQUISA' AND I.DESCRICAO = 'PESSOA'
            `;
    }

    return connection.getConn().request()
        .input('LOGIN', mssql.VarChar, usuario.login)
        .query(`
            ;WITH PERMISSOES(ID_ITEM)
            AS
            (
                SELECT I.ID_ITEM
                FROM SISPESQUISA.sispesquisa.ITENS I
                    INNER JOIN SISPESQUISA.sispesquisa.SECOES S ON (I.ID_SECAO = S.ID_SECAO)
                WHERE ${permissoes}
            )

            INSERT INTO SISPESQUISA.sispesquisa.USUARIO_ITENS
                (ID_USUARIO,ID_ITEM)
            SELECT U.id, P.ID_ITEM
            FROM PERMISSOES P
            OUTER APPLY(
                SELECT U.id
                FROM SISPESQUISA.sispesquisa.usuario U
                WHERE U.login = @LOGIN
            ) U
            WHERE NOT EXISTS (
                SELECT TOP 1 1
                FROM SISPESQUISA.sispesquisa.USUARIO_ITENS UI
                WHERE UI.ID_USUARIO = U.id AND UI.ID_ITEM = P.ID_ITEM
            )
        `)
        .then(result => {
            return {
                status: API_CODES.CODE_SUCESSO,
                msg: 'Controle de acesso inserido com sucesso.'
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na insercao das permissoes de ${usuario.login}.`, 'insertControleAcessoUsuario_Sispesquisa_Usuario'); });
}

export let removeControleAcessoUsuarioLogin_Sispesquisa_Usuario = function (usuario: Usuario) {

    return connection.getConn().request()
        .input('LOGIN', mssql.VarChar, usuario.login)
        .query(`
            DELETE UI
            FROM SISPESQUISA.sispesquisa.USUARIO_ITENS UI
                INNER JOIN SISPESQUISA.sispesquisa.usuario U ON (UI.ID_USUARIO = U.id)
            WHERE U.login = @LOGIN
        `)
        .then(result => {
            return {
                status: API_CODES.CODE_SUCESSO,
                msg: 'Controle de acesso removido com sucesso.'
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na remocao das permissoes de ${usuario.login}.`, 'removeControleAcessoUsuarioLogin_Sispesquisa_Usuario'); });
}

export let removeControleAcessoUsuarioID_Sispesquisa_Usuario = function (id: number) {

    return connection.getConn().request()
        .input('ID', mssql.Int, id)
        .query(`
            DELETE UI
            FROM SISPESQUISA.sispesquisa.USUARIO_ITENS UI
            WHERE UI.ID_USUARIO = @ID
        `)
        .then(result => {
            return {
                status: API_CODES.CODE_SUCESSO,
                msg: 'Controle de acesso removido com sucesso.'
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na remocao das permissoes de ${id}.`, 'removeControleAcessoUsuarioID_Sispesquisa_Usuario'); });
}

export let updateControleAcessoUsuario_Sispesquisa_Usuario = function (id: number, listaPermissoes: string) {

    return connection.getConn().request()
        .input('ID', mssql.Int, id)
        .input('LISTAPERMISSOES', mssql.VarChar, listaPermissoes)
        .query(`
            MERGE sispesquisa.USUARIO_ITENS AS TARGET
            USING (SELECT CAST(VALUE AS INT), @ID FROM STRING_SPLIT(@LISTAPERMISSOES,',')) AS SOURCE(ID_ITEM, ID_USUARIO)
                ON (TARGET.ID_USUARIO = SOURCE.ID_USUARIO AND TARGET.ID_ITEM = SOURCE.ID_ITEM)
            WHEN NOT MATCHED BY SOURCE AND TARGET.ID_USUARIO = @ID THEN
                DELETE
            WHEN NOT MATCHED BY TARGET THEN
                INSERT(ID_ITEM,ID_USUARIO)
                VALUES(ID_ITEM,ID_USUARIO);
        `)
        .then(result => {
            return {
                status: API_CODES.CODE_SUCESSO,
                msg: 'Controle de acesso atualizado com sucesso.'
            }
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelas atualização das permissoes de ${id}.`, 'updateControleAcessoUsuario_Sispesquisa_Usuario'); });
}