import * as mssql from 'mssql';
import * as rp from 'request-promise-native';

import { connection } from '../../services/db.service';
import { Pessoa } from '../schemas';
import { MODEL_PRIORITY, API_CONFIG, API_CODES } from '../../config';
import {
    trataResultado,
    resultNotFound,
    resultFound,
    logErroBuscaBD,
    resultFoundRaw,
    flat,
    print
} from '../../utils';

const fonte = MODEL_PRIORITY['utils'].fonte;
const rank  = MODEL_PRIORITY['utils'].rank;
const grupo = MODEL_PRIORITY['utils'].grupo;

export let getMunicipioUF_ReceitaNovo_UFMunicipio = function (uf, municipio = null) {

    let extra;
    if (municipio) {
        extra = `AND Municipio LIKE '${municipio}%' COLLATE Latin1_General_CI_AI`;
    }

    return connection.getConn().request()
        .input('uf', mssql.NVarChar, uf)
        .query(`SELECT Municipio as municipio, UF as uf

                FROM ReceitaNovo.dbo.UF_MUNICIPIO
                WHERE UF=@uf
                ${extra}
        `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar os municipios de ${uf}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos municipios de ${uf}.`, 'getMunicipioUF_ReceitaNovo_UFMunicipio'); });
}

export let getPromotoriasMPPB_Integra_Promotorias = function (promotoria: string){

    return connection.getConn().request()
        .input('PROMOTORIA', mssql.VarChar, '%' + promotoria + '%')
        .query(`SELECT
                    ID_PROMOTORIA as id,
                    DESCRICAO as descricao

                FROM Integra.dbo.PROMOTORIAS
                WHERE DESCRICAO LIKE @PROMOTORIA COLLATE Latin1_General_CI_AI
                ORDER BY 2
        `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar a promotoria ${promotoria}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);

        }).catch( error => logErroBuscaBD(error, `Falha na busca pela promotoria ${promotoria}.`, 'getPromotoriasMPPB_Integra_Promotorias'));
};

export let getOrgaoSagresMunicipal = function (orgao: string) {

    return connection.getConn().request()
        .input('orgao', mssql.NVarChar, '%' + orgao + '%')
        .query(`
            SELECT DISTINCT cd_Ugestora as cdUgestora, UPPER(de_Ugestora) as ugestora
            FROM SAGRES_CONTROLE.dbo.Codigo_Ugestora
            WHERE de_Ugestora LIKE @orgao COLLATE Latin1_General_CI_AI
            ORDER BY 2
        `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o orgao ${orgao}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo orgao de ${orgao}.`, 'getOrgaoSagresMunicipal'); });
}

export let getOrgaoSagresMunicipalEstadual = function (orgao: string) {

    return connection.getConn().request()
        .input('orgao', mssql.NVarChar, '%' + orgao + '%')
        .query(`
            ;WITH LOTACAO(CDUGESTORA, UGESTORA, ESFERA)
            AS
            (
                SELECT DISTINCT cd_Ugestora as cdUgestora, UPPER(de_Ugestora) as ugestora, 'M'
                FROM SAGRES_CONTROLE.dbo.Codigo_Ugestora
                WHERE de_Ugestora LIKE @orgao  COLLATE Latin1_General_CI_AI

                UNION

                SELECT DISTINCT NULL, de_Lotacao COLLATE Latin1_General_CI_AI, 'E'
                FROM SAGRES_ESTADUAL.dbo.t_FL_Lotacao
                WHERE de_Lotacao COLLATE Latin1_General_CI_AI LIKE @orgao
            )

            SELECT CDUGESTORA as cdUgestora, UGESTORA as ugestora, ESFERA as esfera
            FROM LOTACAO
        `)
        .then(result => {
            if (result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar o orgao ${orgao}.`, fonte);
            else
                return resultFound(result, fonte, rank, grupo);
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelo orgao de ${orgao}.`, 'getOrgaoSagresMunicipalEstadual'); });
}

export let getGeoCoordenadas = function (resultadoConsulta) {

    const dados = (resultadoConsulta.resultado.dados && resultadoConsulta.status === API_CODES.CODE_SUCESSO) ? resultadoConsulta.resultado.dados : [];
    if (dados.length) {
        return Promise.all(dados.map(e => promiseGeoCoordenadas(e)))

    } else {
        return resultadoConsulta;
    }
}

const fonte_geocoordenadas = MODEL_PRIORITY['geocoordenadas.google'].fonte;
const rank_geocoordenadas  = MODEL_PRIORITY['geocoordenadas.google'].rank;
const grupo_geocoordenadas = MODEL_PRIORITY['geocoordenadas.google'].grupo;

let promiseGeoCoordenadas = function (endereco) {

    const query = `${endereco.logradouro} ${endereco.numero} ${endereco.cep}`;
    const options = {
        method: 'GET',
        uri: `https://maps.googleapis.com/maps/api/geocode/json?address=${query}&key=${API_CONFIG.SIAP_GOOGLE_MAPS_API_KEY}`,
        timeout: 5000,
        json: true
    };

    return rp(options)
        .then(data => {
            if (data.status === 'OK' && data.results.length > 0) {
                const geo = data.results.map(item => {
                    return {
                        lat: item.geometry.location.lat,
                        lng: item.geometry.location.lng
                    }
                });
                return resultFoundRaw(geo, fonte_geocoordenadas, rank_geocoordenadas, grupo_geocoordenadas);
            }
            else
                return resultNotFound(`Nao foi possivel encontrar as geocoordenadas para ${query}.`, fonte_geocoordenadas);
        })
        .catch(error => { return logErroBuscaBD(error, `Falha na busca pelas geocoordenadas para ${query}.`, 'promiseGeoCoordenadas'); });
}