import * as moment from 'moment';
import * as apicache from 'apicache';
const redis = require('redis');

import {API_CONFIG} from './../config';

import {
    logErroBuscaBD,
    guardValidacaoApiCache
} from './../utils';

const method  = (process.env.SIAP_APICACHE_METHOD)  ? process.env.SIAP_APICACHE_METHOD  : 'bypass';
const enabled = (process.env.SIAP_APICACHE_ENABLED) ? JSON.parse(process.env.SIAP_APICACHE_ENABLED) : false;
const debug   = (process.env.SIAP_APICACHE_DEBUG)   ? JSON.parse(process.env.SIAP_APICACHE_DEBUG)   : false;

const redis_server = (process.env.REDIS_SERVER) ? process.env.REDIS_SERVER : '127.0.0.1';
const redis_port   = (process.env.REDIS_PORT)   ? process.env.REDIS_PORT   : 6379;

const cacheConfig = {
    enabled:  enabled,
    debug:    debug,
    method:   method,
    duration: '20 day',

    redisConfig: {
        host: redis_server,
        port: redis_port,
        retry_strategy: function (options) {
            const hora = moment().format('YYYY-MM-DDTHH:mm:ss');

            console.error(`${hora} - Erro: REDIS APICACHE`);
            if (options.error) {
                console.error(`Codigo: ${(options.error.code) ? options.error.code : ''}`);
                console.error(`Mensagem: ${(options.error.syscall)} ${options.error.errno} ${options.error.address}:${options.error.port}`, '\n');
            } else {
                console.error(options);
            }

            // Tenta reconectar ao REDIS 3 vezes
            if (options.attempt > 2) {
                console.error(`ERRO: APICACHE com metodo REDIS, aplicando modo default para cache`);
                return undefined;
            }

            // Reconecta
            return Math.min(options.attempt * 1000, 3000);
        }
    }
};


export class CacheService {
    cache;
    method  : string;
    duration: string;

    debug     : boolean;
    enabled   : boolean;
    connecting: boolean;
    connected : boolean;

    // Redis
    redisConfig;

    constructor(cacheConfig) {
        this.connected  = false;
        this.connecting = false;

        this.enabled  = cacheConfig.enabled;
        this.debug    = cacheConfig.debug;
        this.method   = cacheConfig.method;
        this.duration = cacheConfig.duration;

        this.redisConfig = cacheConfig.redisConfig;
    }

    connect() {

        // Checagem forçada, sem o === true ele está entrando com valores false
        if(this.enabled) {
            if (!this.connected && !this.connecting) {
                this.connecting = true;

                if (this.method === 'redis') {
                    this._criaClienteRedis(this.redisConfig);
                } else if (this.method === 'bypass') {
                    this._criaClienteBypass();
                } else {
                    this._criaClienteMem();
                }
            }
        } else {
            if (!this.connected && !this.connecting) {
                this._criaClienteBypass();
            }
        }

        return this.cache;
    }

    _criaClienteBypass() {
        this.method     = 'bypass';
        this.connecting = false;
        this.connected  = true;
        this.cache      = apicache.options({ debug: this.debug, enabled: false }).middleware(this.duration, guardValidacaoApiCache);

        console.log('CACHE: BYPASS efetuado com sucesso.');
    }

    _criaClienteMem() {
        this.method     = 'mem';
        this.connecting = false;
        this.connected  = true;
        this.cache      = apicache.options({ debug: this.debug }).middleware(this.duration, guardValidacaoApiCache);

        console.log('CACHE: Cache em Memória efetuada com sucesso.');

        return this.cache;
    }

    _criaClienteRedis(redisConfig) {
        this.method = 'redis';

        let client = redis.createClient(redisConfig);
        let self   = this;

        client.on('error', function (err) {
            const hora = moment().format('YYYY-MM-DDTHH:mm:ss');

            console.error(`${hora} - Erro: REDIS APICACHE`);
            console.error(`Codigo: ${(err.code) ? err.code : ''}`);
            console.error(`Mensagem: ${(err.syscall)} ${err.errno} ${err.address}:${err.port}`, '\n');
        });

        client.on('connect', function (connect) {
            this.connecting = true;
            this.connected  = false;
        });

        client.on('ready', function (ready) {
            this.connected  = true;
            this.connecting = false;

            console.log('CACHE: Conexao com REDIS efetuada com sucesso.');
            self.cache = apicache.options({ redisClient: client, debug: cacheConfig.debug }).middleware(cacheConfig.duration, guardValidacaoApiCache);
        });

        this.cache = apicache.options({ redisClient: client, debug: cacheConfig.debug }).middleware(cacheConfig.duration, guardValidacaoApiCache);

        // Se tentou conectar mas deu algum erro
        if (client.attempts > 0 && !this.connecting && !client.connected) {
            console.log('Se tentou conectar mas deu algum erro');
            console.log(this.connecting, this.connected);
            console.log(client.attempts, client.connected);

            this._criaClienteMem();
        }

        return this.cache;
    }

    getCache() {
        if (!this.cache) {
            return null;
        } else {
            return this.cache;
        }
    }

    clear(): void {
        this.cache.clear();
    }

    logVar() {
        console.log('\n- CacheService\n');

        console.log('this.connected', this.connected);
        console.log('this.connecting', this.connecting);
        console.log('this.enabled', this.enabled);
        console.log('this.debug', this.debug);
        console.log('this.method', this.method);
        console.log('this.duration', this.duration);
        console.log('');
    }
}

export let cache = new CacheService(cacheConfig);