import * as jwt from 'jsonwebtoken';
import * as sha1 from 'sha1';
import { Request, Response } from 'express';

import * as logModel from '../../models/log.model';
import { Log } from '../../schemas/log.schema';

import { API_CONFIG, API_CODES, API_MSGS, LOG_CODES, LOG_MSGS } from '../../config';
import { trataErro, getId_Token } from '../../utils';

/**
 *  Guard que valida o token recebido e garante ou não a passagem para a rota desejada.
 *  Garante acesso para qualquer usuário logado ao sistema.
 *
 * @param req
 * @param res
 * @param next
 */
export let GuardRotasLogado = function (req: Request, res: Response, next){
    var token = req.body.token || req.headers['token'] || req.headers['Authorization'] || req.headers['authorization'];

    if (API_CONFIG.CFG_ENV === 'test' && req.query.test === 's') {
        next();
    } else {
        if (token) {
            token = token.split(" ")[1];

            jwt.verify(token, API_CONFIG.SIAP_SENHA_TOKEN, { algorithms: ['HS256'] }, function (err, decoded) {
                if (err) {
                    if (API_CONFIG.CFG_ENV == 'production') {
                        logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_TENTATIVA_CONSULTA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_MSGS.MSG_TOKEN_INVALIDO}`, user_agent: req.headers['user-agent'] }));
                    }

                    res.status(401).send(trataErro(API_CODES.CODE_TOKEN_INVALIDO, API_MSGS.MSG_TOKEN_INVALIDO));
                }
                else
                    next();
            })
        } else {
            if (req.method === 'OPTIONS') {
                res.status(200).send('');
            } else {
                if (API_CONFIG.CFG_ENV == 'production') {
                    logModel.salvaLog(new Log({ ip: req.headers['x-real-ip'], usuario: 'null', tipo: API_CODES.CODE_TOKEN_NAO_ENCONTRADO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - GuardRotasLogado - ${API_MSGS.MSG_TOKEN_NAO_ENCONTRADO}`, user_agent: req.headers['user-agent'] }));
                }

                res.status(401).send(trataErro(API_CODES.CODE_TOKEN_NAO_ENCONTRADO, API_MSGS.MSG_TOKEN_NAO_ENCONTRADO));
            }
        }
    }
};

/**
 *  Guard que valida o token recebido e garante ou não a passagem para a rota desejada.
 *  Garante acesso para os usuários com perfil 'privado'.
 *
 * @param req
 * @param res
 * @param next
 */
export let GuardRotasPerfilPrivado = function (req, res, next){
    var token = req.body.token || req.headers['token'] || req.headers['Authorization'] || req.headers['authorization'];

    if (API_CONFIG.CFG_ENV === 'test' && req.query.test === 's') {
        next();
    } else {
        if (token) {
            token = token.split(" ")[1];

            jwt.verify(token, API_CONFIG.SIAP_SENHA_TOKEN, { algorithms: ['HS256'] }, function (err, decoded: any) {
                if (err) {
                    if (API_CONFIG.CFG_ENV == 'production') {
                        logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_TENTATIVA_CONSULTA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_MSGS.MSG_TOKEN_INVALIDO}`, user_agent: req.headers['user-agent'] }));
                    }
                    res.status(401).send(trataErro(API_CODES.CODE_TOKEN_INVALIDO, API_MSGS.MSG_TOKEN_INVALIDO));
                }
                else {
                    if (decoded.user.perfil === "admin" || decoded.user.perfil === "privado")
                        next();
                    else {
                        if (API_CONFIG.CFG_ENV == 'production') {
                            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_TENTATIVA_CONSULTA_SEM_PERFIL, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_MSGS.MSG_ROTA_NAO_AUTORIZADA}`, user_agent: req.headers['user-agent'] }));
                        }
                        res.status(403).send(trataErro(API_CODES.CODE_ROTA_NAO_AUTORIZADA, API_MSGS.MSG_ROTA_NAO_AUTORIZADA));
                    }
                }
            })
        } else {
            if (req.method === 'OPTIONS') {
                res.status(200).send('');
            } else {
                if (API_CONFIG.CFG_ENV == 'production') {
                    logModel.salvaLog(new Log({ ip: req.headers['x-real-ip'], usuario: 'null', tipo: API_CODES.CODE_TOKEN_NAO_ENCONTRADO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - GuardRotasPerfilPrivado - ${API_MSGS.MSG_TOKEN_NAO_ENCONTRADO}`, user_agent: req.headers['user-agent'] }));
                }
                res.status(401).send(trataErro(API_CODES.CODE_TOKEN_NAO_ENCONTRADO, API_MSGS.MSG_TOKEN_NAO_ENCONTRADO));
            }
        }
    }
};

/**
 *  Guard que valida o token recebido e garante ou não a passagem para a rota desejada.
 *  Garante acesso para os usuários com perfil 'admin'.
 *
 * @param req
 * @param res
 * @param next
 */
export let GuardRotasPerfilAdmin = function (req, res, next){
    var token = req.body.token || req.headers['token'] || req.headers['Authorization'] || req.headers['authorization'];

    if (API_CONFIG.CFG_ENV === 'test' && req.query.test === 's') {
        next();
    } else {
        if (token) {
            token = token.split(" ")[1];

            jwt.verify(token, API_CONFIG.SIAP_SENHA_TOKEN, { algorithms: ['HS256'] }, function (err, decoded: any) {
                if (err)
                    res.status(401).send(trataErro(API_CODES.CODE_TOKEN_INVALIDO, API_MSGS.MSG_TOKEN_INVALIDO));
                else {
                    if (decoded.user.perfil === "admin") {
                        next();
                    }
                    else {
                        if (API_CONFIG.CFG_ENV == 'production') {
                            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_TENTATIVA_CONSULTA_ADMIN_SEM_PERFIL, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_MSGS.MSG_ROTA_NAO_AUTORIZADA}`, user_agent: req.headers['user-agent'] }));
                        }
                        res.status(403).send(trataErro(API_CODES.CODE_ROTA_NAO_AUTORIZADA, API_MSGS.MSG_ROTA_NAO_AUTORIZADA));
                    }
                }
            })
        } else {
            if (req.method === 'OPTIONS') {
                res.status(200).send('');
            } else {
                if (API_CONFIG.CFG_ENV == 'production') {
                    logModel.salvaLog(new Log({ ip: req.headers['x-real-ip'], usuario: 'null', tipo: API_CODES.CODE_TOKEN_NAO_ENCONTRADO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - GuardRotasPerfilAdmin - ${API_MSGS.MSG_TOKEN_NAO_ENCONTRADO}`, user_agent: req.headers['user-agent'] }));
                }
                res.status(401).send(trataErro(API_CODES.CODE_TOKEN_NAO_ENCONTRADO, API_MSGS.MSG_TOKEN_NAO_ENCONTRADO));
            }
        }
    }
};

/**
 *  Guard que valida o token recebido e garante ou não a passagem para a rota desejada.
 *  Garante acesso para os usuários que tenham o atributo 'operacoes'.
 *
 * @param req
 * @param res
 * @param next
 */
export let GuardRotasOperacoes = function (req, res, next){
    var token = req.body.token || req.headers['token'] || req.headers['Authorization'] || req.headers['authorization'];

    if (API_CONFIG.CFG_ENV === 'test' && req.query.test === 's') {
        next();
    } else {
        if (token) {
            token = token.split(" ")[1];

            jwt.verify(token, API_CONFIG.SIAP_SENHA_TOKEN, { algorithms: ['HS256'] }, function (err, decoded:any) {
                if (err)
                    res.status(401).send(trataErro(API_CODES.CODE_TOKEN_INVALIDO, API_MSGS.MSG_TOKEN_INVALIDO));
                else {
                    if (decoded.user.perfil === "admin" || decoded.user.operacoes) {
                        next();
                    }
                    else {
                        if (API_CONFIG.CFG_ENV == 'production') {
                            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_TENTATIVA_CONSULTA_ADMIN_SEM_PERFIL, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_MSGS.MSG_ROTA_NAO_AUTORIZADA}`, user_agent: req.headers['user-agent'] }));
                        }
                        res.status(403).send(trataErro(API_CODES.CODE_ROTA_NAO_AUTORIZADA, API_MSGS.MSG_ROTA_NAO_AUTORIZADA));
                    }
                }
            })
        } else {
            if (req.method === 'OPTIONS') {
                res.status(200).send('');
            } else {
                if (API_CONFIG.CFG_ENV == 'production') {
                    logModel.salvaLog(new Log({ ip: req.headers['x-real-ip'], usuario: 'null', tipo: API_CODES.CODE_TOKEN_NAO_ENCONTRADO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - GuardRotasOperacoes - ${API_MSGS.MSG_TOKEN_NAO_ENCONTRADO}`, user_agent: req.headers['user-agent'] }));
                }
                res.status(401).send(trataErro(API_CODES.CODE_TOKEN_NAO_ENCONTRADO, API_MSGS.MSG_TOKEN_NAO_ENCONTRADO));
            }
        }
    }
};

/**
 *  Guard que valida o token recebido e garante ou não a passagem para a rota desejada.
 *  Garante acesso para os usuários que tenham o atributo 'operacoes'.
 *
 * @param req
 * @param res
 * @param next
 */
export let GuardRotasMembro = function (req, res, next){
    var token = req.body.token || req.headers['token'] || req.headers['Authorization'] || req.headers['authorization'];

    if (API_CONFIG.CFG_ENV === 'test' && req.query.test === 's') {
        next();
    } else {
        if (token) {
            token = token.split(" ")[1];

            jwt.verify(token, API_CONFIG.SIAP_SENHA_TOKEN, { algorithms: ['HS256'] }, function (err, decoded:any) {
                if (err)
                    res.status(401).send(trataErro(API_CODES.CODE_TOKEN_INVALIDO, API_MSGS.MSG_TOKEN_INVALIDO));
                else {
                    if (decoded.user.perfil === "admin" || decoded.user.membro) {
                        next();
                    }
                    else {
                        if (API_CONFIG.CFG_ENV == 'production') {
                            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_TENTATIVA_INTEGRA_SEM_PERFIL, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_MSGS.MSG_ROTA_NAO_AUTORIZADA}`, user_agent: req.headers['user-agent'] }));
                        }
                        res.status(403).send(trataErro(API_CODES.CODE_ROTA_NAO_AUTORIZADA, API_MSGS.MSG_ROTA_NAO_AUTORIZADA));
                    }
                }
            })
        } else {
            if (req.method === 'OPTIONS') {
                res.status(200).send('');
            } else {
                if (API_CONFIG.CFG_ENV == 'production') {
                    logModel.salvaLog(new Log({ ip: req.headers['x-real-ip'], usuario: 'null', tipo: API_CODES.CODE_TOKEN_NAO_ENCONTRADO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - GuardRotasMembro - ${API_MSGS.MSG_TOKEN_NAO_ENCONTRADO}`, user_agent: req.headers['user-agent'] }));
                }
                res.status(401).send(trataErro(API_CODES.CODE_TOKEN_NAO_ENCONTRADO, API_MSGS.MSG_TOKEN_NAO_ENCONTRADO));
            }
        }
    }
};


/**
 *  Guard que valida o hash 'hs' recebido e garante ou não a passagem para a rota desejada.
 *
 *  O hash 'hs' foi criado como mais uma camada de segurança ao sistema, impede que um atacante
 *  roube uma requisição genuína feita por um usuário e a replique para ter acesso ao sistema.
 *
 *  Ele é composto por um cálculo utilizando o ID do usuário, o User Agent da requisição e o tempo de quando
 *  ela foi criada.
 *
 *  Com o hash, toda requisição tem um tempo de vida definido através da variável API_CONFIG.CFG_TEMPO_VALIDADE_HASH.
 *
 * @param req
 * @param res
 * @param next
 */
export let GuardRotasChecaHash = function (req, res, next) {
    const hs = req.headers['hs'];

    if (API_CONFIG.CFG_ENV === 'test' && req.query.test === 's') {
        next();
    } else {
        if (hs) {
            const hash = hs.slice(0, 40);
            const ts = hs.slice(40, hs.length);

            let token = req.headers['Authorization'] || req.headers['authorization'];
            token = token.split(' ')[1];

            const decoded = <any>jwt.decode(token, { complete: true });
            const id = decoded.payload.user.id;
            const agent = req.headers['user-agent'];
            let contraHash = sha1(id + agent + ts);

            const deltats = (Date.now() - parseInt(ts, 10)); //  Converte ms pra s

            if (hash === contraHash && (deltats < API_CONFIG.CFG_TEMPO_VALIDADE_HASH)) {
                //if (hash === contraHash){
                next();
            } else {
                if (API_CONFIG.CFG_ENV == 'production') {
                    logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_TENTATIVA_CONSULTA_HASH_INVALIDO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - Hash inválido ou não presente.`, user_agent: req.headers['user-agent'] }));
                }
                res.status(401).send(trataErro(API_CODES.CODE_HASH_INVALIDO, API_MSGS.MSG_HASH_INVALIDO));
            }
        } else {
            if (req.method === 'OPTIONS') {
                res.status(200).send('');
            } else {
                if (API_CONFIG.CFG_ENV == 'production') {
                    logModel.salvaLog(new Log({ ip: req.headers['x-real-ip'], usuario: 'null', tipo: API_CODES.CODE_HASH_NAO_ENCONTRADO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_MSGS.MSG_HASH_NAO_ENCONTRADO}`, user_agent: req.headers['user-agent'] }));
                }
                res.status(401).send(trataErro(API_CODES.CODE_HASH_NAO_ENCONTRADO, API_MSGS.MSG_HASH_NAO_ENCONTRADO));
            }
        }
    }
}

export let ControleAcessoGuard = function(options) {
    const opts = options || {};

    return function (req, res, next) {
        var token = req.body.token || req.headers['token'] || req.headers['Authorization'] || req.headers['authorization'];

        if (API_CONFIG.CFG_ENV === 'test' && req.query.test === 's') {
            next();
        } else {
            if (token) {
                token = token.split(" ")[1];

                jwt.verify(token, API_CONFIG.SIAP_SENHA_TOKEN, { algorithms: ['HS256'] }, function (err, decoded: any) {
                    if (err) {
                        if (API_CONFIG.CFG_ENV == 'production') {
                            logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_TENTATIVA_CONSULTA, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_MSGS.MSG_TOKEN_INVALIDO}`, user_agent: req.headers['user-agent'] }));
                        }

                        res.status(401).send(trataErro(API_CODES.CODE_TOKEN_INVALIDO, API_MSGS.MSG_TOKEN_INVALIDO));
                    }
                    else {
                        const acesso = decoded.user.acesso;

                        if (acesso[opts.secao] && acesso[opts.secao].indexOf(opts.item) !== -1) {
                            next();
                        } else {
                            if (API_CONFIG.CFG_ENV == 'production') {
                                logModel.salvaLogPorId(new Log({ ip: req.headers['x-real-ip'], usuario_id: getId_Token(req.headers['authorization']), tipo: LOG_CODES.TIPO_TENTATIVA_CONSULTA_SEM_PERFIL, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - ${API_MSGS.MSG_ROTA_NAO_AUTORIZADA}`, user_agent: req.headers['user-agent'] }));
                            }
                            res.status(403).send(trataErro(API_CODES.CODE_ROTA_NAO_AUTORIZADA, API_MSGS.MSG_ROTA_NAO_AUTORIZADA));
                        }
                    }
                })
            } else {
                if (req.method === 'OPTIONS') {
                    res.status(200).send('');
                } else {
                    if (API_CONFIG.CFG_ENV == 'production') {
                        logModel.salvaLog(new Log({ ip: req.headers['x-real-ip'], usuario: 'null', tipo: API_CODES.CODE_TOKEN_NAO_ENCONTRADO, mensagem: `${API_CONFIG.CFG_NOME_SISTEMA} - GuardRotasLogado - ${API_MSGS.MSG_TOKEN_NAO_ENCONTRADO}`, user_agent: req.headers['user-agent'] }));
                    }

                    res.status(401).send(trataErro(API_CODES.CODE_TOKEN_NAO_ENCONTRADO, API_MSGS.MSG_TOKEN_NAO_ENCONTRADO));
                }
            }
        }
    };
}