import * as mssql from 'mssql';

import { BenchmarkService } from './../../services/benchmark.service';
import { connection } from '../../services/db.service';
import {
    MODEL_PRIORITY,
    API_CONFIG
} from './../../config';

import {
    resultNotFound,
    resultFound,
    logErroBuscaBD,
    printTempoExecucao
} from '../../utils';

const fonte   = MODEL_PRIORITY['sagres_estadual.empenhos'].fonte;
const rank    = MODEL_PRIORITY['sagres_estadual.empenhos'].rank;
const grupo   = MODEL_PRIORITY['sagres_estadual.empenhos'].grupo;

export let getModalidadesCNPJ_Sagres_Estadual = function (cnpj: string){

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('cnpj', mssql.Char(14), cnpj)
        .query(`
            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                E.dt_Ano as dataAno,
                TPL.tp_Licitacao as tpLicitacao,
                TPL.de_TipoLicitacao as deLicitacao,
                COUNT(*) as qtd

            FROM SAGRES_ESTADUAL.dbo.Empenhos E
                INNER JOIN SAGRES_ESTADUAL.dbo.Tipo_Modalidade_Licitacao TPL ON (E.tp_Licitacao = TPL.tp_Licitacao)
            WHERE cd_Credor=@cnpj
            GROUP BY E.dt_Ano, TPL.tp_Licitacao, TPL.de_TipoLicitacao
            ORDER BY E.dt_Ano, TPL.de_TipoLicitacao
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getModalidadesCNPJ_Sagres_Estadual');

            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar empenhos de ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, 'modalidadesestadual');
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos empenhos de ${cnpj}.`, 'getModalidadesCNPJ_Sagres_Estadual'); });
};

export let getValoresModalidadesCNPJ_Sagres_Estadual = function (cnpj: string){

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('cnpj', mssql.Char(14), cnpj)
        .query(`
            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                E.dt_Ano as dataAno,
                TPL.tp_Licitacao as tpLicitacao,
                TPL.de_TipoLicitacao as deLicitacao,
                SUM(E.vl_Empenho) AS vlEmpenho,
                SUM(P.vl_Pagamento) as vlPagamento

            FROM SAGRES_ESTADUAL.dbo.Empenhos E
                INNER JOIN SAGRES_ESTADUAL.dbo.Tipo_Modalidade_Licitacao TPL ON (E.tp_Licitacao = TPL.tp_Licitacao)
                OUTER APPLY(
                    SELECT SUM(P.vl_Pagamento) AS vl_Pagamento
                    FROM SAGRES_ESTADUAL.dbo.Pagamentos P WHERE (E.cd_Ugestora = P.cd_Ugestora) AND (E.dt_Ano = P.dt_Ano) AND (E.cd_UnidOrcamentaria = P.cd_UnidOrcamentaria) AND (E.nu_Empenho = P.nu_Empenho)
                ) P
            WHERE cd_Credor=@cnpj
            GROUP BY E.dt_Ano, TPL.tp_Licitacao, TPL.de_TipoLicitacao
            ORDER BY E.dt_Ano, TPL.tp_Licitacao, TPL.de_TipoLicitacao
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getValoresModalidadesCNPJ_Sagres_Estadual');

            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar empenhos de ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, 'valoresestadual');
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos empenhos de ${cnpj}.`, 'getValoresModalidadesCNPJ_Sagres_Estadual'); });
};

export let getTotalEmpenhadoPagoCNPJ_Sagres_Estadual = function (cnpj: string){

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('cnpj', mssql.Char(14), cnpj)
        .query(`
            SELECT TOP ${API_CONFIG.SIAP_MAX_RESULTS}
                MIN(E.dt_Ano) as anoPrimeiroEmpenho,
                MAX(E.dt_Ano) as anoUltimoEmpenho,
                SUM(E.vl_Empenho) as totalEmpenho,
                SUM(P.vl_Pagamento) as totalPago

            FROM SAGRES_ESTADUAL.dbo.Empenhos E
                LEFT OUTER JOIN SAGRES_ESTADUAL.dbo.Pagamentos P ON (E.cd_Ugestora = P.cd_Ugestora) AND (E.dt_Ano = P.dt_Ano) AND (E.cd_UnidOrcamentaria = P.cd_UnidOrcamentaria) AND (E.nu_Empenho = P.nu_Empenho)
            WHERE cd_Credor=@cnpj
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTotalEmpenhadoPagoCNPJ_Sagres_Estadual');

            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar os totais de empenho e pagamentos de ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, 'totalempenhoestadual');
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos totais de empenho e pagamentos de ${cnpj}.`, 'getTotalEmpenhadoPagoCNPJ_Sagres_Estadual'); });
};

export let getTopEmpenhosPagosCNPJ_Sagres_Estadual = function (cnpj: string){

    const tempoInicial = new BenchmarkService();
    return connection.getConn().request()
        .input('cnpj', mssql.Char(14), cnpj)
        .query(`
            SELECT TOP 5
                UPPER(UG.de_ugestora) as uGestora,
                P.dt_ano as dataAno,
                P.vl_Pagamento as vlPagamento,
                UPPER(E.de_Historico) as descricao

            FROM SAGRES_ESTADUAL.dbo.Empenhos E
                INNER JOIN SAGRES_ESTADUAL.dbo.PAGAMENTOS P ON (E.cd_Ugestora = P.cd_Ugestora) AND (E.dt_Ano = P.dt_Ano) AND (E.cd_UnidOrcamentaria = P.cd_UnidOrcamentaria) AND (E.nu_Empenho = P.nu_Empenho)
                OUTER APPLY(
                    SELECT TOP 1 de_Ugestora
                    FROM SAGRES_ESTADUAL.dbo.Codigo_UGestora
                    WHERE cd_Ugestora=P.cd_UGestora
                    ORDER BY dt_Ano DESC
                ) UG
            WHERE E.cd_Credor=@cnpj
            ORDER BY P.vl_Pagamento DESC
        `)
        .then(result => {
            printTempoExecucao(tempoInicial, 'getTopEmpenhosPagosCNPJ_Sagres_Estadual');

            if(result.recordset.length === 0)
                return resultNotFound(`Nao foi possivel encontrar os totais de empenho e pagamentos de ${cnpj}.`, fonte);
            else
                return resultFound(result, fonte, rank, 'topempenhoestadual');
        }).catch(error => { return logErroBuscaBD(error, `Falha na busca pelos totais de empenho e pagamentos de ${cnpj}.`, 'getTopEmpenhosPagosCNPJ_Sagres_Estadual'); });
};
